#!/bin/bash

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# --- WIZARD_UIFILE / uninstall_uifile
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Set language Strings ...
# -----------------------------------------------------------------------------------
init_lang() {
/bin/cat << EOF
unTitle             = "Deinstallation von AvrLogger :"
unBoxBackupDesc     = "Sollen die Daten, inklusive der Einstellungen, gesichert werden?"
unBoxBackupText     = "Ja, sollen sie!"
unPathBackupDesc    = "Sicherungsverzeichnis"
unPathBackupText    = "Ordner"

# -----------------------------------------------------------------------------------
upTitle2              = "Wiederherstellung der scemd.xml :"

upTitleDesc2          = "AvrLogger kann die <b>scemd.xml</b>, wie sie bei der Installation vorgefunden und gesichert wurde, wiederherstellen."

upTitleDesc2_1        = "Das wird in <b>jedem Fall</b>, auch wenn die Funktion in AvrLogger <u><i>niemals genutzt</i></u> wurde, die <b>scemd.xml</b> überschreiben. "
upTitleDesc2_1       += "Damit werden auch alle händisch gemachten Änderungen zurückgesetzt."

upBoxCheckDesc2       = "Ja, die scemd.xml wiederherstellen."
upBoxCheckDesc2_1     = "Ok, habe ich zur Kenntnis genommen."
upBoxCheckValidator2  = "Bitte den Haken : <b>Ok, habe ich zur Kenntnis genommen.</b> setzen um fortzufahren!"
EOF
}

# -----------------------------------------------------------------------------------
# --- json WIZARD_UIFILE
# -----------------------------------------------------------------------------------
init_wizard() {
/bin/cat << EOF
[{
   "step_title": "${unTitle}",
   "items": [{
      "type": "multiselect",
      "desc": "${unBoxBackupDesc}",
      "subitems": [{
         "defaultValue": ${SecureFlag},
         "desc":       "${unBoxBackupText}",
         "key":        "wizSaveData"
         }]
   },{
      "type": "textfield",
      "desc": "${unPathBackupDesc}",
      "subitems": [{
         "defaultValue": "${SecurePath}",
         "desc":         "${unPathBackupText}",
         "key":          "wizUserDir",
         "emptyText":    "Data/AvrLogger"
          }]
      }]
},{
   "step_title": "${upTitle2}",
   "items": [{
		"type": "multiselect",
		"desc": "${upTitleDesc2}",
		"subitems": [{
		   "defaultValue": true,
			"desc":         "${upBoxCheckDesc2}",
			"key":          "wizCheckScemd"
			}]
   },{
		"type": "multiselect",
		"desc": "${upTitleDesc2_1}",
      "subitems": [{
		   "defaultValue": false,
			"desc":         "${upBoxCheckDesc2_1}",
			"key":          "wizCheckShmNote",
			"validator":    {"fn": "{var v=arguments[0]; if (!v) return '${upBoxCheckValidator2}'; return true;}"}
			        }]
		    }]
}]
EOF
}

# -----------------------------------------------------------------------------------
# --- Prepare "json WIZARD_UIFILE" ...
# -----------------------------------------------------------------------------------
FindeShare()
{
    local path UserDir=$1

    [ "/" == "${UserDir:0:1}" ] && UserDir="${UserDir:1}"
    [ "/" == "${UserDir: -1}" ] && UserDir="${UserDir:: -1}"
    local tmp="${UserDir,,}"
    [ -z "${tmp%%volume*}" ] && UserDir="${UserDir#*/}"

    for path in $(find "/" -maxdepth 1 -name "volume?"); do
        if [ -d "${path}/${UserDir}" ]
            then path="${path}/${UserDir}"; break
            else unset path
        fi
    done

    [ -v path ] && echo "\"$UserDir\"" "\"$path\""
}
# -----------------------------------------------------------------------------------

if [[ -f "/var/packages/AvrLogger/etc/config/log.conf" ]]; then
    userDir="$(grep -m 1 "SvgArchivePath" "/var/packages/AvrLogger/etc/config/log.conf" | sed 's/^[^"]*"\([^"]*\)".*/\1/g')"
    if [[ -n "$userDir" ]]; then
        eval arr=("$(FindeShare "$userDir")")
        [ -v arr ] && SecurePath="${arr[0]}"
    fi
fi

if [ -v SecurePath ]
    then SecureFlag="true"
    else SecureFlag="false"; SecurePath=""
fi

# -----------------------------------------------------------------------------------
var=""
while IFS= read -r line; do
    if [[ -n "${line%%\#*}" ]]; then
        value="${line#*\"}"
        if [ -z "${line/* \+\= *}" ]
            then var+="${line%% *}+=\"${value%\"*}\"\n"
            else var+="${line%% *}=\"${value%\"*}\"\n"
        fi
    fi
done <<<"$(init_lang)"
source <(echo -e $var)
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
echo -e "$(init_wizard)" > $SYNOPKG_TEMP_LOGFILE
# -----------------------------------------------------------------------------------

exit 0

# ---- END ----
