#!/bin/sh
# restore.sh / 2021.09.12

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# --- Check / change ownership ...
# -----------------------------------------------------------------------------------
f_Check() {

   echo "--------------------------------------------------------------------------------"
   echo "Check whether the system has the new FHS and whether it is affected"
   if [ -v appConf ]; then
      for iDev in ${appConf[@]}; do
         echo "   Check : ${iDev}"
         [[ "${sUser}" == "$( stat -c '%U:%G' "${iDev}" )"  ]] && iAffected=
      done
   fi

   # --------------------------------------------------------------------------------
   # --- System has new FHS (Package Filesystem Hierarchy Standard) but is not affected
   if [ ! -v iAffected ]; then
      echo "Your system is ${Gre}not${NC} affected."
      echo "--------------------------------------------------------------------------------"
      return
   fi

   # --------------------------------------------------------------------------------
   local Time="$(date '+%Y.%m.%d %H:%M:%S')"

   # --------------------------------------------------------------------------------
   if [ -v iChange ]; then
   #  echo "---------------------------------------------------------------------------------"     >> "${ERROR}"
      echo "${Time} Restore   : Start scanning ownership's :"                                      >> "${ERROR}"
   fi

   echo "--------------------------------------------------------------------------------"
   echo "Start scanning ownership's :"

   # --------------------------------------------------------------------------------
   # --- Walk through devices (/volume*) ...
   for sDev in ${appConf[@]}; do
      echo "--------------------------------------------------------------------------------"
      echo "${sDev} :"
      if [ -v iChange ]; then
         echo "---------------------------------------------------------------------------------"  >> "${ERROR}"
         echo "${Time} Restore   : ${sDev} :"                                                      >> "${ERROR}"
      fi
      # -----------------------------------------------------------------------------
      # --- Walk through Basefolder ...
      unset iFirst
      while IFS= read -r sDir; do
         # --------------------------------------------------------------------------
         # --- Skip entry if empty or not a Folder ...
         [[ -z "${sDir}" ]] || [[ ! -d "${sDir}" ]] && continue

         # --------------------------------------------------------------------------
         curOwner="$( stat -c '%U:%G' "${sDir}" )"                   # Current Ownership
         scrOwner="$( stat -c '%U:%G' "${sDir/appconf/appdata}" )"   # Real source Ownership

         # --------------------------------------------------------------------------
         # --- Skip if both Ownerships are identical ...
         [[ "${curOwner}" == "${scrOwner}" ]] && [ ! -v iForce ] && continue

         echo "   -----------------------------------------------------------------------------"
         echo "   Ownership ${Blu}${sDir}${NC} is ${Red}${curOwner}${NC} instead of ${Gre}${scrOwner}${NC}."

         # -----------------------------------------------------------------------------
         # --- Restore Ownership based on /volume*/@appdata ...
         echo "      Changeing ${Blu}${sDir}${NC} to ${Gre}${scrOwner}${NC}."
         if [[ -v iChange ]]; then
            chown "${scrOwner}" "${sDir}"
            echo "${Time} Restore   :    Changed \"${sDir}\" to \"${scrOwner}\""                   >> "${ERROR}"
         fi

         # --------------------------------------------------------------------------
         # --- Skip first Folder (Basefolder) ...
         if [ ! -v iFirst ]; then
            iFirst=
            continue
         fi

         # --------------------------------------------------------------------------
         # --- Walk through SPK-Folder ...
         unset iFile
         find "${sDir}" -print0 | while IFS= read -r -d '' sFile; do
            # -----------------------------------------------------------------------
            # --- Skip first entry (Base SPK-Folder) ...
            if [ ! -v iFile ]; then
               iFile=
               continue
            fi

            # --------------------------------------------------------------------------
            # --- Restore Ownership based on /volume*/@appdata ...
            echo "      Changeing ${Blu}${sFile}${NC} to ${Gre}${scrOwner}${NC}."
            if [[ -v iChange || -v iForce ]]; then
               chown "${scrOwner}" "${sFile}"
               echo "${Time} Restore   :       Changed \"${sFile}\" to \"${scrOwner}\""             >> "${ERROR}"
            fi
         done

      done <<< "$( find  "${sDev}" -maxdepth 1 -name "*" )"
   done

   # --------------------------------------------------------------------------------
   if [ -v iChange ]; then
      echo "---------------------------------------------------------------------------------"     >> "${ERROR}"
      echo "${Time} Restore   : Restoration of Ownership's completed"                              >> "${ERROR}"
      echo "---------------------------------------------------------------------------------"     >> "${ERROR}"
   fi

   echo "--------------------------------------------------------------------------------"
   echo "Restoration of Ownership's completed"
   echo "--------------------------------------------------------------------------------"

}
