#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# --- Global options :
# -----------------------------------------------------------------------------------
Debug     = False

# -----------------------------------------------------------------------------------
import sys
import os

sys.path.append( os.path.dirname(os.path.abspath(sys.argv[0])) + "/lib")
sys.path.append( os.path.dirname(os.path.abspath(sys.argv[0])) + "/gui")

import glob
import json
import threading
import time

import errno
from   traceback  import format_exc as Trace

import asset
import basics  as io
import lib
import Html

import isAdmin

# -----------------------------------------------------------------------------------
pMain     = sys.modules[ __name__ ]
pMain.Avr = dict()

# -----------------------------------------------------------------------------------
# --- Redirect stderr to "ERROR" as early as possible to really log all errors ...
if not sys.stdin.isatty() and not Debug :
   sys.stderr = open( asset.ERROR, 'a' )

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
class BaseException( Exception ) :
   """Base exception for this Module"""
   # --- "Exception" is a Built-in Exceptions

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
class cgiException( BaseException ) :
   """Main Exception handler"""

# -----------------------------------------------------------------------------------
# --- Helper function for getting the current exception in a try/except block
# -----------------------------------------------------------------------------------
def getException() :
   return sys.exc_info()[1]

# -----------------------------------------------------------------------------------
# --- cgi "MAIN" :
# -----------------------------------------------------------------------------------
def Main() :

   # --------------------------------------------------------------------------------
   # --- Check if we have the necessary privileges ...
   if asset.f_Dsm7 :
      if os.path.exists( asset.avrRootText ) :
         import Cgi.changelog as ChangeLog

         Lang = lib.InitLanguage( sName='admin', iOnly=True )
         # --------------------------------------------------------------------------
         # --- Check if a suitable language file is available ...
         ChangeLog.Func( WhatsNew = lib.InitLanguage( 'admin', asset.avrRootInfo, sLang=Lang, suffix='.html', opt=True ) )
         return

   # --------------------------------------------------------------------------------
   # --- Start ''admin-check''  ...
   try :
      if not isAdmin.quick() :
         PID = threading.Thread( target=isAdmin.check, args=( asset.cgiNameChart, 5, ) )
         PID.start()
         iCheck = True
      else :
         iCheck = False
   except :
      raise cgiException( f'"Admin-Check" faild\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Determine the process ID of the daemon ...
   pMain.AvrPid = lib.GetPPidOf( f'{asset.SpkDaemon}.py' )

   # --------------------------------------------------------------------------------
   # --- Get Config ...
   try :
      if not bool( pMain.Avr ) : pMain.Avr = lib.ConfigRead( asset.avrConfigFile )
   except :
      raise cgiException( f'"Reading in the configuration failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Make sure that the admin check is finished before the cgiSynoCheck is queried.
   # --- Wait until "admin-check" finished ...
   if iCheck :
      while PID.is_alive() :
         time.sleep( 0.005 )

   # --------------------------------------------------------------------------------
   # --- If "admin-check" failed ... 
   try :
      if not os.path.exists( asset.cgiSynoCheck ) and not Debug :
         Html.NoPermissions ()
         return
   except :
      raise cgiException( f'"Output of NoPermissions" faild\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Get our cookies ...
   try :
      if not Debug :
         asset.sysCookies = json.loads( open( asset.cgiSynoCheck ).read().strip() )
         sUser = asset.sysCookies['SynoUser']
      else :
         sUser = 'admin'

      asset.msg        = lib.InitLanguage( sUser, asset.avrLangFile )
      isAdmin.schedule()
   except :
      raise cgiException( f'"Read of sysCookies" faild\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- If this was an update, show news ...
   try :
      if os.path.exists( asset.f_NewsAvrPKG ) :
         Html.WhatsNew( sCgi='AvrChart' )
         try    : os.remove( asset.f_NewsAvrPKG )
         except : pass
         return
   except :
      raise cgiException( f'"Call of WhatsNew" faild\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Prepare "status"-content ... LineChart and so on ...
   if os.path.exists( asset.f_AvrDisabled ):
      iID = 0
   else:
      if pMain.AvrPid : iID = 1
      else            : iID = 2

   sIconNice = json.loads( asset.msg['nacCgiStausIcn'] )
   sIcon     = asset.cgiNavIcon[iID]
   sText     = sIconNice[ str(iID) ]

   if not os.path.exists( asset.f_avrStatus ) :
      io.ioWrite( json.dumps( { 'rtfStatusIcon' : sIcon, 'rtfStatusText' : sText }, sort_keys=True, indent=3 ), asset.f_avrStatus )

   # --------------------------------------------------------------------------------
   # --- Init "First" view (load) & Reloader ...
   sLineChart = glob.glob( asset.LineChart + '/tmp_*.svg' )
   if sLineChart : sLineChart = sLineChart[-1].split( '/' )[-1]

   try :
      Out  =  'Content-type: text/html\n'
      Out +=  '\n'
      Out +=  '<!DOCTYPE html>\n'
      Out +=  '<html xmlns="http://www.w3.org/1999/xhtml">\n'
      Out +=  '   <head>\n'
      Out +=  '      <title> -- Avr-Chart -- </title>\n'
      Out +=  '      <meta http-equiv="Content-Type" content="text/html; charset=utf-8" />\n'
      Out +=  '      <meta name="viewport" content="width=device-width, initial-scale=1">\n'
      Out += f'      <link rel="shortcut icon" href="images/AvrChart_32.png" type="image/x-icon" />\n'
      Out += f'      <link rel="stylesheet" type="text/css" href="css/Normalize.css?v={asset.cssVersion}"/>\n'
      Out += f'      <link rel="stylesheet" type="text/css" href="css/Style.css?v={asset.cssVersion}"/>\n'
      Out += f'      <script src="js/AvrLib.js?v={asset.cssVersion}"></script>\n'
      Out += f'      <script src="js/jQuery.js?v={asset.cssVersion}"></script>\n'
      Out +=  '   </head>\n'
      Out +=  '   <body class="Chart">\n'
      Out +=  '      <div class="callConfig">\n'
      Out += f'         <img id="callConfig" class="callConfig" src="images/{sIcon}" title="{sText}"/>\n'
      Out += f'         <img id="LineChart" class="SVG" src="images/{sLineChart}"/>\n'
      Out +=  '         <script>setInterval(function(){ requestAjax( "svgLineChart" ); }, %s);</script>\n' % pMain.Avr['PollingFrequency']
      Out +=  '      </div>\n'
      Out +=  '   </body>\n'
      Out +=  '</html>'

      print ( Out )

   except :
      raise cgiException( f'"Output of LineChart" faild\n\n{Trace()}' )

# -----------------------------------------------------------------------------------
# --- Entry point :
# -----------------------------------------------------------------------------------
if __name__ == '__main__' :

   # --------------------------------------------------------------------------------
   # --- Call "main"-Function ...
   try : Main()

   except BaseException :
      # -----------------------------------------------------------------------------
      # --- Catch "cgi BaseException" and print more Information to LOG-File ...
      lib.Error( f'BaseException in : {getException()}' )

   except Exception :
      # -----------------------------------------------------------------------------
      # --- Catch all "Untreated Exception's" and print more Information to LOG-File ...
      lib.Error( f'Exception :\n{Trace()}' )

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
