#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# --- Global options :
# -----------------------------------------------------------------------------------
Debug     = False

# -----------------------------------------------------------------------------------
import time
nTime = time.time()

import sys
import os

sys.path.append( os.path.dirname( os.path.abspath( sys.argv[ 0 ] ) ) + "/lib" )
sys.path.append( os.path.dirname( os.path.abspath( sys.argv[ 0 ] ) ) + "/gui" )

import asset
import basics as io

# -----------------------------------------------------------------------------------
# --- Redirect stderr to "ERROR" as early as possible to really log all errors ...
if not sys.stdin.isatty() :
   if Debug : sys.stderr = open( asset.ERROR, 'a' )
   else     : sys.stderr = io.ErrorLog( sys.stderr, asset.ERROR )
# -----------------------------------------------------------------------------------

from   urllib.parse import unquote_plus as unquote
from   urllib.parse import quote
import threading
import json
import errno
from   traceback    import format_exc as Trace
import lib

from   Html   import *
import isAdmin

# -----------------------------------------------------------------------------------
pMain     = sys.modules[ __name__ ]
pMain.Avr = dict()

# -----------------------------------------------------------------------------------
# Print ( 'import : %.4f' % ( time.time() - nTime) )

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
class BaseException( Exception ) :
   """Base exception for AvrLogger.cgi"""
   # --- "Exception" is a Built-in Exceptions

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
class cgiException( BaseException ) :
   """Main Exception handler"""

# -----------------------------------------------------------------------------------
# --- Helper function for getting the current exception in a try/except block
# -----------------------------------------------------------------------------------
def getException() :
   return sys.exc_info()[1]

# -----------------------------------------------------------------------------------
# --- cgi "MAIN" :
# -----------------------------------------------------------------------------------
def Main() :

   # --------------------------------------------------------------------------------
   # --- Check if we have the necessary privileges ...
   if asset.f_Dsm7 :
      if os.path.exists( asset.avrRootText ) :
         import Cgi.changelog as ChangeLog

         Lang = lib.InitLanguage( sName='admin', iOnly=True )
         # --------------------------------------------------------------------------
         # --- Check if a suitable language file is available ...
         ChangeLog.Func( WhatsNew = lib.InitLanguage( 'admin', asset.avrRootInfo, sLang=Lang, suffix='.html', opt=True ) )

         return

   # --------------------------------------------------------------------------------
   # --- Start ''admin-check''  ...
   try :
      if not isAdmin.quick() :
         PID = threading.Thread( target=isAdmin.check, args=( asset.cgiNameConfig, 5, ) )

         PID.start()
         iCheck = True
      else :
         iCheck = False
   except :
      raise cgiException( f'"Admin-Check" faild\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Determine the process ID of the daemon ...
   try    :
      pMain.AvrPid = lib.GetPPidOf( asset.SpkDaemon + '.py' )
   except :
      raise cgiException( f'"Get AvrPid" faild\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Get AvrLogger-Config ...
   try :
      if not bool( pMain.Avr ) : pMain.Avr = lib.ConfigRead( asset.avrConfigFile )
      pMain.iTimeOut = int( pMain.Avr[ 'sysCpuScore' ] )
   except :
      raise cgiException( f'"Reading in the configuration failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Set some default Values ...
   mainpage = ''

   page     = 'Config'
   more     = ''
   site     = ''

   # --------------------------------------------------------------------------------
   # --- Query "GET" / "POST" from input Stream ( "QUERY_STRING" ) ...
   # --------------------------------------------------------------------------------
   try :
      sQS     = False
      sQsGet  = False
      sQsPost = False

      Env     = os.environ
      method  = Env.get( 'REQUEST_METHOD', False ).upper()

      # -----------------------------------------------------------------------------
      if method :
         if method == 'GET' :
            sQsGet  = unquote( Env.get( 'QUERY_STRING',   False ) )
         elif method == 'POST' :
            iLength =     int( Env.get( 'CONTENT_LENGTH', False ) )
            sQS     = unquote( Env.get( 'QUERY_STRING',   False ) )
            if iLength :
               sFP     = sys.stdin.buffer
               sQsPost = unquote( sFP.read( iLength ).decode( 'utf-8' ) )

      # -----------------------------------------------------------------------------
      if sQsGet :
         sQS = sQsGet

      if sQsPost :
         if sQS : sQS += f'&{sQsPost}'
         else   : sQS  =     sQsPost

      # -----------------------------------------------------------------------------
   except :
      raise cgiException( f'"Query of QUERY_STRING" failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Parse "form data" by self ...
   # --------------------------------------------------------------------------------
   try :
      if sQS : InCgi = sQS.split( '&' )
      else   : InCgi = []

      Switch = False
      # -----------------------------------------------------------------------------
      for i, Item in enumerate( InCgi ) :
         if Switch or not Item :
            Switch = False
            continue

         # --------------------------------------------------------------------------
         Key,Value = Item.split( '=', 1 )

         # --------------------------------------------------------------------------
         if Key == 'page':
            page = Value
            continue

         # --------------------------------------------------------------------------
         if Value == 'Switch' :
            try :
               if Key in InCgi[ i + 1 ] :
                  Value  = 'Yes'
                  Switch = True
               else:
                  Value  = 'No'
            except:
               Value = 'No'

         # --------------------------------------------------------------------------
         asset.cgiInput[Key] = Value

   except :
      raise cgiException( f'"Parse of FieldStorage" failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Examine key "page"-content ...
   if page :
      if '-' in page : mainpage,site = page.split('-', 1)
      else           : mainpage      = page

      if '+' in site : site,more     = site.split('+', 1)

   pMain.page = page
   pMain.site = site
   pMain.more = more

   # --------------------------------------------------------------------------------
   # --- Make sure that the admin check is finished before the cgiSynoCheck is queried.
   # --- Wait until "admin-check" finished ...
   if iCheck :
      while PID.is_alive() :
         time.sleep( 0.005 )

   # --------------------------------------------------------------------------------
   # --- If "admin-check" failed ... 
   try :
      if not os.path.exists( asset.cgiSynoCheck ) :
         NoPermissions ()
         return
   except :
      raise cgiException( f'"Output of NoPermissions" failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Get our cookies ...
   try :
      pMain.curLang    = lib.InitLanguage( iOnly=True )
      asset.sysCookies = json.loads( io.ioRead( asset.cgiSynoCheck ) )
      asset.msg        = lib.InitLanguage( sLang=pMain.curLang, sPath=asset.avrLangFile )
   except :
      raise cgiException( f'"Read of sysCookies" failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Check Schedule-Status ...
   try :
      ChangeOrder = isAdmin.schedule()
   except :
      raise cgiException( f'"Check of Schedule-Status" failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- If this was an update, show news ...
   try :
      if io.check( asset.f_NewsAvrPKG ) :
         WhatsNew()
         io.ioRm ( asset.f_NewsAvrPKG )
         return
   except :
      raise cgiException( f'"Call of WhatsNew" failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # --- Prepare "HTML-Header" but skip if the module does the page building itself
   if mainpage not in [ 'changelog', 'groot', 'debug' ] :
      try :
         OpenPage ( sTitel= asset.SpkName + '@' + pMain.Avr[ 'sysHostName' ], sMethod='POST' )
      except :
         raise cgiException( f'"Call of OpenPage" failed\n\n{Trace()}' )

   # --------------------------------------------------------------------------------
   # Dispatcher for called "mainpage" functions ...
   while lib.switch( mainpage ) :
      # -----------------------------------------------------------------------------
      # --- "status" is handled later ...
      if lib.case( 'status' ) :
         break

      if lib.case( 'debug' ) :
         NoPermissions ()
         return

      if 'main' not in sys.modules : import main

      try :
         sNext = main.main ( mainpage, site, more )
      except Exception :
         raise cgiException( f'"main.main"\n\tCalled Page : [{mainpage}-{site}+{more}]\n\n{Trace()}' )

      if sNext :
         try :
            print ( f'<meta http-equiv="refresh" content="0; url=AvrConfig.cgi?page={quote(sNext)}">', flush=True )
         except :
            raise cgiException( f'"Meta refresh" failed\n{Trace()}' )

      return

   # --------------------------------------------------------------------------------
   # --- If Daemon is busy : Display a message ...
   if pMain.AvrPid and ( os.path.exists( asset.f_CgiUpdate ) or ChangeOrder ) :
      jBusy ( 'msgCgiWaitBusy' )
      loader = True
   else :
      loader = False

   # --------------------------------------------------------------------------------
   # --- Prepare "status"-content ... LineChart and so on ...
   if os.path.exists( asset.f_AvrDisabled ):
      iID = 0
   else:
      if pMain.AvrPid : iID = 1
      else            : iID = 2

   sIconNice = json.loads( asset.msg['nacCgiStausIcn'] )
   sIcon     = asset.cgiNavIcon[iID]
   sText     = sIconNice[ str(iID) ]

   if not os.path.exists( asset.f_avrStatus ) :
      io.ioWrite( json.dumps( { 'rtfStatusIcon' : sIcon, 'rtfStatusText' : sText }, sort_keys=True, indent=3 ), asset.f_avrStatus )

   # --------------------------------------------------------------------------------
   # --- Return if Daemon is not running ...
   if not loader : return

   # --------------------------------------------------------------------------------
   # --- Wait until a config update has been performed or until timeout ...
   TimeOut = int(time.time() ) + pMain.iTimeOut
   io.ioRm( asset.f_Heartbeat )

   try :
      while io.check( asset.f_CgiUpdate ) :
         # --- If the daemon is running amok or has crashed, then we'll try a reboot ...
         if int( time.time() ) >= TimeOut :
            Print( asset.msg['errCgiUpdate'] )
            Print( asset.msg['errCgiReset']  )
            # -----------------------------------------------------------------------
            os.system( ' '.join(( asset.script, 'cgi', 'stop' )) )
            # -----------------------------------------------------------------------
            if io.check(  asset.avrConfigFile + '.save' ) :
               io.ioCopy( asset.avrConfigFile + '.save', asset.avrConfigFile )
            # -----------------------------------------------------------------------
            os.system( ' '.join(( asset.script, 'cgi', 'start' )) )
            # -----------------------------------------------------------------------
            break

         time.sleep( 0.005 )

         if os.path.exists( asset.f_Heartbeat ) :
            TimeOut = int( time.time() ) + pMain.iTimeOut
            io.ioRm( asset.f_Heartbeat )
   except :
      raise cgiException( f'CGI: The update of the configuration seems to have failed\n{getException()}' )

   # --------------------------------------------------------------------------------
   # --- Delete a "Busy" message ...
   if loader :
      try :
         print (f'<meta http-equiv="refresh" content="0; url=AvrConfig.cgi?page=status">', flush=True )
      except :
         raise cgiException( f'"Busy refresh" failed\n\n{Trace()}' )

# -----------------------------------------------------------------------------------
# --- Entry point :
# -----------------------------------------------------------------------------------
if __name__ == '__main__' :

   # --------------------------------------------------------------------------------
   # --- Call "main"-Function ...
   try :
      Main()
   #  Print ( 'Main Done : %.4f' % ( time.time() - nTime ) )

   except ( BrokenPipeError, IOError ) :
      # -----------------------------------------------------------------------------
      # --- Catch "IOError" for later igrnoring ...
      pass

   except ( BaseException ) :
      # -----------------------------------------------------------------------------
      # --- Catch "cgi BaseException" and print more Information to LOG-File ...
      Print( f'BaseException in : {getException()}' )

   except Exception :
      # -----------------------------------------------------------------------------
      # --- Catch all "Untreated Exception's" and print more Information to LOG-File ...
      Print( f'Exception :\n{Trace()}' )

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
