# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import asset
from   Html  import *

from   lib   import switch, case

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
import __main__  as pMain

FuncName = 'ookla'

FuncDesc = { 'IconName'     : 'Ookla.png',
             'SubName'      : asset.msg['modSourcesOoklaName'],
             'SubText'      : asset.msg['modSourcesOoklaText'],
             'iWidth'       : SubMenuWidth,
             'iOnChange'    : f'{asset.PageName}-{FuncName}+OnChange',
           }

Intervall   = "10min 15min 30min 1.0h 2.0h 3.0h 6.0h 12.0h 24.0h"
ListSize    = '10 15 20 25 30 50 100 200'
iOokla      = [ 'SpeedTestServerTemp', ]

# -----------------------------------------------------------------------------------
# --- Pre-Initialization
# -----------------------------------------------------------------------------------
def pre( site, more, sub=False ) :

   while switch( f'{site}+{more}' ) :
      # -----------------------------------------------------------------------------
      if case( 'ookla+save' ) :
         # --------------------------------------------------------------------------
         import main
         main.save()
         break

      # -----------------------------------------------------------------------------
      break

# -----------------------------------------------------------------------------------
# --- Post-Initialization
# -----------------------------------------------------------------------------------
def post( site, more, sub=False ) :

   while switch( f'{site}+{more}' ) :
      # -----------------------------------------------------------------------------
      if case( 'ookla+OnChange' ):
         # --------------------------------------------------------------------------
         # --- Prepare Values ...
         for key in asset.cgiInput :
            asset.Out[key] = asset.cgiInput[key]

         asset.cgiInput['SpeedTestServerTemp'] = asset.cgiInput[ 'SpeedTestServerCUR' ]
         SaveLast( asset.PageName, iOokla )
         return
         break

      # -----------------------------------------------------------------------------
      if case( 'ookla+update' ) :
         # --------------------------------------------------------------------------
         # --- Update Server-List & "speedtest-cli" ...
         jBusy( 'msgCgiWaitServerList', iFlush=True )

         import Ookla
         if not asset.AvrConf : asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )

         if Ookla.UpdateServerList( sListSize=int(asset.cgiInput["SpeedTestServerMax"]), sSearchTerm=asset.cgiInput["SpeedTestSearchTerm"] ) :
            asset.cgiInput['SpeedTestServerCUR' ] = asset.AvrConf['SpeedTestServerCUR' ]
            asset.cgiInput['SpeedTestServerGUI' ] = asset.AvrConf['SpeedTestServerGUI' ]
            asset.cgiInput['SpeedTestServerRAW' ] = asset.AvrConf['SpeedTestServerRAW' ]
            asset.cgiInput['SpeedTestRegistered'] = asset.AvrConf['SpeedTestRegistered']
            asset.cgiInput['SpeedTestAccessible'] = asset.AvrConf['SpeedTestAccessible']

            if asset.cgiInput['SpeedTestServerCUR'] not in asset.cgiInput['SpeedTestServerRAW'] :
               asset.cgiInput['SpeedTestServerCUR'] = asset.cgiInput['SpeedTestServerRAW'].split( ' ' )[0]

            asset.cgiInput['SpeedTestServerTemp']  = asset.cgiInput['SpeedTestServerCUR']
            
            import main
            main.save()
         else :
            # --- If Update fails... disable SpeedTest ...
            asset.cgiInput['SpeedTestAccessible'] = ''
            asset.cgiInput['SpeedTestRegistered'] = ''
            asset.cgiInput['SpeedTestOn'        ] = 'No'
            asset.cgiInput['SpeedTestServerCUR' ] = ''
            asset.cgiInput['SpeedTestServerGUI' ] = ''
            asset.cgiInput['SpeedTestServerRAW' ] = ''
            asset.cgiInput['SpeedTestFailed'    ] = 'Yes'

            import main
            main.save()
            return "Info-log"

         break

      # -----------------------------------------------------------------------------
      break

   # --------------------------------------------------------------------------------
   # --- Prepare Values ...
   for sKey in asset.cgiInput :
      asset.Out[ sKey] = asset.cgiInput[ sKey ]

# -----------------------------------------------------------------------------------
# --- Store / Save a List of Value's
# -----------------------------------------------------------------------------------
def SaveLast ( PageName, iList ) :

   asset.Last     = { **asset.cgiInput }
   asset.cgiInput = dict()

   for sID in iList :
      asset.cgiInput[sID] = asset.Last[sID]

   import main
   main.save()

   asset.cgiInput = { **asset.Last }

# -----------------------------------------------------------------------------------
# --- Function : "engine"
# -----------------------------------------------------------------------------------
def Func ( sSite, sMore ) :

   iWidth   = iMainOutWidth
   iHeight  = iMainHeight

   s           = asset.msg
   Out         = asset.Out

   # --------------------------------------------------------------------------------
   ContentOpen()
   # --------------------------------------------------------------------------------
   ContentRow ()
   ContentTab ( iWidth, 300 )
   # --------------------------------------------------------------------------------

   if Out['SpeedTestRegistered']    : cgiSpeed = True
   else                             : cgiSpeed = False

   if Out['PopUpStickyOn'] != 'Yes' : sSticky = True
   else                             : sSticky = False


   if not cgiSpeed :
      # -----------------------------------------------------------------------------
      # --- Limit menu to init server list
      Section ( 'SpeedList' )

      RowOpen  ()
      Select   ( 'SpeedTestServerMax',  'SpeedTestServerMax', ListSize, Column_1,  65 )
      RowClose ()

      RowOpen  ()
      Input    ( 'SpeedTestSearchTerm', 'SpeedTestSearchTerm',          Column_1,  132 )
      RowClose ()

      RowOpen  ()
      Label    ( 'SpeedTestUpdate', Column_1 )
      Button   ( 'ButtonListeNow',      '- Init -', f'{asset.PageName}-{FuncName}+update', iSize=80 )
      RowClose ()

   else :

      Section ( 'SpeedList' )

      RowOpen  ()
      Select   ( 'SpeedTestServerMax',  'SpeedTestServerMax', ListSize, Column_1,  65 )
      RowClose ()

      RowOpen  ()
      Input    ( 'SpeedTestSearchTerm', 'SpeedTestSearchTerm',          Column_1,  132 )
      RowClose ()

      RowOpen  ()
      Label    ( 'Init Serverlist :', Column_1 )
      Button   ( 'ButtonListeNow',      '- Update -', f'{asset.PageName}-{FuncName}+update', iSize=80 )
      RowClose ()

      Section ( 'SpeedTest' )

      RowOpen  ()
      CheckBox ( 'SpeedTestOn',         'Speedtest :',        Column_1 )
      RowClose ()

      RowOpen  ()
      CheckBox ( 'SpeedTestHttpsOn',    'SpeedTestHttpsOn',   Column_1 )
      RowClose ()

      RowOpen  ()
      Select   ( 'SpeedTestMaxPing',    'Ping Limit:',   '50 60 70 80 90 100 125 150 200 250 300', Column_1, 70 )
      RowClose ()

      RowOpen  ()
      Select   ( 'SpeedTestFrequency',  'SpeedTestFrequency', Intervall,              Column_1, 70 )
      RowClose ()

      RowOpen   ()
      SelectGUI ( 'SpeedTestServerCUR',  'Test-Server :', Out['SpeedTestServerGUI'], Out['SpeedTestServerRAW'], Column_1, 540, True )
      RowClose  ()

      if Out['SpeedTestOn'] != "Yes" or not pMain.AvrPid :
         RowOpen   ()
         Label     ( s['modRunOoklaButton'], Column_1 )
         popButton ( 'ButtonViewJson', '- Test -', 'ookla', iWidth=830, iHight=485, iSize=80, iClose=sSticky, sCgi='AvrConfig' )
         RowClose  ()

      if io.check( asset.SpeedJsonFile ) :
         RowOpen   ()
         Label     ( s['modViewOoklaButton'], Column_1 )
         popButton ( 'ButtonViewJson', '- View -', 'json',  iWidth=830, iHight=485, iSize= 80, iClose=sSticky, sCgi='AvrConfig' )
         RowClose  ()

   ContentTabEnd()
   ContentRowEnd()

   # --------------------------------------------------------------------------------
   # --- Display "Save" & "Undo" Button ...
   SaveUndo( asset.PageName, FuncName, iSize=80, iWidth=iWidth )
   # --------------------------------------------------------------------------------
   ContentClose()

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
