# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import asset
from   Html  import *

from   lib   import switch, case

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
import __main__  as pMain

FuncName = 'shedule'

FuncDesc = { 'IconName'     : 'shedule.png',
             'SubName'      : asset.msg['modConfigSheduleName'],
             'SubText'      : asset.msg['modConfigSheduleText'],
             'iWidth'       : SubMenuWidth,
             'iOnChange'    : f'{asset.PageName}-{FuncName}+OnChange',
           }

GuiTime_H   = "0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23"
GuiTime_M   = "0 5 10 15 20 25 30 35 40 45 50 55"

# -----------------------------------------------------------------------------------
# --- Pre-Initialization
# -----------------------------------------------------------------------------------
def pre( site, more, sub=False ) :
   pass

# -----------------------------------------------------------------------------------
# --- Post-Initialization
# -----------------------------------------------------------------------------------
def post( site, more, sub=False ) :

   while switch( f'{site}+{more}' ) :
      # -----------------------------------------------------------------------------
      if case( 'shedule+OnChange', 'shedule+save' ):
         # --------------------------------------------------------------------------
         for sID in [ '1', '2', '3' ] :
            asset.cgiInput[ f'ScheduleSet{sID}Start' ] = asset.cgiInput[ f'TimeStart{sID}H' ] + ':' + asset.cgiInput[ f'TimeStart{sID}M' ]
            asset.cgiInput[ f'ScheduleSet{sID}Stop'  ] = asset.cgiInput[ f'TimeStop{sID}H'  ] + ':' + asset.cgiInput[ f'TimeStop{sID}M'  ]

         iMask = []
         for sDay in [ '1', '2', '3', '4', '5', '6', '7' ] :
            if asset.cgiInput[ f'ScheduleMask_{sDay}' ] == 'Yes' :
               iMask.append( '1' )
            else :
               iMask.append( '0' )

         asset.cgiInput[ 'ScheduleDayMask' ] = ''.join( iMask )

         # --------------------------------------------------------------------------
         if more == 'save' :
            import main
            main.save()

         # --------------------------------------------------------------------------
         break

      # -----------------------------------------------------------------------------
      break

   # --------------------------------------------------------------------------------
   # --- Prepare Values ...
   for sKey in asset.cgiInput :
      asset.Out[ sKey] = asset.cgiInput[ sKey ]

# -----------------------------------------------------------------------------------
# --- Function : "engine"
# -----------------------------------------------------------------------------------
def Func ( sSite, sMore ) :

   iWidth   = iMainOutWidth
   iHeight  = iMainHeight

   s           = asset.msg
   Out         = asset.Out

   # --------------------------------------------------------------------------------
   ContentOpen()
   # --------------------------------------------------------------------------------
   ContentRow ()
   ContentTab ( iWidth, 350 )
   # --------------------------------------------------------------------------------
   Section ( 'ScheduleSet' )

   for sID in [ '1', '2', '3' ] :

      sHour, sMin = Out[ f'ScheduleSet{sID}Start' ].split( ':', 1 )
      asset.Out[ f'TimeStart{sID}H' ] = sHour
      asset.Out[ f'TimeStart{sID}M' ] = sMin

      sHour, sMin = Out[ f'ScheduleSet{sID}Stop' ].split( ':', 1 )
      asset.Out[ f'TimeStop{sID}H' ] = sHour
      asset.Out[ f'TimeStop{sID}M' ] = sMin

      sLabel = s[ 'dec_ScheduleStartH' ] + f' {sID}:'

      RowOpen  ()
      Select   ( f'TimeStart{sID}H',   sLabel,           GuiTime_H, 95, 50, opt=True )
      Select   ( f'TimeStart{sID}M',  ':',               GuiTime_M,  0, 50, opt=True )
      Label    ( sWhiteSpace, 7 )
      Select   ( f'TimeStop{sID}H',   'ScheduleStopH',   GuiTime_H, 32, 50, opt=True )
      Select   ( f'TimeStop{sID}M',   ':',               GuiTime_M,  0, 50, opt=True )
      TimeBox  ( 20, 200, Out[ f'ScheduleSet{sID}Start' ], Out[ f'ScheduleSet{sID}Stop' ] )
      RowClose ()

   # --------------------------------------------------------------------------------
   Section ( 'Scheduling' )

   CronSchDay = [ x == '1' for x in list( Out[ 'ScheduleDayMask' ] ) ]
   WeekDays   = s[ 'ScheduleWeekDays' ].split( ' ' )

   for iID, sDay in enumerate( [ '1', '2', '3', '4', '5', '6', '7' ] ) :

      if CronSchDay[ iID ] : asset.Out[ f'ScheduleMask_{sDay}' ] = "Yes"
      else                 : asset.Out[ f'ScheduleMask_{sDay}' ] = "No"

      if CronSchDay[ iID ] :
         while switch( Out[ f'Schedule_{sDay}_Mode' ] ) :
            # -----------------------------------------------------------------------
            if case( 'Yes'  ) :
               sStart, sStopp = '00:00', '24:00'
               break
            # -----------------------------------------------------------------------
            if case( 'Set1' ) :
               sStart, sStopp = Out[ 'ScheduleSet1Start' ], Out[ 'ScheduleSet1Stop' ]
               break
            # -----------------------------------------------------------------------
            if case( 'Set2' ) :
               sStart, sStopp = Out[ 'ScheduleSet2Start' ], Out[ 'ScheduleSet2Stop' ]
               break
            # -----------------------------------------------------------------------
            if case( 'Set3' ) :
               sStart, sStopp = Out[ 'ScheduleSet3Start' ], Out[ 'ScheduleSet3Stop' ]
               break
            # ----- default ---------------------------------------------------------
            asset.Out[ f'Schedule_{sDay}_Mode' ] = 'Yes'
            sStart, sStopp = '00:00', '24:00'
            break
      # -----------------------------------------------------------------------------
      else :
         sStart, sStopp = '00:00', '00:00'

      # -----------------------------------------------------------------------------
      sStartH, sStartM = sStart.split( ':', 1 )
      sStart = f'{int(sStartH):02d}:{int(sStartM):02d}'

      sStoppH, sStoppM = sStopp.split( ':', 1 )
      sStopp = f'{int(sStoppH):02d}:{int(sStoppM):02d}'

      asset.Out[ 'RoTimeBox' ] = f'{sStart} - {sStopp}'

      RowOpen  ()
      CheckBox ( f'ScheduleMask_{sDay}', WeekDays[ iID ],  95, opt=True )

      if CronSchDay[ iID ] :
         Label    ( sWhiteSpace, 10 )
         SelectGUI( f'Schedule_{sDay}_Mode',   'Mode :',    s['ScheduleModeGUI'], s['ScheduleModeRAW'], 40, 70, opt=True )
      else :
         Label    ( sWhiteSpace, 138 )
         Hide     ( f'Schedule_{sDay}_Mode' )

      Label    ( sWhiteSpace, 10 )
      RoBox    ( 'RoTimeBox',    '', 0, 105 )
      TimeBox  ( 20, 200, sStart, sStopp )
      RowClose ()

   # --------------------------------------------------------------------------------
   ContentTabEnd()
   ContentRowEnd()

   # --------------------------------------------------------------------------------
   # --- Display "Save" & "Undo" Button ...
   SaveUndo( asset.PageName, FuncName, iSize=80, iWidth=iWidth )
   # --------------------------------------------------------------------------------
   ContentClose()

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
