# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
from   importlib import import_module as Package

import asset
import basics as io

from   Html   import *

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
import __main__ as pMain

iModPlugIns = [ 'Config', 'Chart', 'Hdd', 'Usb', 'Info', ]
iCgiPlugIns = [ 'log', 'changelog', 'ookla', 'json', 'folder', 'groot', 'view' ]

if pMain.Avr[ 'sysVirtualDSM' ] == 'Yes' : iModPlugIns.remove( 'Hdd' )

# -----------------------------------------------------------------------------------
# --- Dispatcher for called functions :
# -----------------------------------------------------------------------------------
def main( mainpage, site, more ):

   # --------------------------------------------------------------------------------
   # --- No App Privileg's ... no function ... return to Status-Page
   if not asset.sysCookies['appPrivileg'] :
      return

   asset.gui()

   # --------------------------------------------------------------------------------
   # --- Module Dispatcher :
   if mainpage not in iModPlugIns + iCgiPlugIns :
      Print ( f'main-Dispatcher called with unknown argument: [{mainpage}]' )
      return 'Info-log'

   if mainpage in iModPlugIns :
      sMain             = Package( f'mod_{mainpage}' )
      asset.PageName    = sMain.PageName
      asset.FuncSubList = sMain.FuncSubList
      asset.FuncName    = f'c{asset.PageName}.{site}'
      asset.Func        = False
   else :
      sMain             = Package( f'Cgi.{mainpage}' )
      asset.PageName    = sMain.FuncName
      asset.FuncSubList = [ sMain.FuncName ]
      asset.FuncName    = f'Cgi.{asset.PageName}'
      asset.Func        = False

   # --------------------------------------------------------------------------------
   if site and more :

      if site not in asset.FuncSubList :
         Print ( f'"Pre"-Dispatcher called with unknown argument: [{mainpage}-{site}+{more}]' )
         return 'Info-log'

      asset.Func = Package( asset.FuncName )

   # --------------------------------------------------------------------------------
   # --- Some Pre-Initialization's ...
   if site and more :

      sNext = asset.Func.pre( site, more )
      if sNext : return sNext

   # --------------------------------------------------------------------------------
   # --- Get the Default "Config"
   asset.Out = dict( pMain.Avr )

   # --------------------------------------------------------------------------------
   # --- Some Post-Initialization's ...
   if site and more :

      sNext = asset.Func.post( site, more )
      if sNext : return sNext

   # --------------------------------------------------------------------------------
   # --- Call a "cgi"-Function ...
   if mainpage in iCgiPlugIns :
      if not asset.Func : asset.Func = Package( asset.FuncName )

      sNext = asset.Func.Func( site, more )
      if sNext : return sNext
      else     : return

   # --------------------------------------------------------------------------------
   # --- Import PlugIns :
   # --------------------------------------------------------------------------------
   asset.desModule = dict()
   ModDesc   = f'{asset.cgiModulPath}/main.json'

   # --------------------------------------------------------------------------------
   if os.path.exists( ModDesc ) :
      with open( ModDesc, 'r', encoding='utf-8' ) as f:
         asset.desModule = json.loads( f.read() )

      # -----------------------------------------------------------------------------
      if pMain.curLang != asset.desModule.get( 'curLang', '') :
         asset.desModule = dict()

   # --------------------------------------------------------------------------------
   if not asset.desModule :
      asset.desModule[ 'curLang' ] = pMain.curLang

      for i, sName in enumerate( iModPlugIns ) :
         Module                   = Package( f'mod_{sName}' )
         asset.PageName           = Module.PageName
         asset.desModule[ sName ] = Module.pluginDesc()

      io.ioWrite( json.dumps( asset.desModule, sort_keys=True, indent=4 ), ModDesc )

   # --------------------------------------------------------------------------------
   # --- Init OnChange ...
   try :
      sOnChange = asset.desModule[ mainpage ][ 'Subs' ][ site ][ 'iOnChange' ]
      if sOnChange : OnChange ( opt=sOnChange )
   except :
      pass

   # --------------------------------------------------------------------------------
   # --- Main-Header ...
   MainConfigOpen( 'InfoMain' )

   # --------------------------------------------------------------------------------
   # --- Display Icon-Menue  on left side ...

   for sID in iModPlugIns :
      sIcon    = asset.desModule[ sID ][ 'IconName'    ]
      sPage    = asset.desModule[ sID ][ 'PageName'    ]
      sText    = asset.desModule[ sID ][ 'Description' ]

      if sPage == mainpage : iActive = True
      else                 : iActive = False
      menuIcon ( sIcon, sText, sPage, opt=iActive )

   # --------------------------------------------------------------------------------
   # --- Display Sub-Menu :
   if mainpage : index  = iModPlugIns.index( mainpage )
   else        : index  = None

   SubConfigOpen( 'Option :' )

   if index != None :
      sMain.sub( site, more )
      SubConfigClose()

   # ------------------------------------------------------------------------------
   # --- Close Icon-Menue "fieldset" ...
   # ------------------------------------------------------------------------------
   # --- Call Plugin-Sub-Function if selected ....
   if index != None and site :
      if not asset.Func : asset.Func = Package( f'c{asset.PageName}.{site}' )
      asset.Func.Func( site, more )

   # ------------------------------------------------------------------------------
   # --- Close Main-Footer "fieldset" ...
   ClosePage()

# -----------------------------------------------------------------------------------
# --- Save settings ...
# -----------------------------------------------------------------------------------
def save() :

   import time

   # --------------------------------------------------------------------------------
   # --- Read "config" and include cgiInput-Values "AND" keep the original order ...
   # --------------------------------------------------------------------------------
   newConfig = []

   with open( asset.avrConfigFile, 'r', encoding='utf-8' ) as f:
      for sLine in f:
         if sLine :
            sKey, sValue = sLine.strip().split( '=', 1 )
            sValue = sValue[1:-1]

            if sKey in asset.cgiInput:
               sValue = asset.cgiInput[ sKey ]

            newConfig.append( f'{sKey}="{sValue}"' )
            pMain.Avr[ sKey ] = sValue

   # --------------------------------------------------------------------------------
   newConfig[-1] += '\n'
   newConfig      = '\n'.join( newConfig )

   # --------------------------------------------------------------------------------
   # --- If Daemon is running, wait for Idel or Timeout ...
   # --------------------------------------------------------------------------------
   if pMain.AvrPid :
      io.ioRm( asset.f_Heartbeat )
      loader = False

      TimeOut = int( time.time() ) + pMain.iTimeOut

      while os.path.exists( asset.f_AvrLock ) or os.path.exists( asset.f_SenLock ) or os.path.exists( asset.f_SvgLock ) :
         if not loader :
            loader = True
            jBusy( 'msgCgiWaitIdle' )

         if int( time.time() ) >= TimeOut :
            Print( asset.LngConf['errCgiIdleTime'] )
            return

         time.sleep(0.001)
         if os.path.exists( asset.f_Heartbeat ) :
            TimeOut = int( time.time() ) + pMain.iTimeOut
            io.ioRm( asset.f_Heartbeat )

   # --------------------------------------------------------------------------------
   # -- Write back updatete config ...
   io.ioWrite( newConfig, asset.avrConfigFile )

   # --------------------------------------------------------------------------------
   # --- Get Daemon Status (PID) and Test if the daemon runs more than once ...
   if len( pMain.AvrPid ) > 1 :
      Print( asset.LngConf['errCgiFatal'] )
      return

   if pMain.AvrPid :
      # -----------------------------------------------------------------------------
      # --- If Daemon is running, send "Config(s)" has been changed ...
      io.touch( asset.f_CgiUpdate )
      os.kill ( pMain.AvrPid[ 0 ], 10 )
   else :
      # -----------------------------------------------------------------------------
      # --- If Daemon is not running, init CronJob(s) ...
      import Ookla
      import Memory
      import Sys
      import Status
      import Cron

      asset.AvrConf = dict( pMain.Avr )

      Ookla.init    ()
      Memory.init   ()
      Sys.GetSystemInfo()
      Status.init   ()

      Cron.init     ()  # --- Init CronJob Values
      Cron.checkTime()  # --- Init Schedule Status
      Cron.CronJob  ()  # --- Init Cron Job(s)

   # --------------------------------------------------------------------------------
   import isAdmin
   isAdmin.schedule()

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
