# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import asset
from   Html   import *
import lib

# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
PageName    = 'Hdd'
FuncSubList = [ 'list', 'view', ]

ModDesc     = f'{asset.cgiModulPath}/mod_{PageName}.json'

sDiskType   = {   'HDD' : 'hdd_hdd.png',
                  'SSD' : 'hdd_ssd.png', }

iDiskType   = {   'False' : 'hdd_hdd.png',
                  'True'  : 'hdd_ssd.png', }

# -----------------------------------------------------------------------------------
# PlugIn Descriptor :
# -----------------------------------------------------------------------------------
def pluginDesc () :

   # --------------------------------------------------------------------------------
   rDict = { 'IconName'     : 'hdd_hdd.png',                      # Modul-Icon
             'Description'  : asset.msg['modHddTextMain'],        # Modul-Description
             'PageName'     : PageName,                           # Modul-Page Name
             'SubList'      : FuncSubList,                        # List of includet Sub-Functions
             'Subs'         : dict(),
           }

   # --------------------------------------------------------------------------------
   import basics as io
   from importlib import import_module as Package

   for sName in FuncSubList :
      Sub = Package( f'c{PageName}.{sName}' )
      rDict[ 'Subs' ][ sName ] = Sub.FuncDesc

   io.ioWrite( json.dumps( rDict, sort_keys=True, indent=4 ), ModDesc )

   # --------------------------------------------------------------------------------
   return rDict

# -----------------------------------------------------------------------------------
# --- Display the "SUB"-Menue
# -----------------------------------------------------------------------------------
def sub( sSite, sMore ) :

   jBusy ( 'msgCgiWaitInfo' )

   for sSub in asset.desModule[ PageName ]['SubList'] :
      # -----------------------------------------------------------------------------
      # --- Skip function if this doesn't contain a menu button ...
      if not asset.desModule[ PageName ]['Subs'][ sSub ]['IconName'] : continue

      if sSub == sSite : opt = True
      else             : opt = False
      menuBtn  (  asset.desModule[ PageName ]['Subs'][ sSub ]['SubName' ],
                  asset.desModule[ PageName ]['Subs'][ sSub ]['SubText' ],
                  asset.desModule[ PageName ]['Subs'][ sSub ]['iWidth'  ],
                  asset.desModule[ PageName ]['Subs'][ sSub ]['IconName'],
                  '%s-%s' % ( PageName, sSub ), opt=opt )

   # --------------------------------------------------------------------------------
   iWidth      = iMainSubWidth -   6
   iHeight     = iMainHeight   - 123         # << Eine Reihe
#  iHeight     = iMainHeight   - 167         # << Zwei Reihen

   # --------------------------------------------------------------------------------
   import Smart
   f     = Smart.Smart( )
   rTemp = f.getSynoStorage()
   sEnum, sDisks = f.getDriveLocation()

   # --------------------------------------------------------------------------------
   sItems = sEnum[ 'iLocInt' ]
   for sName in sItems :
      Section( sName )
      iList = sItems[ sName ]

      rDict = dict()
      for sDev in iList :
         iNum = int( rTemp[ 'disks' ][ sDev ][ 'number' ] )
         rDict[ f'{iNum:02d}' ] = sDev

      # -----------------------------------------------------------------------------
      for sKey in  sorted( rDict ) :
         sDev = rDict[ sKey ]
         if sDev not in sDisks :
            lib.Error ( 'CGI: The device "{0}" is not in the "enumStorage" ... will be skipped'.format( sDev ) )
            continue
         sDrives = sDisks[ sDev ]
         sPage  = f'{PageName}-view+{sDev}'
         sTitel = '{0} : {vendor} {model} {serial} [{size}]'.format( sDev, **sDrives )
         if sDev == sMore : opt = True
         else             : opt = False

         menuBtn ( '{name}'.format( **sDrives ) , sTitel, SubMenuWidth, iDiskType[ str( sDrives[ 'isSSD' ] ) ], sPage, opt=opt )

   # --------------------------------------------------------------------------------
   if sEnum[ 'iLocM2' ] :
      Section ( 'M.2 Slot' )

      # -----------------------------------------------------------------------------
      rDict = dict()
      for sDev in sEnum[ 'iLocM2' ] :
         rDict[ rTemp[ 'disks' ][ sDev ][ 'number' ] ] = sDev

      # -----------------------------------------------------------------------------
      for sKey in  sorted( rDict ) :
         sDev = rDict[ sKey ]
         sDrives = sDisks[ sDev ]
         sPage  = f'{PageName}-view+{sDev}'
         sTitel = '{0} : {vendor} {model} {serial} [{size}]'.format( sDev, **sDrives )
         if sDev == sMore : opt = True
         else             : opt = False

         sName = asset.msg[ 'sysSSDName' ] + ' ' + rTemp[ 'disks' ][ sDev ][ 'number' ]
         menuBtn ( sName, sTitel, SubMenuWidth, iDiskType[ str( sDrives[ 'isSSD' ] ) ], sPage, opt=opt )

   # --------------------------------------------------------------------------------
   sItems = sEnum[ 'iLocBox' ]
   for sName in sItems :
      Section( 'EUnit ' + sName )
      iList = sItems[ sName ]

      rDict = dict()
      for sDev in iList :
         iNum = int( rTemp[ 'disks' ][ sDev ][ 'number' ] )
         rDict[ f'{iNum:02d}' ] = sDev

      for sKey in  sorted( rDict ) :
         sDev    = rDict[ sKey ]
         sDrives = sDisks[ sDev ]
         sPage   = f'{PageName}-view+{sDev}'
         sTitel  = '{0} : {vendor} {model} {serial} [{size}]'.format( sDev, **sDrives )
         if sDev == sMore : opt = True
         else             : opt = False

         menuBtn ( '{name}'.format( **sDrives ) , sTitel, SubMenuWidth, iDiskType[ str( sDrives[ 'isSSD' ] ) ], sPage, opt=opt )

   # --------------------------------------------------------------------------------
   if sEnum[ 'iLocUsb' ] or sEnum[ 'iLocEsata' ] :
      Section ( 'modHddExtern' )

      # -----------------------------------------------------------------------------
      rDict = dict()
      for sDev in sEnum[ 'iLocUsb' ] :
         iID = int( rTemp[ 'disks' ][ sDev ][ 'number' ] )
         rDict[ f'{iID:02d}' ] = sDev

      for sKey in  sorted( rDict ) :
         sDev = rDict[ sKey ]
         sDrives = sDisks[ sDev ]
         sPage  = f'{PageName}-view+{sDev}'
         sTitel = '{0} : {vendor} {model} {serial} [{size}]'.format( sDev, **sDrives )
         if sDev == sMore : opt = True
         else             : opt = False

         menuBtn ( '{name}'.format( **sDrives ) , sTitel, SubMenuWidth, iDiskType[ str( sDrives[ 'isSSD' ] ) ], sPage, opt=opt )

      # -----------------------------------------------------------------------------
      rDict = dict()
      for sDev in sEnum[ 'iLocEsata' ] :
         rDict[ rTemp[ 'disks' ][ sDev ][ 'number' ] ] = sDev

      for sDev in sorted( sEnum[ 'iLocEsata' ] ) :
         sDrives = sDisks[ sDev ]
         sPage  = f'{PageName}-view+{sDev}'
         sTitel  = '{0} : {vendor} {model} {serial} [{size}]'.format( sDev, **sDrives )
         if sDev == sMore : opt = True
         else             : opt = False

         menuBtn ( '{name}'.format( **sDrives ) , sTitel, SubMenuWidth, iDiskType[ str( sDrives[ 'isSSD' ] ) ], sPage, opt=opt )

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
