# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

# -----------------------------------------------------------------------------------
# --- Class to handle Config- / Language-Files.
# -----------------------------------------------------------------------------------
class Configs :

   # --------------------------------------------------------------------------------
   def __init__( self, sSource=False, sGuest=False, isLang=False, version=False ) :
      self.tSource = sSource
      self.tGuest  = sGuest
      self.isLang  = isLang
      self.Version = version
      self.Values  = dict()

      if self.tSource : self.lSource, self.sSource, self.mSource = self.Import( self.tSource )
      if self.tGuest  : self.lGuest,  self.sGuest,  self.mGuest  = self.Import( self.tGuest  )

   # --------------------------------------------------------------------------------
   def init( self, sSource=False, sGuest=False ) :

      if sSource :
         self.tSource = sSource
         self.lSource, self.sSource, self.mSource = self.Import( self.tSource )

      if sGuest :
         self.tGuest  = sGuest
         self.lGuest,  self.sGuest,  self.mGuest  = self.Import( self.tGuest  )

   # --------------------------------------------------------------------------------
   def Preset( self, Source ) :

      with open( Source, 'r', encoding='utf-8' ) as f :
         Source = f.readlines()

      sPreset = ''

      for Line in Source :

         Line = Line.strip()

         if not Line or Line.startswith( '#' ) :
            sPreset += Line + '\n'
            continue

         sKey = Line.split( '=', 1 )[0]
         sID  = sKey.strip()

         if sID not in self.sSource :
            continue

         sPreset += f'{sKey}= "{self.sSource[sID]}"\n'

      return sPreset.strip() + '\n'

   # --------------------------------------------------------------------------------
   def Import( self, Source ) :

      with open( Source, 'r', encoding='utf-8' ) as f :
         Source = f.readlines()

      lClean = []
      sClean = dict()
      lenKey = 0

      for Line in Source :
         try    : key, value = self.Tuple( Line.strip() )
         except : continue

         lClean.append( key )
         sClean[key] = value

         if len( key) > lenKey :
            lenKey = len( key)

      return lClean, sClean, lenKey

   # --------------------------------------------------------------------------------
   def Tuple ( self, line ) :
      if len( line ) == 0 or line[0] == '#' : return

      try    : key, value = line.split( '=', 1 )
      except : return

      key   = key.strip()
      value = value.strip()

      if not self.isLang and key == 'Version' : return

      if len( value ) == 0 : return

      # -----------------------------------------------------------------------------
      if self.isLang :
         if '#' in value :
            value = value.split( '#', 1 )[0].strip()

         value = value[1:-1].replace( '\\','')
      else :
         if value.startswith( '""' ) :
            if '""' in value[2:] :
               value = '"' + value[2:].split( '""', 1 )[0] + '"'
            else :
               value = ''
         else :
            value = value[1:].split( '"', 1 )[0]

      # -----------------------------------------------------------------------------
      return key, value

   # --------------------------------------------------------------------------------
   def ClearValues( self ) :
      self.Values = dict()

   # --------------------------------------------------------------------------------
   def NewValues( self, New ) :
      for Key in New :
         self.Values[Key] = New[Key]

   # --------------------------------------------------------------------------------
   def GetCleaned( self, sort=False, nice=False, guest=False ) :
      sText = ''

      if self.isLang : Quote = "'"
      else           : Quote = '"'

      if sort : iList = sorted( self.lSource )
      else    : iList = self.lSource
      sSource = { **self.sSource }

      if self.Version : 
         iList.insert( 0, 'Version' )
         sSource['Version'] = self.Version

      if guest and self.tGuest :
         for key in iList :
            if key in self.sGuest :
               sSource[key] = self.sGuest[key]

      for key in self.Values :
         sSource[key] = self.Values[key]

      for sID in iList :
         if nice : sText += '{0:{width}} = {2}{1}{2}\n'.format( sID, sSource[sID], Quote, width=self.mSource )
         else    : sText += '{0}={2}{1}{2}\n'.format( sID, sSource[sID], Quote )

      return sText.strip()

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
