#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import time

import asset
from   lib      import switch, case
import lib
import basics   as io

# -----------------------------------------------------------------------------------
import __main__ as avr
cron = sys.modules[ __name__ ]
# -----------------------------------------------------------------------------------

sWeekDays = [ 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun' ]

# -----------------------------------------------------------------------------------
# --- Find a job and delete it ...
# -----------------------------------------------------------------------------------
def Remove( sToken ):

   for i in range( 0, len(cron.Tab) ):
      if sToken in cron.Tab[i] :
         cron.Tab.pop(i)
         return True

   return False

# -----------------------------------------------------------------------------------
# --- Find a job, Refresh or create ...
# -----------------------------------------------------------------------------------
def Update( sToken, aTime, sJob ) :

   for i in range( 0, len( cron.Tab ) ):
      # --------------------------------------------------------------------------
      if sToken in cron.Tab[ i ] :
         # --- Compare current Job with the expected one ...
         curCron = ''.join( cron.Tab[ i ].split() )
         cmpCron = ''.join( ( ''.join( aTime ) + sJob + sToken ).split() )

         if curCron == cmpCron :
            # --- if equal: Do nothing, return ...
            return False      # --- False : Found but nothing to do ...
         else :
            # --- Update the current Cron-Job ...
            cron.Tab[i] = '\t'.join( aTime ) + '\t' + sJob + ' ' + sToken
            return True       # ---     1 : Found and updated ...

   # --------------------------------------------------------------------------
   cron.Tab.append( '\t'.join( aTime ) + '\t' + sJob + ' ' + sToken )
   return True                # ---    -1 : Not found but created ...

# -----------------------------------------------------------------------------------
# --- Check crontab if Job exist: Create Job; Update Job; Delete Job
# -----------------------------------------------------------------------------------
def CronJob() :

   # --- We use /etc/cron.d/crontab ...
   if not os.path.exists( asset.CronTabFile ) :
      # --- If not exist: create a new crontab ...
      cron.Tab     = [ 'MAILTO=""' ]
      cron.Tab.append( 'PATH=/sbin:/bin:/usr/sbin:/usr/bin:/usr/syno/sbin:/usr/syno/bin:/usr/local/sbin:/usr/local/bin' )
      cron.Tab.append( '#minute\thour\tmday\tmonth\twday\twho\tcommand' )
      Flag = True
   else :
      # --- Get (read) crontab ...
      cron.Tab = io.ioRead( asset.CronTabFile ).split('\n')
      Flag = False

   # ------------------------------------------------------------------------------------------------
   # -- Configure "Speedtest-Job"
   # ------------------------------------------------------------------------------------------------
   if not asset.AvrConf['SpeedTestRegistered'] or asset.AvrConf['SpeedTestOn'] != "Yes" or cron.ScheduleStatus == -1 or io.check( asset.f_AvrDisabled ) :
      # --- If SpeedTest is "Off" or not "Registered" or Schedule is "On" and Time forbidden: Remove Speedtest-Job ...
      if Remove( 'speed' ) : Flag = True
   else :
      # --- Check Update-Job .....
      if Update( 'speed', cron.CronSpeed, asset.CronJob ) : Flag = True

   # ------------------------------------------------------------------------------------------------
   # -- Configure "Update-Job"
   # ------------------------------------------------------------------------------------------------
   if cron.ScheduleStatus == -1 or io.check( asset.f_AvrDisabled )  :
      # --- If Schedule is "On" and Time forbidden: Remove Update-Job ...
      if Remove( 'update' ) : Flag = True
   else :
      # --- Check Update-Job .....
      if Update( 'update', cron.CronUpdate, asset.CronJob ) : Flag = True

   # ------------------------------------------------------------------------------------------------
   # -- Configure "Archive-Job"
   # ------------------------------------------------------------------------------------------------
   if asset.AvrConf["SvgArchiveMode"] == "Off" or asset.AvrConf["SvgArchiveMode"] == "Stop" or cron.ScheduleStatus == -1 or io.check( asset.f_AvrDisabled ) :
      # --- If Archiving is "Off" OR Schedule is "On" and Time forbidden: Remove Archive-Job ...
      if Remove( 'archive' ) : Flag = True
   else :
      # --- Check Archive-Job ---
      if Update ( 'archive', cron.CronArchive, asset.CronJob ) : Flag = True

   # ------------------------------------------------------------------------------------------------
   # -- Configure "Mail notification-Job"
   # ------------------------------------------------------------------------------------------------
   if not 'Yes' in asset.AvrConf["StatusMailOn"] or cron.ScheduleStatus == -1 or io.check( asset.f_AvrDisabled ) :
      # --- If Mail report is "Off" OR Schedule is "On" and Time forbidden: Remove Archive-Job ...
      if Remove( 'mail' ) : Flag = True
   else :
      # --- Check Archive-Job ---
      if Update( 'mail', cron.CronStatus, asset.CronJob ) : Flag = True

   # ------------------------------------------------------------------------------------------------
   # --- Configure "Schedule start / stop Jobs"
   # ------------------------------------------------------------------------------------------------
   for iID, sID in enumerate( [ '1', '2', '3', '4', '5', '6', '7' ] ) :
      if not cron.CronSchDay[ iID ] or io.check( asset.f_AvrDisabled ) :
         # --- Remove "start" Job ...
         if Remove( f'start {sWeekDays[ iID ]}' ) : Flag = True
         # --- Remove "stop" Job ...
         if Remove( f'stopp {sWeekDays[ iID ]}' ) : Flag = True
      #  # --- Remove "checkTime" Job ...
      #  if Remove( f'check {sWeekDays[ iID ]}' ) : Flag = True

      else :
         # -----------------------------------------------------------------------
         # --- Check Schedule-Jobs ...
         while switch( asset.AvrConf[ f'Schedule_{sID}_Mode' ] ):
            if case( 'Yes' ):
               sMode = '0'
               break
            if case( 'Set1' ):
               sMode = '1'
               break
            if case( 'Set2' ):
               sMode = '2'
               break
            if case( 'Set3' ):
               sMode = '3'
               break
            # --- Default's: ----------------------------------------------------
            sMode = '0'
            break

         # -----------------------------------------------------------------------
         if sMode != '0' :
            sStartH, sStartM = asset.AvrConf[ f'ScheduleSet{sMode}Start' ].split( ':', 1 )
            sStoppH, sStoppM = asset.AvrConf[ f'ScheduleSet{sMode}Stop'  ].split( ':', 1 )
            # --- "schedule" Daemon & for start / stop hibernation ...
            if Update( f'start {sWeekDays[ iID ]}', [ sStartM, sStartH, '*', '*', sID ], asset.CronJob ) : Flag = True
            if Update( f'stopp {sWeekDays[ iID ]}', [ sStoppM, sStoppH, '*', '*', sID ], asset.CronJob ) : Flag = True

   # ------------------------------------------------------------------------------------------------
   if asset.AvrConf[ 'ScheduleDayMask' ] != '0000000' and not io.check( asset.f_AvrDisabled ) :
      if Update( 'check', [ '0', '0', '*', '*', '*' ], asset.CronJob ) : Flag = True
   else :      
      if Remove( f'check' ) : Flag = True

   # ------------------------------------------------------------------------------------------------
   try:    console = avr.AvrDebug
   except: console = False

   # ------------------------------------------------------------------------------------------------
   if Flag :
      if len( cron.Tab ) >= 4 : io.ioWrite( '\n'.join( cron.Tab ) + '\n', asset.CronTabFile, opt=False )
      else                    : io.ioRm( asset.CronTabFile )

   #  if not console and not asset.f_Dsm7 : lib.getCMD( '/usr/syno/sbin/synoservicectl --reload crond &> /dev/null' )
      if not console and not asset.f_Dsm7 : os.system( '/usr/syno/sbin/synoservicectl --reload crond &> /dev/null' )

# --------------------------------------------------------------------------
# --- Check Scheduling and current time is within a time window
# --- >>> Return:
#    -1 = Forbidden time
#     0 = Scheduling is "OFF"
#     1 = Allowed time
# --------------------------------------------------------------------------
def checkTime( opt=False, bash=False ) :

   # -----------------------------------------------------------------------
   if opt :
      asset.AvrConf = lib.ConfigRead( opt )
   else :
      if not bool( asset.AvrConf ) : asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )

   if not asset.AvrConf[ 'ScheduleDayMask' ] :
      init()
      lib.ConfigSave()

   # -----------------------------------------------------------------------
   # --- Init (default) is "False" ...
   cron.ScheduleStatus = False
   cron.CronSchDay = [ x == '1' for x in list( asset.AvrConf[ 'ScheduleDayMask' ] ) ]
   wDay = int( time.strftime('%u') ) - 1

   # -----------------------------------------------------------------------

   if not cron.CronSchDay[ wDay ] :
      cron.ScheduleStatus = 0    # --- Scheduling is "OFF"
      return cron.ScheduleStatus

   # -----------------------------------------------------------------------
   while switch( asset.AvrConf[ f'Schedule_{wDay+1}_Mode' ] ):
      if case( 'Yes' ):
         cron.ScheduleStatus = -1   # --- Scheduling is "ON" and Forbidden time
         return cron.ScheduleStatus
         break
      if case( 'Set1' ):
         sMode = '1'
         break
      if case( 'Set2' ):
         sMode = '2'
         break
      if case( 'Set3' ):
         sMode = '3'
         break
      # --- Default's: ----------------------------------------------------
      sMode = '1'
      break

   # -----------------------------------------------------------------------
   curTime = int( time.strftime('%-H') ) * 3600 + int( time.strftime('%-M') ) * 60 + int( time.strftime('%-S') )

   # -----------------------------------------------------------------------
   sStartH, sStartM = asset.AvrConf[ f'ScheduleSet{sMode}Start' ].split( ':', 1 )
   sStoppH, sStoppM = asset.AvrConf[ f'ScheduleSet{sMode}Stop'  ].split( ':', 1 )

   Start = int( sStartH ) * 3600 + int( sStartM ) * 60
   Stopp = int( sStoppH ) * 3600 + int( sStoppM ) * 60

   # -----------------------------------------------------------------------
   if Start <= Stopp : iStat = Start <= curTime <= Stopp
   else              : iStat = Start <= curTime or curTime <= Stopp

   if iStat : cron.ScheduleStatus = -1 # * iMul   # --- Scheduling is "ON" and Forbidden time
   else     : cron.ScheduleStatus =  1 # * iMul   # --- Scheduling is "ON" and Allowed time

   # -----------------------------------------------------------------------
   if bash : print ( cron.ScheduleStatus )
   else    : return cron.ScheduleStatus

# --------------------------------------------------------------------------
# --- Init CronJob Values ...
# --------------------------------------------------------------------------
def init( opt=True ):

   # -----------------------------------------------------------------------
   # --- Set the default values after an update ...
   if not asset.AvrConf[ 'ScheduleDayMask' ] :
      # --------------------------------------------------------------------
      if asset.AvrConf[ 'ScheduleOn' ] == 'Yes' :
         while switch( asset.AvrConf["ScheduleDay"] ):
            if case( 'Every-Day' ):
               asset.AvrConf[ 'ScheduleDayMask' ] = '1111111'
               break
            if case( 'Working-Days' ):
               asset.AvrConf[ 'ScheduleDayMask' ] = '1111100'
               break
            if case( 'Weekend' ):
               asset.AvrConf[ 'ScheduleDayMask' ] = '0000011'
               break
         # --- Default's: -------------------------------------------------
            asset.AvrConf[ 'ScheduleDayMask' ] = '1111111'
            break

      else :
         asset.AvrConf[ 'ScheduleDayMask' ] = '0000000'

      # --------------------------------------------------------------------
      asset.AvrConf[ 'ScheduleSet1Start' ] = f'{asset.AvrConf[ "ScheduleStartH" ]}:{asset.AvrConf[ "ScheduleStartM" ]}'
      asset.AvrConf[ 'ScheduleSet1Stop'  ] = f'{asset.AvrConf[ "ScheduleStopH"  ]}:{asset.AvrConf[ "ScheduleStopM"  ]}'

      # --------------------------------------------------------------------
      cron.CronSchDay = [ x == '1' for x in list( asset.AvrConf[ 'ScheduleDayMask' ] ) ]

      for iID, sID in enumerate( [ '1', '2', '3', '4', '5', '6', '7' ] ) :
         if cron.CronSchDay[ iID ] :
            asset.AvrConf[ f'Schedule_{sID}_Mode' ] = 'Set1'
         else :
            asset.AvrConf[ f'Schedule_{sID}_Mode' ] = 'Yes'

      # --------------------------------------------------------------------
      if asset.AvrConf[ 'ScheduleMode' ] == 'Yes' :
         asset.AvrConf[ 'ScheduleDayMask' ] = '1111111'

   # -----------------------------------------------------------------------
   # ---  ScheduleDay's (cron job) ...
   cron.CronSchDay = [ x == '1' for x in list( asset.AvrConf[ 'ScheduleDayMask' ] ) ]

   # -----------------------------------------------------------------------
   # --- LogBase & CronArchive (Archiving timings interval) in hours ...
   while switch( asset.AvrConf["SvgArchiveMode"] ):
      if case( 'Off' ):    # --- Cron-Archiving: Off
         cron.CronArchive = []   #  ""
         break
      if case( 'Day' ):    # --- Cron-Archiving: Daily
         cron.CronArchive = [ '0', '0',  '*/1',   '*', '*' ]
         break
      if case( 'Time' ):   # --- Cron-Archiving: Given Time
         cron.CronArchive = [ asset.AvrConf["SvgArchiveMin"], asset.AvrConf["SvgArchiveStd"], '*', '*', '*' ]
         break
      if case( 'Base' ):   # --- Use LogBase for Timmings
         while switch( asset.AvrConf["LogBase"] ):
            if case(   '1' ):
               cron.CronArchive = [ '0',  '*/1',   '*', '*', '*' ]      #  [0  */1  *	 *    *]  = every   1h
               break
            if case(   '2' ):
               cron.CronArchive = [ '0',  '*/2',   '*', '*', '*' ]      #  [0	*/2  *	 *	   *]  = every   2h
               break
            if case(   '3' ):
               cron.CronArchive = [ '0',  '*/3',   '*', '*', '*' ]      #  [0	*/3  *	 *	   *]  = every   3h
               break
            if case(   '6' ):
               cron.CronArchive = [ '0',  '*/6',   '*', '*', '*' ]      #  [0	*/6  *	 *    *]  = every   6h
               break
            if case(  '12' ):
               cron.CronArchive = [ '0', '*/12',   '*', '*', '*' ]      #  [0	*/12 *    *	   *]  = every  12h
               break
            if case(  '24' ):
               cron.CronArchive = [ '0',    '0', '*/1', '*', '*' ]      #  [0	0    */1  *	   *]  = every  24h
               break
            if case(  '48' ):
               cron.CronArchive = [ '0',    '0', '*/2', '*', '*' ]      #  [0	0	  */2  *	   *]  = every  48h
               break
            if case( '168' ):
               cron.CronArchive = [ '0',    '0', '*/7', '*', '*' ]      #  [0	0	  */7  *	   *]  = every 168h
               break
            break
         break
      if case('Stop' ):
         cron.CronArchive = ""
         break
      # --- Default's: ----------------------------------------------------
      cron.CronArchive = ""
      asset.AvrConf["SvgArchiveMode"] = "Off"
      break

   # --- SVG-Tick & CronUpdate (Update interval) in minutes ...
   while switch( asset.AvrConf["SvgTick"] ):
      if case(  '1' ):
         cron.CronUpdate = [ '*/1',  '*', '*', '*', '*' ]  #  [*/1  *   *	  *    *]  = every   1min
         break
      if case(  '2' ):
         cron.CronUpdate = [ '*/2',  '*', '*', '*', '*' ]  #  [*/2  *   *	  *    *]  = every   2min
         break
      if case(  '5' ):
         cron.CronUpdate = [ '*/5',  '*', '*', '*', '*' ]  #  [*/5  *	 *   *    *]  = every   5min
         break
      if case( '10' ):
         cron.CronUpdate = [ '*/10', '*', '*', '*', '*' ]  #  [*/10 *	 *   *    *]  = every  10min
         break
      if case( '20' ):
         cron.CronUpdate = [ '*/20', '*', '*', '*', '*' ]  #  [*/20 *	 *   *    *]  = every  20min
         break
      if case( '30' ):
         cron.CronUpdate = [ '*/30', '*', '*', '*', '*' ]  #  [*/30 *	 *   *    *]  = every  30min
         break
      if case( '60' ):
         cron.CronUpdate = [ '0',  '*/1', '*', '*', '*' ]  #  [0    */1 *	  *    *]  = every  60min
         break
      # --- Default's ----------------------------------------------------
      cron.CronUpdate = [ '*/10', '*', '*', '*', '*' ]
      asset.AvrConf["SvgTick"] = 10 # ---Default: 10min
      break

   while switch( asset.AvrConf["SvgArchiveMode"] ):
      if case( 'Day' ):
         cron.SvgArchivingTime = "daily at 0 o'clock"
         break
      if case( 'Base' ):
         cron.SvgArchivingTime = "every " + asset.AvrConf["LogBase"] + " hours"
         break
      if case( 'Time' ):
         cron.SvgArchivingTime = 'daily at %0.2d:%0.2d' % ( int(asset.AvrConf["SvgArchiveStd"]), int(asset.AvrConf["SvgArchiveMin"]) )
         break
      if case( 'Stop' ):
         cron.SvgArchivingTime = "every time \"Hibernation\" starts."
         break
      break

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
def clearCron ( bash=False ) :

   # --------------------------------------------------------------------------------
   # --- Return (exit) if there is no Cron-Job ...
   if not os.path.exists( asset.CronTabFile ) : return

   # --------------------------------------------------------------------------------
   #    -1 = Forbidden time
   #     0 = Scheduling is "OFF"
   #     1 = Allowed time
   Status = checkTime()

   # --------------------------------------------------------------------------------
   # --- Get (read) crontab ...
   cron.Tab = io.ioRead( asset.CronTabFile ).split('\n')
   Flag = False

   if Remove( 'archive' ) : Flag = True
   if Remove( 'update'  ) : Flag = True
   if Remove( 'speed'   ) : Flag = True
   if Remove( 'mail'    ) : Flag = True
   if Remove( 'check'   ) : Flag = True

   for sDay in sWeekDays :
      if Remove( f'start {sDay}' ) : Flag = True
      if Remove( f'stopp {sDay}' ) : Flag = True

   # ------------------------------------------------------------------------------------------------
   if Flag :
      if len( cron.Tab ) >= 4 : io.ioWrite( '\n'.join( cron.Tab ) + '\n', asset.CronTabFile, opt=False )
      else                    : io.ioRm( asset.CronTabFile )

   #  if not asset.f_Dsm7 : lib.getCMD( '/usr/syno/sbin/synoservicectl --reload crond &> /dev/null' )
      if not asset.f_Dsm7 : os.system( '/usr/syno/sbin/synoservicectl --reload crond &> /dev/null' )

# -----------------------------------------------------------------------------------
# --- Function-Array : Do not move !!
# -----------------------------------------------------------------------------------
iFunctions =   {
                  'check'     : checkTime ,
                  'clear'     : clearCron ,
               }

# -----------------------------------------------------------------------------------
# --- When Cron.py was called up as a stand-alone program :
# -----------------------------------------------------------------------------------
if __name__ == '__main__' :

   sFunc = ''

   # --------------------------------------------------------------------------------
   # --- Check if an argument has been passed ...
   for arg in sys.argv[1:] :
      while lib.switch( arg ) :
         # --------------------------------------------------------------------------
         if lib.case( 'check' ) :
            sFunc = 'check'
            break
         # --------------------------------------------------------------------------
         if lib.case( 'clear' ) :
            sFunc = 'clear'
            break
         # --- Default --------------------------------------------------------------
         break

   # --------------------------------------------------------------------------------
   sFunctions = iFunctions.get( sFunc, False )
   if not sFunctions :
      if sys.argv[1:] : lib.Error( f'CRN: Unknown ARG "{arg}" has passed ...' )
      else            : lib.Error( f'CRN: Missing ARG ...' )
      lib.Error(  'CRN: Execution was canceled' )
   else :
      sFunctions( bash=True )


# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
