# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import time
import datetime
from   shutil  import copy, chown

import basics   as io
from   lib      import switch, case
import lib
import asset

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------

# --------------------------------------------------------------------------
# --- Prepare "CSV-Preview" ...
# --------------------------------------------------------------------------
def Preview( csvConf ) :

   sPreView = ''
   csvConf[ 'BaseListSMA'   ] = [ asset.TabLoad, asset.TabCpu, asset.TabHmax ]
   csvConf[ 'AvrLogCsvMode' ] = 'Day'

   # --- First "Call" with HeadLine :
   csvConf[ 'CsvHeaderPreview' ] = True
   sPreView += Add( csvConf )


   # --- Next "Call's" without HeadLine :
   csvConf[ 'CsvHeaderPreview' ] = False
   sPreView += Add( csvConf )
   sPreView += Add( csvConf )

   return sPreView.strip()

# --------------------------------------------------------------------------
# --- Add a Line to CSV-File ...
# --------------------------------------------------------------------------
def Add( iPreview=False ) :

   if iPreview :
      csvConf        = iPreview
      iBaseListSMA   = csvConf[ 'BaseListSMA' ]
      from random import uniform as fRandom
   else :
      csvConf        = asset.AvrConf
      iBaseListSMA   = asset.BaseListSMA

   # -----------------------------------------------------------------------
   # --- Prepare Filename ...
   while switch( csvConf['AvrLogCsvMode'] ):
      if case( 'Off' ):
         return
         break
      if case( 'Day' ):
         csvTime = datetime.datetime.today()
         csvTime = time.mktime( csvTime.timetuple() )
         break
      if case( 'Week' ):
         csvTime = datetime.datetime.today() - datetime.timedelta( days = datetime.datetime.today().weekday() )
         csvTime = time.mktime( csvTime.timetuple() )
         break
      if case( 'Month' ):
         csvTime = datetime.datetime.today()
         csvTime = time.mktime( datetime.datetime(csvTime.year, csvTime.month, 1).timetuple() )
         break
      break

   # -----------------------------------------------------------------------
   csvName = time.strftime( '%Y.%m.%d log.csv', time.localtime( csvTime ) )
   if not iPreview : csvFile = '/'.join(( asset.sys_CsvPath, csvName ))
   else            : csvFile = 'Test'
   # -----------------------------------------------------------------------

   while switch( csvConf['AvrLogCsvColSign'] ):
      if case( 'Tabstop' ):
         csvCol = '\t'
         break
      if case( 'Semikolon' ):
         csvCol = ';'
         break
      if case( 'Komma' ):
         csvCol = ','
         break
      break

   # -----------------------------------------------------------------------
   # --- Prepare CSV-Headline ...
   if not iPreview : iHead = not io.check( csvFile ) and csvConf['AvrLogCsvHeaderOn'] == 'Yes'
   else            : iHead = csvConf.get( 'CsvHeaderPreview', False ) and csvConf['AvrLogCsvHeaderOn'] == 'Yes'

   if iHead :
      nice = lib.Split( csvConf['SvgBaseDesc'] )

      while switch( csvConf['AvrLogCsvTextSign'] ):
         if case( 'Off' ):
            csvText = ''
            break
         if case( 'Apostrophe' ):
            csvText = '\''
            break
         if case( 'Doublequote' ):
            csvText = '\"'
            break
         break

      line = ''

      if csvConf['AvrLogCsvDate'] != 'Off' : line += csvText + 'Date' + csvText + csvCol

      if csvConf['AvrLogCsvTime'] != 'Off' : line += csvText + 'Time' + csvText + csvCol

      for j in iBaseListSMA :
         line += csvText + nice[j][0:-2] + csvText + csvCol

      line = line.rstrip( csvCol )

      # --------------------------------------------------------------------
      # --- Mark CSV-File as '(with BOM)' / UTF-8
      if not iPreview :
         line = '\ufeff' + line + '\n'

         io.ioWrite( line, csvFile )
         chown ( csvFile, user='admin', group='users' )
         os.chmod ( csvFile, 0o755 )
      else :
         line += '\n'

   else :
      line  = ''

   # -----------------------------------------------------------------------

   # -----------------------------------------------------------------------
   # --- Prepare CSV-Valueline ...
   while switch( csvConf['AvrLogCsvDate'] ) :
      if case( 'YYYY.MM.DD' ):
         line += time.strftime( '%Y.%m.%d' ) + csvCol
         break
      if case( 'DD.MM.YYYY' ):
         line += time.strftime( '%d.%m.%Y' ) + csvCol
         break
      if case( 'Off' ):
         break
      break

   while switch( csvConf['AvrLogCsvTime'] ) :
      if case( 'HH:MM:SS' ):
         line += time.strftime( '%H:%M:%S' ) + csvCol
         break
      if case( 'HH:MM' ):
         line += time.strftime( '%H:%M' ) + csvCol
         break
      if case( 'Off' ):
         break
      break

   while switch( csvConf['AvrLogCsvDecSign'] ) :
      if case( 'Punkt' ):
         csvDec = '.'
         break
      if case( 'Komma' ):
         csvDec = ','
         break
      break

   for j in iBaseListSMA :
      if not iPreview : sRaw = asset.Summe[j] / asset.SMAdepth
      else            : sRaw = fRandom( 35, 45 )

      if asset.TabPing == j :
         value = '%.3f' % ( sRaw * 10 )
      else :
         value = '%.3f' % ( sRaw )

      value = value.replace( '.', csvDec )
      line += value + csvCol
      
   line = line.rstrip( csvCol )

   if not iPreview : io.ioAdd( line, csvFile )
   else            : return line + '\n'

# ----------------------------------------------------------
# --- Prepare CSV-Log Folder ...
# ----------------------------------------------------------
def init( opt=True ):

   asset.sys_CsvPath = None
   if asset.AvrConf['AvrLogCsvMode'] == 'Off' : return

   # ----------------------------------------------------------
   if asset.AvrConf['AvrLogCsvPath'] == '' :
      arr = lib.FindeShare( asset.AvrConf['SvgArchivePath'] )
      if arr :
         sDir = '/'.join(( arr[0], arr[1], 'csv' ))
         asset.AvrConf['AvrLogCsvPath'] = arr[1]

   # ----------------------------------------------------------
   arr = lib.FindeShare( asset.AvrConf['AvrLogCsvPath'] )

   if not arr :
      asset.AvrConf['AvrLogCsvMode'] = 'Off'
      return

   # ----------------------------------------------------------
   asset.sys_CsvPath = '/'.join( arr )

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# sys.exit()
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
