# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import sys
import time
from   math     import log10 as log
import csv

import asset
import basics   as io
from   lib      import switch, case
import lib
import Sys
import __main__ as avr

# -----------------------------------------------------------------------------------
flow = sys.modules[ __name__ ]
# -----------------------------------------------------------------------------------
# --- Global options ...
# -----------------------------------------------------------------------------------
iAxis = [      0,      1,      2,      3,      4,      5 ]
sKey  = [ 'Pri1', 'Sec1', 'Pri2', 'Sec2', 'Pri3', 'Sec3' ]

# -----------------------------------------------------------------------------------
# --- Read DataBase and prepare Plot-Array's ...
# -----------------------------------------------------------------------------------
def init() :

   initFlag    = False
   minFlag     = False
   maxFlag     = False
   tab_lsCut   = False

   plotFlow    = [ '' ]   * asset.iTabLast
   startFlow   = [ '' ]   * asset.iTabLast
   startPos    = [ None ] * asset.iTabLast

   MaxFlow     = asset.sys_MaxLines + 1

#  rawFlow     = [ [ None for x in range( 0, asset.iTabLast ) ] for y in range( 0, MaxFlow ) ]
#  maxAxis     = [ [ None for x in range( 0, 6 )              ] for y in range( 0, MaxFlow ) ]
   rawFlow     = [ [ 0 for x in range( 0, asset.iTabLast ) ] for y in range( 0, MaxFlow ) ]
   maxAxis     = [ [ 0 for x in range( 0, 6 )              ] for y in range( 0, MaxFlow ) ]

   tabMod      = 10                       # mod=10 # Values per Line in Plot
   tab_lf      =  0                       # crlf=0
   tabCut      =  0                       # shift=0
   cutMod      =  MaxFlow - 1 + tabMod    # shift_m=$[max+mod]

   PlotRow     =  0
   RowOffset   =  0

   try:    flowLines = len( asset.DB )
   except: flowLines = False

   # --------------------------------------------------------------------------------
   # --- Determine the number of stored values (lines) ...
   if flowLines :
      io.touch( asset.f_Heartbeat )

      # -----------------------------------------------------------------------------
      # --- Always read in "one" more value (line) than MaxLine ...
      if flowLines >= 2 :
         minFlag  = True
         initFlag = True
      # -----------------------------------------------------------------------------
      # --- Init Count & Flags ....
      if flowLines >= MaxFlow :
         asset.DB = asset.DB[ flowLines - MaxFlow: ]

         flowLines = MaxFlow
         cutMod    = tabMod
         maxFlag   = True
         tab_lsCut = True
         RowOffset = 1

      # -----------------------------------------------------------------------------
      # --- Read in values (lines), prepare and determine max. Values of the axes ...
      for PlotRow in range( 0, flowLines ) :
         rawFlow[ PlotRow ] = [ float( x ) for x in asset.DB[ PlotRow ].split() ]

         # --------------------------------------------------------------------------
         # --- Walk through AxisList (Axis 0 - 3) and determine the highest value ...
         for Axis in iAxis :
            Max = 0
            # -----------------------------------------------------------------------
            # --- Get Max-Value of all Axis-Members ...
            if asset.ScalingAxis_Names[ Axis ] :
               # --- This is still the fastest option ;) 
               for j in asset.ScalingAxis_Names[ Axis ] :
                  if rawFlow[ PlotRow ][ j ] > Max : Max = rawFlow[ PlotRow ][ j ]
            # -----------------------------------------------------------------------
            maxAxis[ PlotRow ][ Axis ] = Max
            # -----------------------------------------------------------------------
            # --- Check if current Max-Value is greater than the last maximum value
            if Max > asset.ScalingAxis_Max[ Axis ] : asset.ScalingAxis_Max[ Axis ] = Max

      # -----------------------------------------------------------------------------
      # --- Walk through AxisList and calculate comparator window ...
      for Axis in iAxis :
         RoundUp( Axis, asset.ScalingAxis_Max[ Axis ] )

         try:    ScalingMinMax += f' {asset.ScalingAxis_Min[Axis]} {asset.ScalingAxis_Max[Axis]}'
         except: ScalingMinMax  =  f'{asset.ScalingAxis_Min[Axis]} {asset.ScalingAxis_Max[Axis]}'

      io.ioWrite( ScalingMinMax + " :initFlow\n", asset.avrScalingFile )
      asset.ScalingMinMax = ScalingMinMax

      # -----------------------------------------------------------------------------
      # --- Walk through AxisList and Prepare each SVG Plot-Line ...
      for Axis in iAxis :
         # --------------------------------------------------------------------------
         if asset.ScalingAxis_Names[ Axis ] == None : continue

         if asset.AvrConf[ sKey[ Axis ] + 'ScaleType' ] == 'Lin' : f_Lin = True
         else                                                    : f_Lin = False

         # --------------------------------------------------------------------------
         for Tab in asset.ScalingAxis_Names[ Axis ] :
            jFlow = []
            if f_Lin :
               iFlow = rawFlow[ 0 ][ Tab ] - asset.BaseDef[ Tab ]
               iPos  = rawFlow[ 0 ][ Tab ]
            else :
               try    : iFlow = log( 1 + rawFlow[ 0 ][ Tab ] - asset.BaseDef[ Tab ] )
               except : iFlow = 0
               try    : iPos  = log( 1 + rawFlow[ 0 ][ Tab ] )
               except : iPos  = 0

            startFlow[ Tab ] = f'\t\t\t\t\t d="M 0 {iFlow:.3f}'
            startPos[ Tab ]  = iPos

            # -----------------------------------------------------------------------
            # --- Always 10 couples per line ...
            j = 0
            for j in range( 0, int( PlotRow / tabMod ) ) :
               jFlow.append( '\t\t\t\t\t' )
               for i in range( j * tabMod + 1, j * tabMod + tabMod + 1 ) :
                  if f_Lin :
                     iFlow = rawFlow[ i ][ Tab ] - rawFlow[ i-1 ][ Tab ]
                  else :
                     iCur = 0
                     iPre = 0
                     try    : iCur = log( 1 + rawFlow[ i   ][ Tab ] - asset.BaseDef[ Tab ] )
                     except : pass
                     try    : iPre = log( 1 + rawFlow[ i-1 ][ Tab ] - asset.BaseDef[ Tab ] )
                     except : pass
                     iFlow = iCur - iPre

                  jFlow.append( f' l 1 {iFlow:.3f}' )

               jFlow.append( "\n" )
            # -----------------------------------------------------------------------

            if j != 0 : j += 1
            # -----------------------------------------------------------------------
            # --- Rest of the pairs ...
            if PlotRow % tabMod != 0 :
               jFlow.append( '\t\t\t\t\t' )
               for i in range( j * tabMod + 1, ( j * tabMod + PlotRow % tabMod ) + 1 ) :
                  if f_Lin :
                     iFlow = rawFlow[ i ][ Tab ] - rawFlow[ i-1 ][ Tab ]
                  else :
                     iCur = 0
                     iPre = 0
                     try    : iCur = log( 1 + rawFlow[ i   ][ Tab ] - asset.BaseDef[ Tab ] )
                     except : pass
                     try    : iPre = log( 1 + rawFlow[ i-1 ][ Tab ] - asset.BaseDef[ Tab ] )
                     except : pass
                     iFlow = iCur - iPre

                  jFlow.append( f' l 1 {iFlow:.3f}' )
            # -----------------------------------------------------------------------

            plotFlow[ Tab ] = ''.join( jFlow )
   # --------------------------------------------------------------------------------

   # --------------------------------------------------------------------------------
   # --- Init Counter ...
   if flowLines :
      if flowLines >= MaxFlow :
         tab_lf   = ( PlotRow + RowOffset ) % tabMod - 1
         tabCut   = ( PlotRow + RowOffset ) % cutMod
      else :
         tab_lf   = ( PlotRow + RowOffset ) % tabMod
         PlotRow += 1
         tabCut   = ( PlotRow + RowOffset ) % cutMod

   # --------------------------------------------------------------------------------
   # --- Export local Vaiable to global ...
   flow.PlotRow   = PlotRow
   flow.RowOffset = RowOffset
   flow.MaxFlow   = MaxFlow
   flow.startFlow = startFlow
   flow.plotFlow  = plotFlow
   flow.rawFlow   = rawFlow
   flow.maxAxis   = maxAxis
   flow.startPos  = startPos

   flow.initFlag  = initFlag
   flow.minFlag   = minFlag
   flow.maxFlag   = maxFlag

   flow.tab_lf    = tab_lf
   flow.tab_lsCut = tab_lsCut
   flow.tabMod    = tabMod
   flow.tabCut    = tabCut

   flow.cutMod    = cutMod

# -----------------------------------------------------------------------------------
# Add a new Row to plotData-Array. If necessary, increment RowOffset (Ring-Buffer)
# -----------------------------------------------------------------------------------
def Add() :

   # --------------------------------------------------------------------------------
   i  = ( flow.PlotRow + flow.RowOffset )     % flow.MaxFlow
   im = ( flow.PlotRow + flow.RowOffset - 1 ) % flow.MaxFlow
   ip = ( flow.PlotRow + flow.RowOffset + 1 ) % flow.MaxFlow

   # --------------------------------------------------------------------------------
   for j in asset.BaseListSMA :
      asset.ioSMA[j] = round( asset.Summe[j] / asset.SMAdepth, asset.rnd_ioSMA )

   # --------------------------------------------------------------------------------
   for Axis in iAxis :
      # -----------------------------------------------------------------------------
      if asset.ScalingAxis_Names[ Axis ] == None : continue

      if asset.AvrConf[ sKey[ Axis ] + 'ScaleType' ] == 'Lin' : f_Lin = True
      else                                                    : f_Lin = False

      # -----------------------------------------------------------------------------
      for j in asset.ScalingAxis_Names[ Axis ] :
         # --------------------------------------------------------------------------
         # --- Add "relative position" to Plot-Line ...
         if not flow.initFlag :
            # -----------------------------------------------------------------------
            # --- Add a new 'Start'-Line ....
            if f_Lin :
               startFlow = asset.ioSMA[ j ] - asset.BaseDef[ j ]
               startPos  = asset.ioSMA[ j ]
            else :
               try    : startFlow = log( 1 + flow.ioSMA[ j ] - asset.BaseDef[ j ] )
               except : startFlow = 0
               try    : startPos  = log( 1 + flow.ioSMA[ j ] )
               except : startPos  = 0

            flow.startFlow[ j ] = f'\t\t\t\t\t d="M 0 {startFlow:.3f}'
            flow.startPos[  j ] = startPos

         else :
            if flow.tab_lf == 0 :
               # --------------------------------------------------------------------
               # --- Add a new 'Value'-Line ...
               flow.plotFlow[ j ] += '\t\t\t\t\t'

            # -----------------------------------------------------------------------
            # --- Add new 'Value' to current Line ...
            if f_Lin :
               plotFlow =asset.ioSMA[ j ] - flow.rawFlow[ im ][ j ]
            else :
               try    : iCur = log( 1 + asset.ioSMA[ j ]        - asset.BaseDef[ j ] )
               except : iCur = 0
               try    : iPre = log( 1 + flow.rawFlow[ im ][ j ] - asset.BaseDef[ j ] )
               except : iPre = 0
               plotFlow = iCur - iPre

            flow.plotFlow[ j ] += f' l 1 {plotFlow:.3f}'

            if ( flow.tab_lf + 1 ) % flow.tabMod == 0 :
               # --------------------------------------------------------------------
               # --- Add a LineFeed (close) to current Line (prepare a new Line) ...
               flow.plotFlow[j] += '\n'

            if flow.tabCut == 0 :
               # --------------------------------------------------------------------
               # --- Remove "First" Line ...
               flow.plotFlow[  j ] = flow.plotFlow[j].split('\n', 1)[1]
               # --------------------------------------------------------------------
               # --- Add a new 'Start'-Line ...
               if f_Lin :
                  startFlow = flow.rawFlow[ ip ][ j ] - asset.BaseDef[ j ]
                  startPos  = flow.rawFlow[ ip ][ j ]
               else :
                  try    : startFlow = log( 1 + flow.rawFlow[ ip ][ j ] - asset.BaseDef[ j ] )
                  except : startFlow = 0
                  try    : startPos  = log( 1 + flow.rawFlow[ ip ][ j ] )
                  except : startPos  = 0

               flow.startFlow[ j ] = f'\t\t\t\t\t d="M 0 {startFlow:.3f}'
               flow.startPos[  j ] = startPos

         # --------------------------------------------------------------------------
         flow.rawFlow[ i ][ j ] = asset.ioSMA[ j ]
   # --------------------------------------------------------------------------------

   # --------------------------------------------------------------------------------
   # --- Add Values to Data-Log-File ...
   io.touch ( asset.f_FlowFlag )
   io.ioAdd ( ' '.join( '%.3f' % ( abs( x ) ) for x in asset.ioSMA ), asset.avrLogDataFile )
   io.ioRm  ( asset.f_FlowFlag )
   # --------------------------------------------------------------------------------

   # --------------------------------------------------------------------------------
   # --- Walk through AxisList (Axis 0 - 3) and get Max-Value ....
   for Axis in iAxis :
      Max = 0
      # -----------------------------------------------------------------------------
      # --- Get Max-Value of each Axis-Members ...
      if asset.ScalingAxis_Names[ Axis ] :
         for j in asset.ScalingAxis_Names[ Axis ] :
            if flow.rawFlow[ i ][ j ] > Max : Max = flow.rawFlow[ i ][ j ]

      flow.maxAxis[ i ][ Axis ] = Max

   # --------------------------------------------------------------------------------
   if not flow.minFlag :
      flow.PlotRow += 1
      if flow.PlotRow >= 2 : flow.minFlag = True
   else :
      if not flow.maxFlag and  flow.PlotRow < flow.MaxFlow :
         flow.PlotRow += 1
      else :
         flow.RowOffset = ( flow.RowOffset + 1 ) % flow.MaxFlow
         flow.maxFlag = True

   # --------------------------------------------------------------------------------
   if not flow.initFlag :
      flow.initFlag = True
   else :
      flow.tab_lf = ( flow.tab_lf + 1 ) % flow.tabMod

   flow.tabCut = ( flow.tabCut + 1 ) % flow.cutMod

   if not flow.tab_lsCut :
      if flow.tabCut == 0 :
         flow.tab_lsCut = True
         flow.cutMod    = flow.tabMod

   # --------------------------------------------------------------------------------
   if avr.AvrDebug and asset.oid5 : Sys.Out_EchoStat( asset.sys_DivCount )
   # --------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- RoundUp Max-Value and check compliance with the limits
# -----------------------------------------------------------------------------------
def RoundUp( Axis, Value ) :

   Max = int( Value ) * 10
   # --------------------------------------------------------------------------------
   # --- ( Remember: All Values are scaled by 10 )
   while switch( len( str( Max ) ) ) :
      if case( 1 ):
         Rnd =    1
         Max =    9
         break
      if case( 2 ):
         Rnd =   10
         break
      if case( 3 ):
         Rnd =   50
         break
      if case( 4 ):
         Rnd =  100
         break
      if case( 5 ):
         Rnd =  500
         break
      Rnd    = 1000
      break

   asset.ScalingAxis_Max[ Axis ] = int( (( int( Max / Rnd ) * Rnd ) + Rnd ) / 10 )

   # --------------------------------------------------------------------------------
   # --- Check compliance with the limits ...
   # --------------------------------------------------------------------------------
   if asset.ScalingAxis_Max[Axis] >  asset.ScalingAuto_Max[Axis] : asset.ScalingAxis_Max[Axis] = asset.ScalingAuto_Max[Axis]
   if asset.ScalingAxis_Max[Axis] <  asset.ScalingAxis_Def[Axis] : asset.ScalingAxis_Max[Axis] = asset.ScalingAxis_Def[Axis]
   if asset.ScalingAxis_Max[Axis] <= asset.ScalingAuto_Min[Axis] : asset.ScalingAxis_Max[Axis] = asset.ScalingAuto_Min[Axis]

   if asset.ScalingAxis_Max[Axis] -  asset.ScalingAxis_Min[Axis] <= 2 : asset.ScalingAxis_Max[Axis] += 1

# -----------------------------------------------------------------------------------
# --- The plot values are relative. We therefore need the right starting value ...
# -----------------------------------------------------------------------------------
def ChangeMin() :

   try  :
      if not flow.startPos : return
   except :
      return

   for Axis in iAxis :
      # -----------------------------------------------------------------------------
      if asset.ScalingAxis_Names[ Axis ] == None : continue

      if asset.AvrConf[ sKey[ Axis ] + 'ScaleType' ] == 'Lin' : f_Lin = True
      else                                                    : f_Lin = False

      # -----------------------------------------------------------------------------
      for j in asset.ScalingAxis_Names[ Axis ] :
         if not flow.startPos[ j ] : continue

         # --------------------------------------------------------------------------
         if f_Lin :
            startFlow = flow.startPos[ j ] - asset.BaseDef[ j ]
         else :
            # -----------------------------------------------------------------------
            # --- Remember :
            # --- Always add an offset of 1 because values less than 1 produce incorrect results
            try    : startFlow = log( 1 + flow.startPos[ j ] - asset.BaseDef[ j ] )
            except : startFlow = 0

         flow.startFlow[ j ] = f'\t\t\t\t\t d="M 0 {startFlow:.3f}'

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
