#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#


# -----------------------------------------------------------------------------------
# --- HTTP-Statuscode :
# --> https://de.wikipedia.org/wiki/HTTP-Statuscode
# -----------------------------------------------------------------------------------
Codes = {
   # --- 1xx – informational response :
   100: 'Continue',
   101: 'Switching Protocols',
   102: 'Processing',
   103: 'Early Hints (Checkpoint)',
   122: 'Request URL too long',                    # Not officially listed

   # --- 2xx – Successful operation :
   200: 'Ok',
   201: 'Created',
   202: 'Accepted',
   203: 'Non-Authoritative Information',
   204: 'No Content',
   205: 'Reset Content',
   206: 'Partial Content',
   207: 'Multi-Status',
   208: 'Already Reported',
   218: 'This is fine',                            # Unofficial code (Apache Web Server)
   226: 'IM Used',

   # --- 3xx - Redirection :
   300: 'Multiple Choices',
   301: 'Moved Permanently',
   302: 'Found (Moded Temporarily',
   303: 'See Other',
   304: 'Not Modified',
   305: 'Use Proxy',
   306: 'Switch Proxy',
   307: 'Temporary Redirect',
   308: 'Permanent Redirect',

   # --- 4xxx - Client Error :
   400: 'Bad Request',
   401: 'Unauthorized',
   402: 'Payment Required',
   403: 'Borbidden',
   404: 'Not Found',
   405: 'Method  Not Allowed',
   406: 'Not Acceptable',
   407: 'Proxy Authentication Required',
   408: 'Request  Timeout',
   409: 'Conflict',
   410: 'Gone',
   411: 'Length Required',
   412: 'Precondition Failed',
   413: 'Payload Too Large',
   414: 'URI Too Long',
   415: 'Unsupported Media Type',
   416: 'Range Not Satisfiable',
   417: 'Expectation Failed',
   418: 'I\'m a teapot',
   419: 'Page Expired',                            # Unofficial code (Laravel Framework)
   420: 'Method Failure / Enhance Your Calm',      # Unofficial code (Spring Framework & Twitter)
   421: 'Misdirected Request',
   422: 'Unprocessable Entity',
   423: 'Locked',
   424: 'Failed Dependency',
   425: 'Too Early',
   426: 'Upgrade Required',
   428: 'Precondition Required',
   429: 'Too Many Requests',
   430: 'Request Header Fields Too Large',         # Unofficial code (Shopify)
   431: 'Request Header Fields Too Large',
   440: 'Login Time-out',                          # Unofficial code (Microsoft IIS)
   444: 'No Response',                             # Unofficial code (nginx)
   449: 'Retry With',                              # Unofficial code (Microsoft IIS)
   450: 'Blocked By Windows Parental Controls',    # Unofficial code (Microsoft)
   451: 'Unavailable For Legal Reasons',
   460: 'Client closed the connection',            # Unofficial code (AWS Elastic Load Balancer)
   463: 'Received an X-Forwarded-For request',     # Unofficial code (AWS Elastic Load Balancer)
   494: 'Request header too large',                # Unofficial code (nginx)
   495: 'SSL Certificate Error',                   # Unofficial code (nginx)
   496: 'SSL Certificate Required',                # Unofficial code (nginx)
   497: 'HTTP Request Sent to HTTPS Port',         # Unofficial code (nginx)
   498: 'Invalid Token',                           # Unofficial code (Esri)
   499: 'Token Required',                          # Unofficial code (Esri)

   # --- 5xxx - Server Error :
   500: 'Internal Server Error',
   501: 'Not Implemented',
   502: 'Bad Gateway',
   503: 'Service Unavailable',
   504: 'Gateway Timeout',
   505: 'HTTP Version Not Supported',
   506: 'Variant Also Negotiates',
   507: 'Insufficient Storage',
   508: 'Loop Detected',
   509: 'Bandwidth Limit Exceeded',                # Unofficial code (Apache Web Server)
   510: 'Not Extended',
   511: 'Network Authentication Required',
   520: 'Web Server Returned an Unknown Error',    # Unofficial code (Cloudflare)
   521: 'Web Server Is Down',                      # Unofficial code (Cloudflare)
   522: 'Connection Timed Out',                    # Unofficial code (Cloudflare)
   523: 'Origin Is Unreachable',                   # Unofficial code (Cloudflare)
   524: 'A Timeout Occurred',                      # Unofficial code (Cloudflare)
   525: 'SSL Handshake Failed',                    # Unofficial code (Cloudflare)
   526: 'Invalid SSL Certificate',                 # Unofficial code (Cloudflare & Cloud Foundry's)
   527: 'Railgun Error',                           # Unofficial code (Cloudflare)
   529: 'Site is overloaded',                      # Unofficial code (Qualys)
   530: 'Site is frozen',                          # Unofficial code (Pantheon)
   598: 'Network read timeout error',              # Unofficial code
   561: 'Unauthorized',                            # Unofficial code (AWS Elastic Load Balancer)

   # --- 9xx - Special codes :
   998: 'Malformatted Meta-Header',                # Unofficial code (BigRonin)
   999: 'Unknown HTML response code',              # Unofficial code (BigRonin)
}

# -----------------------------------------------------------------------------------
# --- HTML response code dispatcher :
# -----------------------------------------------------------------------------------
def getString( iCode=999 ) :

   try    : return Codes[ iCode ]
   except : return Codes[ 999 ]

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
