# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import time
import json
import subprocess

import asset
import basics   as io
from   lib      import switch, case
import lib

# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Init Memory / Set Defaults ...
# -----------------------------------------------------------------------------------
def init( opt=False ):

   while switch( asset.AvrConf[ 'RamSelectDevice' ] ):
      if case( 'none' ):
         asset.AvrConf[ 'PlotMemoryOn' ] = 'No'
         break
      if case( 'reserved' ):
         break
      if case( 'used' ):
         break
      if case( 'buffer' ):
         break
      if case( 'cached' ):
         break
      if case( 'free' ):
         break
      if case( 'utilization' ):
         break
      # -----------------------------------------------------------------------------
      # --- Default Settings ...
      asset.AvrConf[ 'RamSelectDevice' ] = 'none'
      asset.AvrConf[ 'PlotMemoryOn'    ] = 'No'
      break

   # --------------------------------------------------------------------------------
   if os.path.isfile( asset.sysMemorySizeFile ) and int( time.time() ) <= 3600 + io.ioStat( asset.sysMemorySizeFile ):
      Memory = int( io.ioRead( asset.sysMemorySizeFile ) )
   else :
      Memory = GetSystemRam()
      if Memory : io.ioWrite( str( Memory ), asset.sysMemorySizeFile )

   # --------------------------------------------------------------------------------
   if Memory :
      asset.sys_RamSizeKB = int( Memory )
      asset.sys_RamSizeGB = int( Memory / 1024 / 1024 )
   else :
      # -----------------------------------------------------------------------------
      # --- Normally this should never happen ...
      asset.sys_RamSizeKB = False
      asset.sys_RamSizeGB = False
      asset.AvrConf[ 'RamSelectDevice' ] = 'none'
      asset.AvrConf[ 'PlotMemoryOn'    ] = ''

# -----------------------------------------------------------------------------------
# --- Get total amount installed physical RAM ...
# -----------------------------------------------------------------------------------
def GetSystemRam() :

   if not bool( asset.SynoInfo ) : asset.SynoInfo = lib.FileConfig( asset.sysSynoConfig )

   # --------------------------------------------------------------------------------
   # --- Try to read the size of the installed memory from "synoinfo.conf" ...
   if asset.SynoInfo.get( 'mem_factorymodel', 'No' ) == 'yes' and asset.SynoInfo.get( 'mem_max_mb', False ) and asset.SynoInfo.get( 'mem_min_mb', False ) :
      return int( asset.SynoInfo[ 'mem_max_mb' ] ) * 1024

   # --------------------------------------------------------------------------------
   # --- Try to get the size of the installed memory via "dmidecode" ...
   if os.path.isfile( '/sbin/dmidecode' ) :
      Info = lib.getCMD( 'grep "Size:" <<< "$(dmidecode --type memory)" | awk \'{print $2*1024}\''  ).strip().split( '\n' )
      iCount = 0
      for i in Info : iCount += int( i )
      return iCount

   # --------------------------------------------------------------------------------
   # --- Last chance and slowest too, try to get the size of the installed memory via "synowebapi" ...
   try :
      Memory, err = subprocess.Popen( [ '/usr/syno/bin/synowebapi', 'synowebapi', '--exec', 'api=SYNO.Core.System.Utilization', 'method=get', 'version=1', 'resource=[\"memory\"]' ], stdout=subprocess.PIPE, stderr=subprocess.PIPE ).communicate()
      Memory = json.loads( Memory.decode("utf-8") )
      Memory = Memory['data']['memory']['memory_size']
      return Memory
   except :
      pass

   # --------------------------------------------------------------------------------
   return False

# -----------------------------------------------------------------------------------
# --- RAM usage :
# -----------------------------------------------------------------------------------
def Get_RAM():

   # --------------------------------------------------------------------------------
   if not asset.sys_RamSizeKB :
      asset.SMA[asset.LastSMA][asset.TabRAM] = 0
      return

   # --------------------------------------------------------------------------------
   MemInfo = dict()

   sList = io.ioRead( '/proc/meminfo' ).split('\n')

   for line in sList :
      key, value = ''.join( line.rstrip(' kB').split() ).split(':')
      MemInfo[key] = int(value)

   try:    MemInfo["Cached"] = asset.sys_MemInfoCached
   except: MemInfo["Cached"] = int( lib.WalkV2( asset.snmpCommunity, asset.snmpLocalHost, '1.3.6.1.4.1.2021.4.15.0' ).strip().split(' ')[0] ) 

   RamTotal = asset.sys_RamSizeKB

   while switch( asset.AvrConf["RamSelectDevice"] ):
      if case( 'none' ):
         RAM = 0
         break
      if case( 'reserved' ):
         RAM = RamTotal - MemInfo["MemTotal"]
         break
      if case( 'used' ):
         RAM = MemInfo["MemTotal"] - MemInfo["MemFree"] - MemInfo["Buffers"] - MemInfo["Cached"]
         break
      if case( 'buffer' ):
         RAM = MemInfo["Buffers"]
         break
      if case( 'cached' ):
         RAM = MemInfo["Cached"]
         break
      if case( 'free' ):
         RAM = MemInfo["MemFree"]
         break
      if case( 'utilization' ):
         RAM      = MemInfo["MemTotal"] - ( MemInfo["MemFree"] + MemInfo["Buffers"] + MemInfo["Cached"] )
         RamTotal = RAM + MemInfo["MemFree"] + MemInfo["Buffers"] + MemInfo["Cached"]
         break
      # -----------------------------------------------------------------------------
      # --- Default Setting :
      asset.AvrConf["RamSelectDevice"] = 'utilization'
      RAM = MemInfo["MemTotal"] - ( MemInfo["MemFree"] + MemInfo["Buffers"] + MemInfo["Cached"] )
      RamTotal = RAM + MemInfo["MemFree"] + MemInfo["Buffers"] + MemInfo["Cached"]
      break

   asset.SMA[asset.LastSMA][asset.TabRAM] = round( RAM * 100 / RamTotal, asset.rnd_SMA )

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
