# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import base64
import urllib.request as     Request
import urllib.parse   as     Parse
from   urllib.error   import URLError
import json

import asset
from   lib   import switch, case
import lib
import Sys

# -----------------------------------------------------------------------------------
# --->> https://docs.pushbullet.com/#pushbullet-api
# --->> https://docs.pushbullet.com/v7/
# -----------------------------------------------------------------------------------
# --->> https://github.com/samwell/usr-local-bin/blob/master/PushBullet.py
# -----------------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Prepare Authorization-Header ...
# -----------------------------------------------------------------------------------
def GetAuth( Token = False ) :

   if not Token : Token = asset.AvrConf["StatusPushToken"]

   # --------------------------------------------------------------------------
   # --- Encode PushBullet Access-Token ...
   return { 'Authorization' : "Basic " + str( base64.b64encode( Token.encode("utf-8") ), "utf-8" ) }

# -----------------------------------------------------------------------------------
# --- Send notification via Pushbullet ...
# -----------------------------------------------------------------------------------
def Note( sTitel, sBody ) :

   Data = { 'type'               : 'note',
            'source_device_iden' : asset.AvrConf['StatusPushDeviceIden'],
            'title'              : sTitel,
            'body'               : sBody }

   if asset.AvrConf["StatusPushDeviceID"] and asset.AvrConf["StatusPushDeviceID"] != '9999' :
      Data['device_iden'] = asset.AvrConf["StatusPushDeviceID"]

   Data = Parse.urlencode( Data, doseq = True )
   Ret  = PushReqest( Request.Request( asset.u_PushBullet_Note, data = bytes( Data, "utf-8"), headers = GetAuth() ), 'Note' )

# -----------------------------------------------------------------------------------
# --- Send a Test (Push) notification via Pushbullet (GUI) ...
# -----------------------------------------------------------------------------------
def Test( DevIden = False ) :

   if not asset.AvrConf : asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )

   import Status
   Status.init()
   Sys.GetSystemInfo()

   # --------------------------------------------------------------------------
   # --- Send (push) welcome notification ...
   if not asset.msg : asset.msg = lib.InitLanguage( 'admin', asset.avrLangFile )

   Data = { 'type'               : 'note',
            'source_device_iden' : asset.AvrConf['StatusPushDeviceIden'],
            'title'              : Status.MailHost + ' - ' + asset.msg['AlertPushNotifyTest'],
            'body'               : asset.msg['AlertPushTest'] }

   if DevIden and DevIden != '9999' :
      Data['device_iden'] = DevIden

   Data = Parse.urlencode( Data, doseq = True )
   Ret  = PushReqest( Request.Request( asset.u_PushBullet_Note, data = bytes( Data, "utf-8"), headers = GetAuth() ), 'Note' )

   if not Ret : return False
   else       : return True

# --------------------------------------------------------------------------
# --- Check whether there is a token and if necessary re-register PushBullet ...
# --- (Is only run once after an update.)
# --------------------------------------------------------------------------
def ReInit () :

   if not asset.AvrConf : asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )

   if not asset.AvrConf["StatusPushToken"] :
      asset.AvrConf["StatusPushRegistered"] = "No"
      asset.AvrConf["StatusPushDeviceID"]   = "9999"
   else :
      lib.TimerStart()

      if Init ( asset.AvrConf["StatusPushToken"], False ) :
         asset.AvrConf["StatusPushRegistered"] = "Yes"
         msg = asset.msg['msgPushRegistered']
      else :
         asset.AvrConf["StatusPushRegistered"] = "No"
         msg = asset.msg['errPushRegistered']

      if not asset.AvrConf["StatusPushDeviceID"] :
         asset.AvrConf["StatusPushDeviceID"] = "9999"

      lib.Error( msg )

      lib.TimerStop ()
      lib.Echo ( 1, '| ' + msg )

   return

# --------------------------------------------------------------------------
# --- Init / prepare PushBullet settings ...
# --------------------------------------------------------------------------
def Init ( Token, Verbose = True ) :

   Auth   = GetAuth( Token )

   # --------------------------------------------------------------------------
   # --- Get User-Informations ...
   Ret = PushReqest( Request.Request( asset.u_Pushbullet_User, headers = Auth ), 'User' )
   if not Ret : return False # --- If failed ...

   if not asset.AvrConf : asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )
   if not Sys.avrInfo   : Sys.GetSystemInfo()

   Data     = json.loads( Ret )
   UserName = Data['name']
   DevName  = Sys.avrInfo['Hostname'] + "-" + Sys.avrInfo['NAS_Type']

   # --------------------------------------------------------------------------
   # --- Get Device-List ...
   Ret = PushReqest( Request.Request( asset.u_PushBullet_Dev, headers = Auth ), 'Device-List' )
   if not Ret : return False    # --- If failed

   DevIden = False
   idenGUI = '\"All\"'
   idenRAW = '9999'
   Data = json.loads( Ret )

   for dev in Data['devices'] :
      if dev['active'] :
         if dev['nickname'] == DevName :
            DevIden = dev['iden']
         else :
            idenGUI += ' \"' + dev['nickname'] + '\"'
            idenRAW +=   ' ' + dev['iden']

   if not DevIden :
      # --------------------------------------------------------------------------
      # --- Prepare Device-Creation ...
      Own = { 'nickname'     : DevName ,
              'model'        : Sys.avrInfo['NAS_Product']            ,
              'manufacturer' : 'Synology'                            ,
              'app_version'  : 2108                                  ,
              'type'         : 'nas'                                 ,
              'icon'         : 'desktop'                             ,
              'has_sms'      : False }

      Data = Parse.urlencode( Own, doseq = True )

      Ret = PushReqest( Request.Request( asset.u_PushBullet_Dev, data = bytes( Data, "utf-8"), headers = Auth ), 'Create-Device' )
      if not Ret : return False    # --- If failed

      Data = json.loads( Ret )
      DevIden = Data['iden']

   if Verbose :
      # --------------------------------------------------------------------------
      # --- Send (push) welcome notification ...
      if not asset.msg : asset.msg = lib.InitLanguage( 'admin', asset.avrLangFile )

      Data = { 'type'               : 'note',
               'source_device_iden' : DevIden,
               'title'              : DevName,
               'body'               : asset.msg['AlertPushWelcome'] }

      Data = Parse.urlencode( Data, doseq = True )

      Ret  = PushReqest( Request.Request( asset.u_PushBullet_Note, data = bytes( Data, "utf-8"), headers = Auth ), 'Note' )
      if not Ret : return False    # --- On Error : Return "False" ....

   # --------------------------------------------------------------------------
   # --- Store / save PushBullet Settings ...
   asset.AvrConf['StatusPushToken']      = Token
   asset.AvrConf['StatusPushUserName']   = UserName
   asset.AvrConf['StatusPushDeviceName'] = DevName
   asset.AvrConf['StatusPushDeviceIden'] = DevIden
   asset.AvrConf['StatusPushDeviceGUI']  = idenGUI
   asset.AvrConf['StatusPushDeviceRAW']  = idenRAW

   # --------------------------------------------------------------------------
   return True

# --------------------------------------------------------------------------
# --- Performe an PushBulet-Request :
# --------------------------------------------------------------------------
def PushReqest( request, msg ) :

   try:
      res = Request.urlopen( request )
      if res.getcode() == 200 :
         return str( res.read(), "utf-8" )
      else :
         res = str( res.getcode() )

   except URLError as e:
      res = str( e.code )

   except Exception as e :
      lib.Error( 'CLI: PushBullet UrlRequest for \"{0}\" faild.'.format( msg ) )
      lib.Error( 'CLI: Exception = \"{0}\"'.format( e ) )
      res = '500'

   # --------------------------------------------------------------------------
   if not res.isnumeric() : return json.loads( res )

   # --------------------------------------------------------------------------
   # --- Examine BushBullet Error-Code - Provide Full Error-Text :
   while switch( int( res ) ) :
      if case( 400 ) :
         err = "Pretty sure a parameter was missing."
         break
      if case( 401 ) :
         err = "No valid access token provided."
         break
      if case( 402 ) :
         err = "Request was correct, but failed."
         break
      if case( 403 ) :
         err = "The access token is not valid for that request"
         break
      if case( 404 ) :
         err = "The requested item doesn't exist."
         break
      if case( 429 ) :
         err = "You have been ratelimited for making too many requests."
         break
      if case( 500 ) :
         err = "Unknown exceptional situation occurred."
         break
      err = "Something went wrong on Pushbullet's side."
      break

   lib.Error( 'CLI: PushBullet {0} request faild : {1}'.format( msg, res ) )
   lib.Error( 'CLI: {0}'.format( err ) )
   return

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
