# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import sys
import subprocess
import hashlib
import tarfile
import urllib.request
import shutil
import time

import asset
import basics   as io
import Html
import lib

# --- '/etc/synoinfo.conf'
# --- Controled by User via Control Panel
# --- fan_config_type_internal="full"     = Full-speed mode
# --- fan_config_type_internal="high"     = Cool mode
# --- fan_config_type_internal="low"      = Quiet mode
# --- fan_config_type_internal="low_stop" = Low-power mode
# -----------------------------------------------------------------------------------
# --- Global options ...
scemd = sys.modules[ __name__ ]

if asset.f_Dsm7 : RestartScemd = 'systemctl restart scemd.service > /dev/null 2>&1'
else            : RestartScemd = 'synoservicectl --restart scemd > /dev/null 2>&1'
# -----------------------------------------------------------------------------------


# -----------------------------------------------------------------------------------
# --- Class to handle / manage scemd (Fan) control ...
# -----------------------------------------------------------------------------------
class Fan ( ) :

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   # --- Nothing works without corret initialization ...
   def __new__( cls, xml=False, init=False ) :

      try    : scemd.Conf['FanCurrentHash']
      except : scemd.Conf = lib.ConfigRead( asset.avrConfigFile )

      if not xml and not init :
         # --------------------------------------------------------------------------
         if io.check( asset.f_ExcludeSCEMD ) :
            io.ioRm(  asset.f_ScemdInit  )
            io.touch( asset.f_ScemdError )
            lib.Error ( 'SYS: SCEMD exception : Fan control is not supported on this DiskStation' )
            del scemd.Conf
            return None

         # --------------------------------------------------------------------------
         if io.check( asset.f_VirtualDSM ) : xmlFile  = asset.ScemdVrmFile
         else                              : xmlFile  = asset.ScemdFile

         # --------------------------------------------------------------------------
         # --- Check important files ...
         if ( not io.check( xmlFile )               or   # << "/usr/syno/etc.defaults/scemd.xml"
              not io.check( asset.ScemdPatFile )    or   # << ETC + "/log/scemd.xml"
              not io.check( asset.ScemdMd5File ) ):      # << ETC + "/log/scemd.md5"

            io.ioRm(  asset.f_ScemdInit  )
            io.touch( asset.f_ScemdError )
            lib.Error ( 'SYS: SCEMD exception : Init failed due missing File(s)' )
            del scemd.Conf
            return None

         # --------------------------------------------------------------------------
         # --- Check MD5 Hashes ...
         with open( xmlFile, 'rb' ) as f :
            xmlHash = hashlib.md5( f.read() ).hexdigest()

         if xmlHash != scemd.Conf['FanCurrentHash'] :
            if scemd.Conf['FanProtocolOn'] == 'Yes': lib.Error ( 'SYS: SCEMD : Unexpected changes detected.' )
            io.touch( asset.f_ScemdChanged )

         with open( asset.ScemdPatFile, 'rb' ) as f :
            PatHash = hashlib.md5( f.read() ).hexdigest()

         with open( asset.ScemdMd5File, 'r', encoding='utf-8' ) as f :
            Md5Hash = f.read()

         if PatHash != Md5Hash or PatHash != scemd.Conf['FanScemdHash'] :
            io.ioRm(  asset.f_ScemdInit  )
            io.touch( asset.f_ScemdError )
            lib.Error ( 'SYS: SCEMD exception : Init failed due wrong Hash' )
            del scemd.Conf
            return None

      # -----------------------------------------------------------------------------
      return super( Fan, cls ).__new__( cls )

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def __init__( self, xml=False, init=False ) :
      super( Fan, self ).__init__()

      # -----------------------------------------------------------------------------
      if xml :
         self.xmlFile  = xml
      else :
         if io.check( asset.f_VirtualDSM ) :
            self.xmlFile  = asset.ScemdVrmFile
         else :
            self.xmlFile  = asset.ScemdFile

      # -----------------------------------------------------------------------------
      self.xmlLines  = []
      self.xmlScemd  = dict()
      self.xmlCur    = dict()
      self.xmlMode   = False
      self.xmlHash   = False
      self.LengthErr = False
      self.xmlFatal  = False
      self.AdtCount  = False
      self.PwmCount  = False

      if scemd.Conf['FanProtocolOn']     == 'Yes' : self.xmlDebug = True
      else                                        : self.xmlDebug = False

      if scemd.Conf['FanConfirmationOn'] == 'Yes' : self.xmlWait  = True
      else                                        : self.xmlWait  = False

      if self.xmlFile and os.path.isfile( self.xmlFile ) :
         self.xmlLines = io.ioRead( self.xmlFile ).strip().split('\n')

         try :
            self.xmlScemd, self.xmlMode, self.xmlCur = self.InitScemd( self.xmlLines )
         except :
            self.xmlLines = []
            self.xmlScemd = dict()
            self.xmlCur   = dict()
            self.xmlMode  = False
            self.xmlHash  = False
            io.touch( asset.f_ScemdError )
            self.xmlFatal = 'SYS: SCEMD exception : "scemd.xml" is faulty and could not be read'
            lib.Error( self.xmlFatal )
            return None

      else :
         io.touch( asset.f_ScemdError )
         self.xmlFatal = 'SYS: SCEMD exception : Fatal Error - File "{0}" not found'.format( self.xmlFile )
         lib.Error( self.xmlFatal )
         return None
         
   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def Save( self ) :

      self.LengthErr = False

      try :
         # --------------------------------------------------------------------------
         io.ioWrite( '\n'.join( self.xmlLines ) + '\n', self.xmlFile, opt=False )

         with open( self.xmlFile, 'rb' ) as f :
            self.xmlHash = hashlib.md5( f.read() ).hexdigest()

         # --------------------------------------------------------------------------
         if not io.check( asset.f_VirtualDSM ) and not io.check( asset.f_ExcludeSCEMD ) and not io.check( asset.f_ScemdError ) :
            if self.xmlWait : lib.getCMD( RestartScemd        )
            else            : lib.getCMD( RestartScemd + ' &' )

         # --------------------------------------------------------------------------
         io.ioRm( asset.f_ScemdChanged )
         return True
      except :
         lib.Error ( 'SYS: SCEMD exception : Failed to save Fan settings' )
         return False

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def Update( self, Conf=dict() ) :

      # -----------------------------------------------------------------------------
      # --- Update Debug-Flag ...
      if Conf['FanProtocolOn']     == 'Yes' : self.xmlDebug = True
      else                                  : self.xmlDebug = False

      if Conf['FanConfirmationOn'] == 'Yes' : self.xmlWait  = True
      else                                  : self.xmlWait  = False

      # -----------------------------------------------------------------------------
      with open( self.xmlFile, 'rb' ) as f :
         if self.xmlHash != hashlib.md5( f.read() ).hexdigest() :
            changed = True
         else :
            changed = self.LengthErr or io.check( asset.f_ScemdChanged )

      # -----------------------------------------------------------------------------
      if self.xmlMode == 'ADT' :
         ADT = False

         for Mode in asset.ScemdModeList :
            sMode = Mode + 'ADT'
            sName = 'ADT' + Mode
            if not sMode in self.xmlScemd  or not self.xmlScemd[sMode] : continue

            for sDev in asset.ScemdDevList :
               if not sDev in self.xmlScemd[sMode] : continue

               ADT = True

               iLines = sorted( [ int( x ) for x in self.xmlScemd[sMode][sDev].keys() ] )
               iCount = len( iLines )

               # --------------------------------------------------------------------
               if not sName + sDev + '_C' in Conf : continue

               # --------------------------------------------------------------------
               if iCount != len( Html.Split( Conf[sName + sDev + '_C'] ) ) : return 'ADT entries do not match'

               try :
                  # -----------------------------------------------------------------
                  C = Conf[sName + sDev + '_C'].split(' ')
                  S = Conf[sName + sDev + '_S'].split(' ')

                  for a in range( 0, iCount ) :

                     sJson   = self.xmlScemd[sMode][sDev][ str( iLines[a] ) ]
                     JsonNew = False
                     oLine   = self.xmlLines[ iLines[a] ]

                     # --------------------------------------------------------------
                     if C[a] != sJson['limit'] :
                        Mask = self.xmlLines[ iLines[a] ].split( '>', 1 )[-1].split( '<', 1 )[0]
                        if not Mask.strip() or not Mask.isnumeric() : return 'ADT value Limit ">{0}<" is faulty'.format( Mask )

                        Mask = '>{0}<'.format( Mask )
                        New  = '>{0}<'.format( C[a] )

                        self.xmlLines[ iLines[a] ] = self.xmlLines[ iLines[a] ].replace( Mask, New )
                        changed = True
                        JsonNew = True

                     # --------------------------------------------------------------
                     if S[a] != sJson['fan_speed'][0:2] :
                        Mask = self.xmlLines[ iLines[a] ].split( 'fan_speed="', 1 )[-1].split( '"', 1 )[0]
                        if len( Mask ) != 7 :
                           if len( Mask ) == 6 and Mask[0:1].isnumeric() :
                              New  = 'fan_speed="{0}{1}"'.format( S[a], Mask[1:] )
                           else :
                              return 'ADT fan_speed="{0}" is faulty'.format( Mask )
                        else :
                           New  = 'fan_speed="{0}{1}"'.format( S[a], Mask[2:] )

                        Mask = 'fan_speed="{0}"'.format( Mask )
                        self.xmlLines[ iLines[a] ] = self.xmlLines[ iLines[a] ].replace( Mask, New )
                        changed = True
                        JsonNew = True

                     # --------------------------------------------------------------
                     if JsonNew and self.xmlDebug :
                        lib.Error ( 'CGI: SCEMD Content changed : {0} - DUAL_MODE_{1} - Line {2:2}'.format( self.xmlMode, Mode, iLines[a] + 1 ) )
                        lib.Error ( 'CGI: Old : {0}'.format( oLine.strip() ) )
                        lib.Error ( 'CGI: New : {0}'.format( self.xmlLines[ iLines[a] ].strip() ) )

               except :
                  return 'Failed to update ADT settings'

         # --------------------------------------------------------------------------
         if not ADT : return 'Missing ADT settings'
         else       : return changed

      # -----------------------------------------------------------------------------
      if self.xmlMode == 'PWM' :
         PWM = False

         for Mode in asset.ScemdModeList :
            sMode = Mode + 'PWM'
            sName = 'PWM' + Mode
            if not sMode in self.xmlScemd  or not self.xmlScemd[sMode] or not sName + '_S' in Conf : continue

            # -----------------------------------------------------------------------
            PWM = True

            try :
               # --------------------------------------------------------------------
               # --- First check 'pwm_duty_low' & 'pwm_duty_high' ...
               iLine = [ int( x ) for x in self.xmlScemd[sMode]['duty'].keys() ][0]
               S = Conf[sName + '_S'].split(' ')

               sJson   = self.xmlScemd[sMode]['duty'][ str( iLine ) ]
               JsonNew = False
               oLine   = self.xmlLines[ iLine ]

               if sJson['pwm_duty_low'] != S[0] :
                  self.ChangePWM( 'pwm_duty_low',  iLine, S[0] )
                  changed = True
                  JsonNew = True

               if sJson['pwm_duty_high'] != S[1] :
                  self.ChangePWM( 'pwm_duty_high', iLine, S[1] )
                  changed = True
                  JsonNew = True

               # --------------------------------------------------------------------
               if JsonNew and self.xmlDebug :
                  lib.Error ( 'CGI: SCEMD Content changed : {0} - DUAL_MODE_{1} - Line {2:2}'.format( self.xmlMode, Mode, iLine + 1 ) )
                  lib.Error ( 'CGI: Old : {0}'.format( oLine.strip() ) )
                  lib.Error ( 'CGI: New : {0}'.format( self.xmlLines[ int(iLine) ].strip() ) )

            except :
               return 'Failed to update "PWM {0}" settings'.format( Mode )

            # -----------------------------------------------------------------------
            # --- Next check Devices and Alerts ...
            for sDev in asset.ScemdDevList :
               if not sDev in self.xmlScemd[sMode] or not self.xmlCur[ sName + sDev + '_A' ] : continue

               try :
                  iLine = [ int( x ) for x in self.xmlScemd[sMode][sDev].keys() ][0]
                  C = Conf[sName + sDev + '_C'].split(' ')
                  A = self.xmlCur[sName + sDev + '_A'].split(' ')

                  sJson   = self.xmlScemd[sMode][sDev][ str( iLine ) ]
                  JsonNew = False
                  oLine   = self.xmlLines[ iLine ]

                  for a in range( 0, len( A ) ) :
                     sID = [key for key, value in asset.ScemdPwmShort.items() if value == A[a] ][0]

                     if sJson[ sID ] == C[a] : continue

                     self.ChangePWM( sID, iLine, C[a] )
                     changed = True
                     JsonNew = True

                  # -----------------------------------------------------------------
                  if JsonNew and self.xmlDebug :
                     lib.Error ( 'CGI: SCEMD Content changed : {0} - DUAL_MODE_{1} - Line {2:2}'.format( self.xmlMode, Mode, iLine + 1 ) )
                     lib.Error ( 'CGI: Old : {0}'.format( oLine.strip() ) )
                     lib.Error ( 'CGI: New : {0}'.format( self.xmlLines[ int(iLine) ].strip() ) )

               except :
                  return 'Failed to update "PWM {0} {1}" settings'.format( Mode, sDev )

         # --------------------------------------------------------------------------
         if not PWM : return 'Missing PWM settings'
         else       : return changed

      # -----------------------------------------------------------------------------
      return 'Mode "{0}" does not match the expected mode "{1}"'.format( self.xmlMode, Conf['FanSysMode'] )

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def ChangePWM( self, sID, iLine, sNew ) :
      name = '{0}'.format( sID )
      mask = name + '="{0}"'.format( self.xmlLines[ iLine ].split( name, 1 )[-1][2:-1].split( '"', 1 )[0] )
      self.xmlLines[ iLine ] = self.xmlLines[ iLine ].replace( mask, name + '="{0}"'.format( sNew ) )

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def InitScemd( self, Source ) :
      Scemd = dict()
      Cur   = dict()

      Mode = self.Mode( Source )

      if Mode == 'None' : return Scemd, Mode, Cur

      # -----------------------------------------------------------------------------
      for sID in asset.ScemdModeList :
         Scemd[sID] = self.GetSection( sID, Source )

         if Scemd[sID] :
            # -----------------------------------------------------------------------
            # --- Get "PWM" -Values ...
            Scemd[sID + 'PWM' ] = self.GetPwm( Scemd[sID] )

            # -----------------------------------------------------------------------
            # --- Get "ADT" -Values ...
            Scemd[sID + 'ADT' ] = self.GetAdt( Scemd[sID] )

      # -----------------------------------------------------------------------------
      Cur = self.GetCurrent( Scemd )
      Cur['FanSysMode']     = Mode
      Cur['FanDivPWMcount'] = str( self.PwmCount )
      Cur['FanDivADTcount'] = str( self.AdtCount )

      with open( self.xmlFile, 'rb' ) as f :
         self.xmlHash = hashlib.md5( f.read() ).hexdigest()

      # -----------------------------------------------------------------------------

      return Scemd, Mode, Cur

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def GetCurrent( self, Scemd ) :
      self.PwmCount  = 0
      self.AdtCount  = 0
      Conf = dict()

      for Mode in asset.ScemdModeList :
         sMode = Mode + 'PWM'
         if sMode in Scemd and Scemd[sMode] :
            # -----------------------------------------------------------------------
            iLine = [ x for x in Scemd[ sMode ]['duty'].keys() ][0]

            Conf['PWM' + Mode + '_S'] = '{0} {1}'.format( Scemd[ sMode ]['duty'][ iLine ]['pwm_duty_low' ],    # >>> Normal
                                                          Scemd[ sMode ]['duty'][ iLine ]['pwm_duty_high'] )     # >>> Full

            for sDev in asset.ScemdDevList :
               if sDev in Scemd[sMode] :
                  C = []
                  A = []
                  
                  iLine = [ x for x in Scemd[ sMode ][ sDev ].keys() ][0]
                  for sID in asset.ScemdPwmList :
                     if sID in Scemd[ sMode ][ sDev ][ iLine ] :
                        A.append( asset.ScemdPwmShort[sID] )
                        C.append( Scemd[ sMode ][ sDev ][ iLine ][ sID ] )

                  cnt = len ( C )
                  if cnt > self.PwmCount : self.PwmCount = cnt

                  Conf['PWM' + Mode + sDev + '_C'] = ' '.join( C )
                  Conf['PWM' + Mode + sDev + '_A'] = ' '.join( A )

               else :
                  Conf['PWM' + Mode + sDev + '_C'] = ''
                  Conf['PWM' + Mode + sDev + '_A'] = ''

         else :
            # -----------------------------------------------------------------------
            Conf['PWM' + Mode + '_S'] = ''
            for sDev in asset.ScemdDevList :
               Conf['PWM' + Mode + sDev + '_C'] = ''
               Conf['PWM' + Mode + sDev + '_A'] = ''

      # -----------------------------------------------------------------------------
      for Mode in asset.ScemdModeList :
         sMode = Mode + 'ADT'
         if sMode in Scemd and Scemd[sMode] :
            # -----------------------------------------------------------------------
            for sDev in asset.ScemdDevList :
               C = []
               S = []
               A = []

               if sDev in Scemd[sMode] :
                  iList = sorted( [ int( x ) for x in Scemd[sMode][sDev].keys() ] )
                  for sID in iList :
                     C.append( Scemd[sMode][sDev][str(sID)]['limit'] )
                     S.append( Scemd[sMode][sDev][str(sID)]['fan_speed'][0:2] )
                     A.append( Scemd[sMode][sDev][str(sID)]['action'] )

                  # -----------------------------------------------------------------
                  if not 'ALERT' in A[-2] : A[-2] = 'FULL'

                  cnt = len ( C )
                  if cnt > self.AdtCount : self.AdtCount = cnt

                  Conf['ADT' + Mode + sDev + '_C'] = ' '.join( C )
                  Conf['ADT' + Mode + sDev + '_S'] = ' '.join( S )
                  Conf['ADT' + Mode + sDev + '_A'] = ' '.join( A )

               else :
                  # -----------------------------------------------------------------
                  Conf['ADT' + Mode + sDev + '_C'] = ''   # --> limit (temperature)
                  Conf['ADT' + Mode + sDev + '_S'] = ''   # --> speed (percentage)
                  Conf['ADT' + Mode + sDev + '_A'] = ''   # --> action
         else :
            # -----------------------------------------------------------------------
            for sDev in asset.ScemdDevList :
               Conf['ADT' + Mode + sDev + '_C'] = ''   # --> limit (temperature)
               Conf['ADT' + Mode + sDev + '_S'] = ''   # --> speed (percentage)
               Conf['ADT' + Mode + sDev + '_A'] = ''   # --> action
      # -----------------------------------------------------------------------------
      return Conf

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def GetAdt( self, sList ) :
      ADT = dict()
      Error = False

      for sDev in asset.ScemdDevList :
         sStart    = '<%s_temperature '  % sDev
         sEnd      = '</%s_temperature>' % sDev

         for sID in sorted( sList ) :

            line = ' '.join( sList[sID].split() )
            if sStart in line and sEnd in line :
               line = line.split( sStart, 1 )[1]
               line = line.split( sEnd  , 1 )[0]
               line, threshold = line.split( '>', 1 )
               line += ' limit="%s"' % threshold

               ret = dict()

               for sTuple in line.split( ' ' ) :
                  key, value = sTuple.split( '=' )
                  ret[key] = value.strip( '"' )

               if not ret['limit'] or not ret['limit'].isnumeric() :
                  Error = True

               if not ret['fan_speed'] or len( ret['fan_speed'] ) != 7 or not ret['fan_speed'][0:2].isnumeric() :
                  if len( ret['fan_speed'] ) == 6 and ret['fan_speed'][0:1].isnumeric() :
                     ret['fan_speed'] = '0' + ret['fan_speed']
                     self.LengthErr = True
                  else :
                     Error = True

               if Error :
                  lib.Error ( 'SYS: SCEMD Content error :' )
                  lib.Error ( 'Path : {0}'.format( os.path.dirname ( self.xmlFile ) ) )
                  lib.Error ( 'File : {0}'.format( os.path.basename( self.xmlFile ) ) )
                  lib.Error ( 'Line : {0}'.format( sID ) )
                  lib.Error ( '>>>> : {0}'.format( sList[sID].strip() ) )
                  sys.exit( 1 )

               try    : ADT[sDev]
               except : ADT[sDev] = dict()

               try    : ADT[sDev][sID].update( { ret } )
               except : ADT[sDev][sID] =         ret

      # -----------------------------------------------------------------------------
      return ADT

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def GetPwm( self, sList ) :
      PWM = dict()
      Error = False

      # -----------------------------------------------------------------------------
      # --- First find Token "pwm_config" for Speed-Value ....
      for sID in sorted( sList ) :

         line = ' '.join( sList[sID].split() )
         if not line or not 'pwm_config' in line : continue

         # --------------------------------------------------------------------------
         line = line.split( '<pwm_config '  , 1 )[1]
         line = line.split( '></pwm_config>', 1 )[0]

         ret = dict()
         for sTuple in line.split( ' ' ) :
            key, value = sTuple.split( '=' )
            ret[key] = value.strip( '"' )

         try    : PWM['duty']
         except : PWM['duty'] = dict()

         try    : PWM['duty'][sID].update( { ret } )
         except : PWM['duty'][sID] =         ret

         break

      # -----------------------------------------------------------------------------
      # --- Next  find Token "alert_config" for Device-Value ....
      for sDev in asset.ScemdDevList :
         for sID in sorted( sList ) :

            line = ' '.join( sList[sID].split() )
            if not line or not 'alert_config' or not 'name="%s"' % sDev in line : continue

            # -----------------------------------------------------------------------
            line = line.split( '<alert_config '  , 1 )[1]
            line = line.split( '></alert_config>', 1 )[0]

            ret = dict()
            for sTuple in line.split( ' ' ) :
               key, value = sTuple.split( '=' )
               ret[key] = value.strip( '"' )

            try    : PWM[sDev]
            except : PWM[sDev] = dict()

            try    : PWM[sDev][sID].update( { ret } )
            except : PWM[sDev][sID] =         ret

            break
      # -----------------------------------------------------------------------------
      return PWM

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def GetSection( self, sToken, lScemd ) :

      sList  = dict()
      sToken = 'DUAL_MODE_%s' % sToken

      # -----------------------------------------------------------------------------
      for a in range( 0, len( lScemd ) ) :
         line = lScemd[a]

         # --- Search for the start-Token ...
         if not sList :
            if sToken in line :
               sList[str(a)] = line
         else :
            # --- Search for the end of the area "Mode" ...
            sList[str(a)] = line
            if '</adt_fan_config>' in line or '</fan_config>' in line : break

      # -----------------------------------------------------------------------------
      return sList

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def Mode( self, source ) :

      if not source : return False

      # -----------------------------------------------------------------------------
      # --- Determine whether Fan-Control is supported on this NAS ...
      Type  = False
      Speed = False
      Mode  = 'None'

      # -----------------------------------------------------------------------------
      for line in source :

         if not Type :
            # -----------------------------------------------------------------------
            # --- Finde the first section with "type=" ...
            if 'type=' in line :
               Type = line
         else :
            # -----------------------------------------------------------------------
            # --- Find the first "disk_temperature" ...
            if not Speed and '</disk_temperature>' in line :
               Speed = line.split( 'fan_speed="', 1 )[-1].split( '"', 1 )[0]

            # --------------------------------------------------------------------------
            # --- Count the occurrence of '</pwm_config>' ...
            if '</pwm_config>' in line :
               try    : Pwm.append( line )
               except : Pwm = [ line ]

            # --------------------------------------------------------------------------
            # --- Find close Token ...
            if '</adt_fan_config>' in line or '</fan_config>' in line :
               break


      # -----------------------------------------------------------------------------
      if Type :
         if '<adt_fan_config' in Type and len( Pwm ) == 1 :
            Mode = 'PWM'

         if '<fan_config' in Type and Speed and 'hz' in Speed :
            Mode = 'ADT'

      # -----------------------------------------------------------------------------
      return Mode

   # --------------------------------------------------------------------------------
   # --------------------------------------------------------------------------------
   def Reset( self, reset=False ) :
      if not reset :
         self.xmlFatal = 'SYS: SCEMD exception : Function "Reset" - Missing parameter "reset="'
         lib.Error( self.xmlFatal )
         return False         

      # -----------------------------------------------------------------------------
      if io.check( asset.f_VirtualDSM ) :
         io.ioCopy( asset.ScemdPatFile , asset.ScemdVrmFile )
      else :
         io.ioCopy( asset.ScemdPatFile , asset.ScemdFile, opt=False )

      # -----------------------------------------------------------------------------
      if not io.check( asset.f_VirtualDSM ) and not io.check( asset.f_ExcludeSCEMD ) and not io.check( asset.f_ScemdError ) :
         if self.xmlWait : lib.getCMD( RestartScemd        )
         else            : lib.getCMD( RestartScemd + ' &' )


      # -----------------------------------------------------------------------------
      ret = { 'FanCurrentHash' : self.xmlHash,
              'FanCurMode'     : 'LOW' }

      ret.update( self.xmlCur )

      io.ioRm(  asset.f_ScemdError   )
      io.ioRm(  asset.f_ScemdChanged )
      io.touch( asset.f_ScemdInit    )

      return ret

# -----------------------------------------------------------------------------------
# --- "Quick" check the validity of the scemd.xml (PAT file) ...
# -----------------------------------------------------------------------------------
def Validate( conf=dict() ) :

   if not conf : Conf = lib.ConfigRead( asset.avrConfigFile )
   else        : Conf = conf

   # --------------------------------------------------------------------------------
   if io.check( asset.f_VirtualDSM ) :
      xmlFile  = asset.ScemdVrmFile
   else :
      xmlFile  = asset.ScemdFile

   # --------------------------------------------------------------------------------
   # --- Check important files ...
   if ( not io.check( xmlFile )               or   # << "/usr/syno/etc.defaults/scemd.xml"
        not io.check( asset.ScemdPatFile )    or   # << ETC + "/log/scemd.xml"
        not io.check( asset.ScemdMd5File )    or   # << ETC + "/log/scemd.md5"
        not io.check( asset.f_ScemdInit ) ):

      io.ioRm( asset.f_ScemdInit    )
      io.ioRm( asset.f_ScemdError   )
      io.ioRm( asset.f_ScemdChanged )
      return False

   # --------------------------------------------------------------------------------
   # --- Check MD5 Hashes ...
   with open( asset.ScemdPatFile, 'rb' ) as f :
      PatHash = hashlib.md5( f.read() ).hexdigest()

   with open( asset.ScemdMd5File, 'r', encoding='utf-8' ) as f :
      Md5Hash = f.read()

   if PatHash != Md5Hash or PatHash != Conf['FanScemdHash'] :

      io.ioRm( asset.f_ScemdInit    )
      io.ioRm( asset.f_ScemdError   )
      io.ioRm( asset.f_ScemdChanged )
      return False

   with open( xmlFile, 'rb' ) as f :
      xmlHash = hashlib.md5( f.read() ).hexdigest()

   if xmlHash != Conf['FanCurrentHash'] :
      io.touch( asset.f_ScemdChanged )

   # --------------------------------------------------------------------------------
   # --- All Checks passed ...
   return True

# -----------------------------------------------------------------------------------
# --- Examine Fan-Mode ( PWM / ADT / False ) ...
# --- Examine Disk-Model ( if "VirtualDSM" or "DDSM" = False ) ...
# -----------------------------------------------------------------------------------
def ScemdGetMode( sFile=False ) :

   # --------------------------------------------------------------------------------
   # --- Get DiskStation Model ...
   Model = lib.FileConfig( '/etc.defaults/synoinfo.conf' )['upnpmodelname']
   if Model == 'DDSM' : Model = 'VirtualDSM'

   if not sFile :
      if Model != 'VirtualDSM' : xmlFile = asset.ScemdFile
      else                     : xmlFile = asset.ScemdVrmFile
   else :
      xmlFile = sFile

   # --------------------------------------------------------------------------------
   if not io.check( xmlFile ) or io.check( asset.f_ExcludeSCEMD ) : return Model, False

   # --------------------------------------------------------------------------------
   # --- Determine whether Fan-Control is supported on this NAS ...
   Type  = False
   Speed = False
   Mode  = False
   Pwm   = ''

   # --------------------------------------------------------------------------------
   with open( xmlFile, 'r', encoding='utf-8' ) as f:
      for line in f :

         if not Type :
            # --------------------------------------------------------------------------
            # --- Finde the first section with "type=" ...
            if 'type=' in line :
               Type = line
         else :
            # --------------------------------------------------------------------------
            # --- Find the first "disk_temperature" ...
            if not Speed and '</disk_temperature>' in line :
               Speed = line.split( 'fan_speed="', 1 )[-1].split( '"', 1 )[0]

            # --------------------------------------------------------------------------
            # --- Count the occurrence of '</pwm_config>' ...
            if '</pwm_config>' in line :
               try    : Pwm.append( line )
               except : Pwm = [ line ]

            # --------------------------------------------------------------------------
            # --- Find close Token ...
            if '</adt_fan_config>' in line or '</fan_config>' in line :
               break

   # --------------------------------------------------------------------------------
   if Type :
      if '<adt_fan_config' in Type and len( Pwm ) == 1 :
         Mode = 'PWM'

      if '<fan_config' in Type and Speed and 'hz' in Speed :
         Mode = 'ADT'

   # --------------------------------------------------------------------------------
   return Model, Mode

# -----------------------------------------------------------------------------------
# --- Show a Download Progress-Bar (Only if we run in a terminal) ...
# -----------------------------------------------------------------------------------
def Progress( count, dl_size, total_size ) :
   if not sys.stdout.isatty() : return

   percent = int( count * dl_size * 50 / total_size )
   sys.stdout.write( "[%-50s] %3d%%" % ('=' * (percent-1) + ">", 2 * percent ) )
   sys.stdout.flush()
   sys.stdout.write( "\b" * 57 )

# -----------------------------------------------------------------------------------
# --- Download, unpack and initialize the appropriate PAT file for this
# --- DiskStation and the current DSM version.
# -----------------------------------------------------------------------------------
def PatInit( sArchive='', cgi=False, index=False, Version=False  ) :

   # --------------------------------------------------------------------------------
   # --- Check whether DSM is running in a virtual environment and which fan mode is supported ...
   if not index :
      ModelName, FanMode = ScemdGetMode()

      if not FanMode :
         io.touch( asset.f_ExcludeSCEMD )
         lib.Error ( 'SYS: Fan control is not supported on this DiskStation "{0}"'.format( ModelName ) )
         lib.Error ( 'SYS: Fan-Control initialization is canceled' )
         return False
      else :
         io.ioRm( asset.f_ExcludeSCEMD )
   else :
      ModelName = index

   # --------------------------------------------------------------------------------
#  if ModelName == 'VirtualDSM' :
   if ModelName == 'VirtualDSM' and not Version :
      lib.Error ( 'SYS: A virtual environment was detected' )
      lib.Error ( 'SYS: The fan control is started in emulation mode')
      Virtual = True
   else :
      Virtual = False

   # --------------------------------------------------------------------------------
   # --- Check PAT-storage folder ...
   if not Virtual :
      arr = lib.FindeShare( sArchive )
      if arr :
         patHome = os.path.join( *arr )
      else   :
         lib.Error ( 'SYS: Folder "{0}" does not exist'.format( sArchive ) )
         lib.Error ( 'SYS: Fan-Control initialization is canceled' )
         return False

      # -----------------------------------------------------------------------------
      # --- Get some DSM-Info ...
   #  Version  = lib.FileConfig( '/etc.defaults/VERSION' )
      if not Version : Version  = lib.FileConfig( '/etc.defaults/VERSION' )

      # -----------------------------------------------------------------------------
      # --- Prepare to download the DSM PAT-Files to extract the "scemd.xml" file ...
      patName = 'DSM_{0}_{buildnumber}.pat'.format( ModelName, **Version )
      lib.Error ( 'SYS: Download for DSM version {productversion}-{buildnumber} >> "{0}"'.format( patName, **Version ) )

      # -----------------------------------------------------------------------------
      # --- Build request-URL's ... ( https://archive.synology.com/download/DSM/release/ )
      md5URL = '{0}/{1}/{2}/{3}.md5'.format( asset.ScemdBaseURL, Version['productversion'], Version['buildnumber'], patName )
      patURL = '{0}/{1}/{2}/{3}'.format    ( asset.ScemdBaseURL, Version['productversion'], Version['buildnumber'], patName )

      # -----------------------------------------------------------------------------
      # --- Download MD5 Hash-File ...
      # --- Synology removed the MD5 checksum files
   #  try :
   #     md5Hash = ( urllib.request.urlopen( md5URL ).read() ).decode('utf-8')
   #     print ( md5Hash )
   #  except urllib.error.HTTPError as e :
   #     lib.Error ( 'SYS: Download of MD5 Hash-File "{0}.md5" failed'.format( patName ) )
   #     lib.Error ( 'SYS: {0}'.format( e ) )
   #     lib.Error ( 'SYS: Fan-Control initialization is canceled' )
   #     return False

      # -----------------------------------------------------------------------------
      # --- Download DSM.pat file ...
      patFolder = '{0}_{productversion}-{buildnumber}'.format( ModelName, **Version )
      patHome = os.path.join( patHome, patFolder )

      isDir = os.path.isdir( patHome )
      if not isDir : os.mkdir( patHome )

      patDest = os.path.join( patHome, patName )

      if not os.path.isfile( patDest ) :
         lib.Error ( 'SYS: Download pat-File "{0}"'.format( patName ) )
         if cgi : lib.CallTask( Html.jBusy, 'Downloading pat-File :<br>{0}'.format( patName ) )

         try :
            urllib.request.urlretrieve( patURL, patDest, reporthook=Progress )
            if not cgi : print ()

         except urllib.error.HTTPError as e :
            lib.Error ( 'SYS: Download of pat-File "{0}" failed'.format( patName ) )
            lib.Error ( 'SYS: {0}'.format( e ) )
            lib.Error ( 'SYS: Fan-Control initialization is canceled' )
            return False
      else :

      # -----------------------------------------------------------------------------
      # -----------------------------------------------------------------------------
         lib.Error ( 'SYS: Download "{0}" skipped, already done'.format( patName ) )
      # -----------------------------------------------------------------------------
      # -----------------------------------------------------------------------------

      # -----------------------------------------------------------------------------
      # --- Open PAT-File in "byte"-Mode and calculate MD5-Hash ...
      # --- Synology removed the MD5 checksum files
   #  with open( patDest, 'rb' ) as f :
   #     patHash = hashlib.md5( f.read() ).hexdigest()

   #  if patHash != md5Hash :
   #     lib.Error ( 'SYS: The expected MD5-Hash does not match' )
   #     lib.Error ( 'SYS: Fan-Control initialization is canceled' )
   #     shutil.rmtree( patHome )
   #     return False

      # -----------------------------------------------------------------------------
      # --- Extract "scemd.xml" from PAT-File ...
      if not os.path.isfile( os.path.join( patHome, 'hda1.tgz' ) ) :
         lib.Error ( 'SYS: Extracting "hda1.tgz" from pat-File ...' )
         if cgi : lib.CallTask( Html.jBusy, 'Extracting "hda1.tgz"<br> from pat-File ...' )

         with tarfile.open( patDest, mode='r' ) as tf:
            tf.extract( 'hda1.tgz', path=patHome )
      else :
         lib.Error ( 'SYS: Extracting "hda1.tgz" ... skipped, already done' )

      # -----------------------------------------------------------------------------
      if not os.path.isfile( os.path.join( patHome, 'hda1' ) ) :
         lib.Error ( 'SYS: Extracting "hda1" from "hda1.tgz" ...' )
         if cgi : lib.CallTask( Html.jBusy, 'Extracting "hda1"<br> from "hda1.tgz" ...' )
         bash = '7z e "{0}/hda1.tgz" -o"{0}" -aoa'.format( patHome )
         ret, err = subprocess.Popen( bash, shell=True, stdout=subprocess.PIPE, stdin=subprocess.PIPE ).communicate()
      else :
         lib.Error ( 'SYS: Extracting "hda1" ....... skipped, already done' )

      # -----------------------------------------------------------------------------
      # --- Check if "scemd.xml" has already been extracted "and" Hashes matches ...
      xmlFile  = '{0}/scemd.xml'.format( patHome )
      md5File  = '{0}/scemd.md5'.format( patHome )
      sMember  = 'usr/syno/etc.defaults/scemd.xml'

      if os.path.isfile( xmlFile ) and os.path.isfile( md5File ) :
         with open( xmlFile, 'rb' ) as f :
            xmlHash = hashlib.md5( f.read() ).hexdigest()

         with open( md5File, 'r', encoding='utf-8' ) as f :
            md5Hash = f.read()

      # -----------------------------------------------------------------------------
      try :
         if xmlHash == md5Hash :
            lib.Error ( 'SYS: Extracting "scemd.xml" .. skipped, already done' )
      except :
         lib.Error ( 'SYS: Extracting "scemd.xml" from "hda1" ...' )
         with tarfile.open( '{0}/hda1'.format( patHome ), mode='r' ) as tf:
            tf.extract( sMember, path=patHome )

         # --------------------------------------------------------------------------
         io.ioCopy( os.path.join( patHome, sMember ) , xmlFile )
         shutil.rmtree( patHome + '/usr' )

      if index :
         io.ioCopy( xmlFile, sArchive + '/' + index + ' scemd.xml' )
         return

   else :
      # -----------------------------------------------------------------------------
      # --- Files & Path for virtual environment
      patHome  = asset.PKG + '/templates/Scemd'
      xmlFile  = '{0}/scemd.xml'.format( patHome )
      md5File  = '{0}/scemd.md5'.format( patHome )

   # --------------------------------------------------------------------------------
   # --- Open "scemd.xml"-File in "byte"-Mode and calculate MD5-Hash ...
   with open( xmlFile, 'rb' ) as f :
      xmlHash = hashlib.md5( f.read() ).hexdigest()

   with open( md5File, 'w+', encoding='utf-8' ) as f:
      f.write( xmlHash )

   # --------------------------------------------------------------------------------
   if not Virtual :
      shutil.chown( patHome + '/' , user='admin', group='users' )
      os.chmod( patHome, 0o777 )

   # --------------------------------------------------------------------------------
   io.ioCopy( xmlFile, asset.ScemdPatFile )   # -->> ETC + "/log/scemd.xml"
   io.ioCopy( md5File, asset.ScemdMd5File )   # -->> ETC + "/log/scemd.md5"

   # --------------------------------------------------------------------------------
   # --- Clear / Reset all Settings ...
   # --------------------------------------------------------------------------------
   F = Fan( init=True )

   if F and not F.xmlFatal :
      sRet = { 'FanScemdHash'   : xmlHash,
               'FanCurrentHash' : F.xmlHash,
               'FanCurMode'     : 'LOW' }

      sRet.update( F.xmlCur )

      # --------------------------------------------------------------------------------
      io.ioRm(  asset.f_ScemdError   )
      io.ioRm(  asset.f_ScemdChanged )
      io.touch( asset.f_ScemdInit    )
      return sRet
   else :
      # --------------------------------------------------------------------------------
      io.ioRm(  asset.f_ScemdError   )
      io.ioRm(  asset.f_ScemdChanged )
      io.ioRm(  asset.f_ScemdInit    )
      return False

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# sys.exit()
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
