# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#

import os
import sys
import glob
import time
import json
from   urllib.parse import quote
import subprocess

import asset
import basics   as     io
import Cron     as     cron
from   lib      import switch, case
import lib
import Sys
import Line
import Sensors

Conf  = dict()

AlertName = [None] * asset.iTabLast
AlertTemp = [None] * asset.iTabLast
AlertWarn = [None] * asset.iTabLast
AlertTime = [None] * asset.iTabLast
AlertMax  = [None] * asset.iTabLast

# -----------------------------------------------------------------------------------
smtp = sys.modules[ __name__ ]
stat = sys.modules[ __name__ ]

import __main__ as avr

# -----------------------------------------------------------------------------------
eSuffix  = [ 'Pri1LabelSuffix', 'Sec1LabelSuffix', 'Pri2LabelSuffix', 'Sec2LabelSuffix', 'Pri3LabelSuffix', 'Sec3LabelSuffix' ]
eInfo    = [ 'LU WWN Device Id', 'Device is', 'Local Time is', 'SMART support is', 'ATA Version is' ]
iFlag    = True


# -----------------------------------------------------------------------------------
# --- Check Alert thresholds (60 Seconds intervall) ...
# -----------------------------------------------------------------------------------
def Threshold( opt=False ) :

   # --------------------------------------------------------------------------------
   if not stat.AlertList : return

   # --------------------------------------------------------------------------------
   # --- Walk through the list: check thresholds and times ...
   for j in stat.AlertList :
      Current = int( asset.Summe[j] / asset.SMAdepth )
      if not stat.AlertWarn[j] :
      # Then -------------------------------------------------------------------------
         if not stat.AlertTime[j] :
            if Current >= stat.AlertTemp[j] :
               stat.AlertTime[j] = int( time.time() ) + stat.AlertDelayUp
               stat.AlertMax[j]  = Current
         else :
            if Current < stat.AlertTemp[j] :
               stat.AlertTime[j] = None
            else :
               if Current > stat.AlertMax[j] :
                  stat.AlertMax[j] = Current
         # --------------------------------------------------------------------------

         if stat.AlertTime[j] and int ( time.time() ) >= stat.AlertTime[j] :
            stat.AlertWarn[j] = "GoHigh"
            stat.AlertTime[j] = None
         # --------------------------------------------------------------------------

      else :
         if not stat.AlertTime[j] :
            if Current <  stat.AlertTemp[j] : stat.AlertTime[j] = int( time.time() ) + stat.AlertDelayDown
         else :
            if Current >= stat.AlertTemp[j] : stat.AlertTime[j] = None

         if stat.AlertTime[j] and int( time.time() ) >= stat.AlertTime[j] :
            stat.AlertWarn[j] = "GoLow"

   # --------------------------------------------------------------------------------
   for j in stat.AlertList :
      if stat.AlertWarn[ j ] :
         Suffix = asset.AvrConf[ eSuffix[ asset.BaseNum[ j ] ] ]

         # --------------------------------------------------------------------------
         # --- Create & submit notification ...
         while switch( stat.AlertWarn[j] ):
            if case( 'GoHigh' ):
               if asset.AvrConf["Alert" + asset.BaseName[j] +  "PtcOn"] == "Yes" :
                  ThresholdPTCnotifyHigh( j, Suffix )

               if asset.AvrConf["Alert" + asset.BaseName[j] +  "DsmOn"] == "Yes" :
                  ThresholdDSMnotifyHigh( j, Suffix )

               if asset.AvrConf["Alert" + asset.BaseName[j] + "MailOn"] == "Yes" and asset.AvrConf["StatusAlertMailTo"] != "Off" :
                  ThresholdMailNotifyHigh( j, Suffix )

               if asset.AvrConf["Alert" + asset.BaseName[j] + "PushOn"] == "Yes" :
                  ThresholdPushNotifyHigh( j, Suffix )

               stat.AlertWarn[j] = "Send"
               break

            if case( 'GoLow' ):
               if asset.AvrConf["Alert" + asset.BaseName[j] +  "PtcOn"]  == "Yes" :
                  ThresholdPTCnotifyLow( j, Suffix )

               if asset.AvrConf["Alert" + asset.BaseName[j] +  "DsmOn"]  == "Yes" :
                  ThresholdDSMnotifyLow( j, Suffix )

               if asset.AvrConf["Alert" + asset.BaseName[j] + "MailOn"] == "Yes" and asset.AvrConf["StatusAlertMailTo"] != "Off" :
                  ThresholdMailNotifyLow( j, Suffix )

               if asset.AvrConf["Alert" + asset.BaseName[j] + "PushOn"] == "Yes" :
                  ThresholdPushNotifyLow( j, Suffix )

               stat.AlertWarn[j] = None
               break
            break
         # --------------------------------------------------------------------------

# -----------------------------------------------------------------------------------
# --- Protocol-Center notification : Threshold exceeded
# -----------------------------------------------------------------------------------
def ThresholdPTCnotifyHigh( iID, Suffix ) :

   AlertDSM = NotificationText( iID, Suffix, 'AlertPtcAlert' )
   os.system( f'synologset1 sys warn 0x11800000 "{AlertDSM}"' )

# -----------------------------------------------------------------------------------
# --- Protocol-Center notification : Threshold back to normal
# -----------------------------------------------------------------------------------
def ThresholdPTCnotifyLow( iID, Suffix ) :

   AlertDSM = NotificationText( iID, Suffix, 'AlertPtcNormal' )
   os.system( f'synologset1 sys info 0x11800000 "{AlertDSM}"' )

# -----------------------------------------------------------------------------------
# --- DSM / LOG notification : Threshold exceeded
# -----------------------------------------------------------------------------------
def ThresholdDSMnotifyHigh( iID, Suffix ) :

   AlertSub = asset.msg['AlertDsmNotifyHigh'] + ' "' + stat.AlertName[iID] + '"'
   AlertDSM = NotificationText( iID, Suffix, 'AlertDsmAlert' )

   ThresholdDSMnotifyDo( iID, AlertSub, AlertDSM, 'StatusAlertColorUp', 'StatusAlertLogUpOn' )

# -----------------------------------------------------------------------------------
# --- DSM / LOG notification : Threshold back to normal
# -----------------------------------------------------------------------------------
def ThresholdDSMnotifyLow( iID, Suffix ) :

   AlertSub = asset.msg['AlertDsmNotifyLow'] + ' "' + stat.AlertName[iID] + '"'
   AlertDSM = NotificationText( iID, Suffix, 'AlertDsmNormal' )
   ThresholdDSMnotifyDo( iID, AlertSub, AlertDSM, 'StatusAlertColorDown', 'StatusAlertLogDownOn' )

# -----------------------------------------------------------------------------------
# --- Send notification to DSM and to "$ERROR"-Log ...
# -----------------------------------------------------------------------------------
def ThresholdDSMnotifyDo( iID, sSub, sDSM, sColor, sKey ) :

   if asset.f_Dsm7 : lib.Note( sSection='Notify', sTitel='Titel', sText='CustomNotification', i18n=True, sCustom=sDSM )
   else            : lib.Note( sSection='Notify', sTitel=sSub, sText=sDSM )

   if asset.AvrConf[sKey] != "Yes" : return

   Chart = Line.SVG ( iID, 700, 160, -20, 0, asset.AvrConf[sColor] )   # "#008E00"
   with open( asset.ERROR, 'a+', encoding='utf-8' ) as f:
      f.write( Chart )

# -----------------------------------------------------------------------------------
# --- Email notification : Threshold exceeded
# -----------------------------------------------------------------------------------
def ThresholdMailNotifyHigh( iID, Suffix ) :

   if not stat.MailTo : return

   AlertSub = asset.msg['AlertMailNotifyHigh']

   AlertDSM  = "<!DOCTYPE html>\n<html>\n<body>\n"
   AlertDSM += "<pre>" + asset.msg['AlertMailSalutation']
   AlertDSM += NotificationText( iID, Suffix, 'AlertMailAlert' ) + "</pre>"
   AlertDSM += "<pre>" + asset.msg['AlertMailFooter'] + " " + smtp.MailHost + "-" + smtp.MailProduct + "</pre>\n"
   AlertDSM += "</body></html>\n"
   ThresholdMailNotifyDo( AlertSub, AlertDSM )

# -----------------------------------------------------------------------------------
# --- Email notification : Threshold back to normal
# -----------------------------------------------------------------------------------
def ThresholdMailNotifyLow( iID, Suffix ) :

   if not stat.MailTo : return

   AlertSub = asset.msg['AlertMailNotifyLow']

   AlertDSM  = "<!DOCTYPE html>\n<html>\n<body>\n"
   AlertDSM += "<pre>" + asset.msg['AlertMailSalutation']
   AlertDSM += NotificationText( iID, Suffix, 'AlertMailNormal' ) + "</pre>"
   AlertDSM += "<pre>" + asset.msg['AlertMailFooter'] + " " + smtp.MailHost + "-" + smtp.MailProduct + "</pre>\n"
   AlertDSM += "</body></html>\n"
   ThresholdMailNotifyDo( AlertSub, AlertDSM )

# -----------------------------------------------------------------------------------
# --- Send eMail notification ...
# -----------------------------------------------------------------------------------
def ThresholdMailNotifyDo( sSub, sDSM, eMail=False ) :

   if not stat.MailTo : return

   if not eMail : eMail = smtp.AlertMailTo

   Mail  = "To: " + eMail + "\n"
   Mail += "From: " + smtp.MailHost + " - " + smtp.MailFrom + " <" + smtp.MailUser + ">\n"
   Mail += "Subject: " + smtp.MailSubject + ": " + sSub + "\n"
   Mail += "Content-Transfer-Encoding: quoted-printable\n"
   Mail += "Content-Type: text/html; charset=\\\"utf-8\\\"\n"
   Mail += "\n"
   Mail += sDSM.replace( '"', '\\\"' )

   iRet = lib.getCMD( 'echo -e "' + Mail + '" | ssmtp ' + eMail + ' 2>&1' )     # Send DSM notification ...

   if iRet :
      lib.Error( 'SYS: Email notification failed ...' )
      lib.Error(f'SYS: {iRet}' )
      smtp.ErrorMail = True
      return iRet

# -----------------------------------------------------------------------------------
# --- Alert notification via Pushbullet : Threshold exceeded
# -----------------------------------------------------------------------------------
def ThresholdPushNotifyHigh( iID, Suffix ) :

   import Push
   AlertSub = stat.MailHost + ' - ' + asset.msg['AlertPushNotifyHigh'] + ' "' + stat.AlertName[iID] + '"'
   AlertDSM = NotificationText( iID, Suffix, 'AlertPushAlert' )
   Push.Note( AlertSub, AlertDSM )

# -----------------------------------------------------------------------------------
# --- Alert notification via Pushbullet : Threshold back to normal
# -----------------------------------------------------------------------------------
def ThresholdPushNotifyLow( iID, Suffix ) :

   import Push
   AlertSub = stat.MailHost + ' - ' + asset.msg['AlertPushNotifyLow'] + ' "' + stat.AlertName[iID] + '"'
   AlertDSM = NotificationText( iID, Suffix, 'AlertPushNormal' )
   Push.Note( AlertSub, AlertDSM )

# -----------------------------------------------------------------------------------
# --- Prepare Notification-Text :
# -----------------------------------------------------------------------------------
def NotificationText( iID, Suffix, vKey ) :

   THRESHOLD = "%.1f%s" % ( stat.AlertTemp[iID], Suffix )
   CURRENT   = "%.1f%s" % ( stat.AlertMax[iID],  Suffix )
   PLOTNAME  = stat.AlertName[iID]

   msg = asset.msg[vKey]

   msg = msg.replace( '%PLOTNAME%',  PLOTNAME  )
   msg = msg.replace( '%THRESHOLD%', THRESHOLD )
   msg = msg.replace( '%CURRENT%',   CURRENT )

   return msg

# -----------------------------------------------------------------------------------
# --- Get (read) the /usr/syno/etc/synosmtp.conf ...
# --- Es scheint so als ob nicht "alle" synosmtp.conf's sauber aufgebaut sind, daher hier
# --- der umständliche "saubere" weg ...
# -----------------------------------------------------------------------------------
def init( cgi=False ):

   if not asset.AvrConf : asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )

   # --------------------------------------------------------------------------------
   # --- Init Values & Flag's ...
   dTemp            = dict()
   smtp.MailTo      = False
   smtp.AlertMailTo = False
   smtp.eMailMask   = [ False ] * 12
   smtp.eMailList   = [ '' ]    * 12

   if os.path.exists( asset.SynoSmtpFile ) : dTemp = lib.FileConfig( asset.SynoSmtpFile )

   # --------------------------------------------------------------------------------
   # --- Trying to get email addresses ...
   if asset.f_Dsm7 :
      eMail = dTemp.get( 'eventmails', '' ).split( ';' )
   else :
      eMail = []
      eMail.append( dTemp.get( 'eventmail1', '' ) )
      eMail.append( dTemp.get( 'eventmail2', '' ) )

   # --------------------------------------------------------------------------------
   for ID, Value in enumerate( eMail ) :
      if not Value : continue
      smtp.eMailMask[ ID ] = True
      smtp.eMailList[ ID ] = Value

   # --------------------------------------------------------------------------------
   smtp.Conf['eventsmtp']          = dTemp.get( 'eventsmtp',          '' )
   smtp.Conf['eventsubjectprefix'] = dTemp.get( 'eventsubjectprefix', '' )
   smtp.Conf['eventuser']          = dTemp.get( 'eventuser',          '' )

   smtp.AlertDelayUp   = int( asset.AvrConf["StatusAlertDelayUp"]   ) * 60
   smtp.AlertDelayDown = int( asset.AvrConf["StatusAlertDelayDown"] ) * 60
   smtp.StatusTime     = int( time.time() ) + ( 60 - ( int ( time.time() ) % 60 ) )

   # --------------------------------------------------------------------------------
   if not True in smtp.eMailMask :
      # --- Disable email service ...
      cron.CronStatus                    = []
      return

   # --------------------------------------------------------------------------------
   TimeM = asset.AvrConf["StatusMailTimeM"]
   TimeH = asset.AvrConf["StatusMailTimeH"]

   # --------------------------------------------------------------------------------
   for ID, Value in enumerate( asset.AvrConf["StatusMailOn"].split( ' ' ) ):
      if not 'Yes' in Value or not smtp.eMailMask[ ID ] : continue

      try    : smtp.MailTo += ', ' + smtp.eMailList[ ID ]
      except : smtp.MailTo  =        smtp.eMailList[ ID ]

   # --------------------------------------------------------------------------------
   for ID, Value in enumerate( asset.AvrConf["StatusAlertMailOn"].split( ' ' ) ):
      if not 'Yes' in Value or not smtp.eMailMask[ ID ] : continue

      try    : smtp.AlertMailTo += ', ' + smtp.eMailList[ ID ]
      except : smtp.AlertMailTo  =        smtp.eMailList[ ID ]

   # --------------------------------------------------------------------------------
   if cgi : return

   # --------------------------------------------------------------------------------
   while switch( asset.AvrConf["StatusMailMode"] ):
      if case( 'Daily' ):
         cron.CronStatus = [ TimeM, TimeH, '*/1', '*', '*' ]   #  Min. Std.   */1	*	*  = Täglich           zur eingestellten Zeit
         break
      if case( 'Weekly' ):
         cron.CronStatus = [ TimeM, TimeH,   '*', '*', '1' ]   #  Min. Std.   *     *  1  = Jeden Montag      zur eingestellten Zeit
         break
      if case( 'Monthly' ):
         cron.CronStatus = [ TimeM, TimeH,   '1', '*', '*' ]   #  Min. Std.   1     *  *  = Jeden 1. im Monat zur eiingestellten Zeit.
         break
      # --- Default's ----------------------------------------------------
      cron.CronStatus = [ TimeM, TimeH,   '1', '*', '*' ]
      asset.AvrConf["StatusMailMode"] = "Monthly"
      break

   if not Sys.avrInfo : Sys.GetSystemInfo()

   smtp.MailHost    = Sys.avrInfo['Hostname'] # <<< "hostname'"
   smtp.MailProduct = Sys.avrInfo['NAS_Type'] # <<< Product Type ("RackStation" / "DiskStation")
   smtp.MailFrom    = Sys.avrInfo['Mailfrom'] # <<< "mailfrom"
   smtp.MailSubject = smtp.Conf['eventsubjectprefix']
   smtp.MailUser    = smtp.Conf['eventuser']
   smtp.ErrorMail   = False

# -----------------------------------------------------------------------------------
# --- Build / Send AvrLogger Testmail ...
# -----------------------------------------------------------------------------------
def TestMail() :

   # --- Only if e-mail notification is set up correctly ...
   if not stat.MailTo : return

   # --------------------------------------------------------------------------------
   sDSM  =  "<!DOCTYPE html>\n<html>\n<body>\n"
   sDSM += f'<pre>{asset.msg["MailTestSalutation"]}\n'
   sDSM +=  '</pre>\n'
   sDSM += f'<pre>{asset.msg["MailStatusFooter"]} {smtp.MailHost}-{smtp.MailProduct}</pre>\n'
   sDSM +=  "</body>\n</html>\n"
   sDSM  =  sDSM.replace( '"', '\\\"' )

   Mail  = f'To: {smtp.MailTo}\n'
   Mail += f'From: {smtp.MailHost} - {smtp.MailFrom} <{smtp.MailUser}>\n'
   Mail += f'Subject: {smtp.MailSubject}: {asset.msg["MailTestNotify"]}\n'
   Mail +=  "Content-Transfer-Encoding: quoted-printable\n"
   Mail +=  "Content-Type: text/html; charset=\\\"utf-8\\\"\n"
   Mail +=  "\n"

   iRet = lib.getCMD( f'echo -e "{Mail}{sDSM}" | ssmtp {smtp.MailTo} 2>&1' )

   if iRet: 
      lib.Error( 'SYS: Email notification failed ...' )
      lib.Error(f'SYS: {iRet}' )
      return iRet

# -----------------------------------------------------------------------------------
# --- Build / Send AvrLogger Status-Report Email ...
# -----------------------------------------------------------------------------------
def StatusMail( f_TestMail=False, iReport=True ) :

   # --- Only if e-mail notification is set up correctly ...
   if not stat.MailTo : return

   avr          = asset.AvrConf
   stat.Body    = ""
   StatusMail_f = False
   iRet         = False

   # -----------------------------------------------------------------------------------
   # --- Include System information ...
   if avr["StatusMailSysOn"] == "Yes" :
      System()
      StatusMail_f = True

      if avr["StatusMailDrive"] == "Off" :
         stat.Body += "+--------------------------------------------------------------------------------------+\n"

   # -----------------------------------------------------------------------------------
   # --- Include Disk information ...
   if avr["StatusMailDrive"] == "List" :
      getDisk()
      StatusMail_f = True

      if avr["StatusMailLogOn"] == "No" :
         stat.Body += "+--------------------------------------------------------------------------------------+\n"

   # -----------------------------------------------------------------------------------
   # --- Include SMART information ...
   if avr["StatusMailDrive"] == "Smart" :
      getSmart()
      StatusMail_f = True

      if avr["StatusMailLogOn"] == "No" :
         stat.Body += "+--------------------------------------------------------------------------------------+\n"

   # -----------------------------------------------------------------------------------
   # --- Include $ERROR-LOG ...
   if avr["StatusMailLogOn"] == "Yes" :
      stat.Body += "+= Error LOG =-------------------------------------------------------------------------+\n"
      if os.path.exists( asset.ERROR ) :
         stat.Body += io.ioRead( asset.ERROR )
         stat.Body += "\n"

      stat.Body += "+--------------------------------------------------------------------------------------+\n"
      StatusMail_f = True

   if not StatusMail_f : stat.Body += asset.msg['MailStatusEmpty'] + '\n'

   # -----------------------------------------------------------------------------------
   # --- Prepare eMail Header & Body ...
   AlertSub  = asset.msg['MailStatusNotify']

   AlertDSM  =  "<!DOCTYPE html>\n<html>\n<body>\n"
   AlertDSM += f'<pre>{asset.msg["MailStatusSalutation"]}\n'
   AlertDSM +=  stat.Body + "</pre>\n"
   AlertDSM += f'<pre>{asset.msg["MailStatusFooter"]} {smtp.MailHost}-{smtp.MailProduct}</pre>\n'
   AlertDSM +=  "</body>\n</html>\n"

   # -----------------------------------------------------------------------------------
   # --- Prepare eMail Header & Body ...
   if iReport : iRet = ThresholdMailNotifyDo( AlertSub, AlertDSM, smtp.MailTo )
   else       : return AlertDSM

   # -----------------------------------------------------------------------------------
   # --- Clear ERROR-LOG ...
   if not f_TestMail and not smtp.ErrorMail and avr["StatusMailClearOn"] == "Yes" :
      io.ioRm ( asset.ERROR )
      io.touch( asset.ERROR )

   # -----------------------------------------------------------------------------------
   return iRet

# -----------------------------------------------------------------------------------
# --- Get some System / AvrLogger Informations
# -----------------------------------------------------------------------------------
def System() :

   if not 'SMAdepth' in dir() :
      asset.AvrConf = lib.ConfigRead( asset.avrConfigFile )
      init()
      import Sensors
      import Init
      import Sys
      import Smart
      Sys.GetSystemInfo()

      f = Smart.Smart()
      asset.sysStorage = f.getSynoStorage()

      Sensors.init( True )
      Init.DataBase()
      Init.BaseList()
      Init.MinMax  ()

   info = Sys.avrInfo
   avr  = asset.AvrConf

   cpu_count = info['NAS_Thermal']
   fan_count = info['NAS_Fan']
   i2c_count = info['NAS_i2c']

   stat.Body += "+=[ AvrLogger ]=-----------------------------------------------------------------------+\n"
   stat.Body += "| Version    = %10.10s     Build     = %20.20s  Running = %12.12s |\n"                        % ( info['Avr_Version'], info['Avr_Created'], info['Avr_Uptime'] )
   stat.Body += "| Tick-Time  = %10.10sms   Prescaler = %3d    DataBase = %4d    SMA depth = %3d    |\n"       % ( str(asset.sys_Tick), asset.sys_Div, asset.sys_MaxLines, asset.SMAdepth )
   stat.Body += "| TimeBase   = %10.10sh    SVG-Tick  = %3sMin                                       |\n"      % ( avr["LogBase"], avr["SvgTick"] )
   stat.Body += "+=[ Sensors ]=--------+----------------------------------------------------------------+\n"
   stat.Body += "| CPU Sensors  =  %2s  | Env.Sensor = %3.3s    Name =  %34.34s |\n"                           % ( cpu_count, avr["LogEnvOn"],         avr["LogEnvName"] )
   stat.Body += "| FAN Sensors  =  %2s  | SNMPv3     = %3.3s    IP   =  %34.34s |\n"                           % ( fan_count, avr["LogEnvV3On"],       avr["LogEnvIp"] )
   stat.Body += "| i2c Sensors  =  %2s  | Encryption = %3.3s    OID  =  %34.34s |\n"                           % ( i2c_count, avr["LogEnvEncryption"], avr["LogEnvOID"] )
   stat.Body += "+=[ Syno ]=----------------------------------------------------------------------------+\n"
   stat.Body += "| Model    =%15.15s    Serial   =%15.15s    Product =%15.15s   |\n"                           % ( info['NAS_Product'],  info['NAS_Serial'], info['NAS_Type']   )
   stat.Body += "| Platform =%15.15s    Hostname =%15.15s    Uptime  =%15.15s   |\n"                           % ( info['CPU_Platform'], info['Hostname'],   info['NAS_Uptime'] )
   stat.Body += "+=[ CPU ]=-----------------------------------------------------------------------------+\n"
   stat.Body += "| Manufacturer = %20.20s    Collection = %20.20s             |\n"                             % ( info['CPU_Manufacturer'], info['CPU_Collection'] )
   stat.Body += "| Name         = %20.20s    Clock      = %5s (%2s) x %1.2fGHz             |\n"                % ( info['CPU_Processor'], info['CPU_Cores'], info['CPU_Threads'], int(info['CPU_Frequency']) / 1000 )
   stat.Body += "+=[ Axes ]=------------------+---------------------------------------------------------+\n"

   # ------------------------------------------------------------------------------------------------------
   # --- Init Values & Flag's ...
   if True in smtp.eMailMask : GuiMail = True
   else                      : GuiMail = False

   if avr['StatusPushToken'] : GuiPush = True
   else                      : GuiPush = False

   TmpMinMax = [ int(x) for x in io.ioRead( asset.avrScalingFile ).split(' ') if x.isdigit() ]

   # ------------------------------------------------------------------------------------------------------
   # --- Axis Member ...
   for Axis in range( 0, 6 ) :
      printMIN = int( TmpMinMax[ 2 * Axis     ] )
      printMAX = int( TmpMinMax[ 2 * Axis + 1 ] )

      while switch( Axis ):
         if case( 0 ):
            printLIM = int( avr["Pri1ScaleMax"] )
            break
         if case( 1 ):
            printLIM = int( avr["Sec1ScaleMax"] )
            break
         if case( 2 ):
            printLIM = int( avr["Pri2ScaleMax"] )
            break
         if case( 3 ):
            printLIM = int( avr["Sec2ScaleMax"] )
            break
         if case( 4 ):
            printLIM = int( avr["Pri3ScaleMax"] )
            break
         if case( 5 ):
            printLIM = int( avr["Sec3ScaleMax"] )
            break
         break

      if Axis > 0 : stat.Body += "+--------------------------------------------------------------------------------------+\n"

      while switch( Axis ):
         if case( 0 ):
            stat.Body += "| #%1d Temperature °C   Scaling :   Start = %4s    Min/Max = %4d : %-4d [%4d]         |\n"  % ( Axis, avr["Pri1Min"], printMIN, printMAX, printLIM )
            break
         if case( 1 ):
            stat.Body += "| #%1d Load percent %%   Scaling :   Start = %4s    Min/Max = %4d : %-4d [%4d]         |\n" % ( Axis, avr["Sec1Min"], printMIN, printMAX, printLIM )
            break
         if case( 2 ):
            stat.Body += "| #%1d Network  MiB/s   Scaling :   Start = %4s    Min/Max = %4d : %-4d [%4d]         |\n"  % ( Axis, avr["Pri2Min"], printMIN, printMAX, printLIM )
            break
         if case( 3 ):
            stat.Body += "| #%1d Storage  MiB/s   Scaling :   Start = %4s    Min/Max = %4d : %-4d [%4d]         |\n"  % ( Axis, avr["Sec2Min"], printMIN, printMAX, printLIM )
            break
         if case( 4 ):
            stat.Body += "| #%1d Ping ms          Scaling :   Start = %4s    Min/Max = %4d : %-4d [%4d]         |\n"  % ( Axis, avr["Pri3Min"], printMIN, printMAX, printLIM )
            break
         if case( 5 ):
            stat.Body += "| #%1d USV AC voltage   Scaling :   Start = %4s    Min/Max = %4d : %-4d [%4d]         |\n"  % ( Axis, avr["Sec3Min"], printMIN, printMAX, printLIM )
            break
         break

      # ---------------------------------------------------------------------------------------------------------
      if not asset.ScalingAxis_Names[ Axis ] : continue

      # ---------------------------------------------------------------------------------------------------------

      for j in asset.ScalingAxis_Names[ Axis ] :
         msg  = "|    [%2d] %-14s" % ( j, asset.BaseName[j] )
         msg += "%-15s" % ( avr['Plot' + asset.BaseName[j] + 'Label'] )

         # ------------------------------------------------------------------------------------------------------
         # --- Alert Switches ...
         if stat.AlertList :
         #  for i in [int(x) for x in stat.AlertList.split()] :
            for i in stat.AlertList :
               if j == i :
                  msg += " Threshold = %4s    Alert :" % ( avr['Alert' + asset.BaseName[j] + 'Max'] )
                  if avr['Alert' + asset.BaseName[j] + 'DsmOn'] == "Yes" :            msg += " [DSM ]"
                  if GuiPush and avr['Alert' + asset.BaseName[j] + 'PushOn'] == "Yes" : msg += " [Push]"
                  if GuiMail and avr['Alert' + asset.BaseName[j] + 'MailOn'] == "Yes" : msg += " [Mail]"

         stat.Body += "%-86s |\n" % ( msg )

# -----------------------------------------------------------------------------------
# --- Collect Disk's Data
# -----------------------------------------------------------------------------------
def getDisk() :

   import Smart

   stat.Body += "+=[ Disk's ]=--------------------------------------------------------------------------+\n"
   stat.Body += "| Location | Typ | Model                         | Serial             | Hours   |  °C  |\n"
   stat.Body += "+----------+-----+-------------------------------+--------------------+---------+------+\n"

   if io.check( asset.f_VirtualDSM ) :
      stat.Body += "+ AvrLogger runs in a virtual environment : No Disk's available!                       +\n"
      return

   # ---------------------------------------------------
   f    = Smart.Smart( )
   iSdl = f.scanDrives()[ 'iSdl' ]

   for sDev in iSdl :
      sDisk = f.getDiskInfo( sDev )

      if sDisk[ 'isSSD' ] : sDisk[ 'SSD' ] = 'SSD'
      else                : sDisk[ 'SSD' ] = 'HDD'

      # -----------------------------------------------------------------------------
      while switch( sDisk['type'] ) :
         # --------------------------------------------------------------------------
         if case( 'internal' ) :
            sDisk[ 'Locati' ] = 'NAS   {0:>2}'.format(   sDisk[ 'name' ].split( ' ')[1] )
            break
         if case( 'ebox' ) :
            sDisk[ 'Locati' ] = 'Box-{0} {1:>2}'.format( sDisk[ 'loc' ].split( '-' )[1], sDisk[ 'name' ].split( ' ' )[1] )
            break
         if case( 'usb' ) :
            sDisk[ 'Locati' ] = 'USB   {0:>2}'.format(   sDisk[ 'name' ].split( ' ')[-1] )
            break
         if case( 'esata' ) :
            sDisk[ 'Locati' ] = 'eSATA {0:>2}'.format(   sDisk[ 'name' ].split( ' ')[-1] )
            break
         # --------------------------------------------------------------------------
         break

      sAirFlow = f.getSmartAttrID( sDev, '190' )      # Airflow_Temperature_Cel
      sCelsius = f.getSmartAttrID( sDev, '194' )      # Temperature_Celsius

      if   not sCelsius : sCelsius = sAirFlow
      elif not sAirFlow : sAirFlow = sCelsius

      sDisk[ 'Celsius' ] = int( sAirFlow )
      sDisk[ 'Hours'   ] = int( f.getSmartAttrID( sDev, '9' ) ) # Power_On_Hours

      # -----------------------------------------------------------------------------------
      stat.Body += "| {Locati} | {SSD:3.3} | {model:<29.29} | {serial:<18.18} | {Hours:>7d} | {Celsius:02d}°C |\n".format( **sDisk )
 
   stat.Body += "+=[ History ]=---+-------------------------------+--------------------+---------+------+\n"
   stat.Body += "|                                                                                      |\n"

   # --------------------------------------------------------------------------------
   for sDev in iSdl :
      sDisk = f.getDiskInfo( sDev )

      sAttrHistory = f.getAttrHistory( sDev )
      sAlert       = sDisk[ 'denote' ]
      iList        = sDisk['sPredict']

      # -----------------------------------------------------------------------------
      while switch( sDisk['type'] ) :
         # --------------------------------------------------------------------------
         if case( 'internal' ) :
            sDisk[ 'Locati' ] = 'NAS   {0:>2}'.format(   sDisk[ 'name' ].split( ' ')[1] )
            break
         if case( 'ebox' ) :
            sDisk[ 'Locati' ] = 'Box-{0} {1:>2}'.format( sDisk[ 'loc' ].split( '-' )[1], sDisk[ 'name' ].split( ' ' )[1] )
            break
         if case( 'usb' ) :
            sDisk[ 'Locati' ] = 'USB   {0:>2}'.format(   sDisk[ 'name' ].split( ' ')[-1] )
            break
         if case( 'esata' ) :
            sDisk[ 'Locati' ] = 'eSATA {0:>2}'.format(   sDisk[ 'name' ].split( ' ')[-1] )
            break
         # --------------------------------------------------------------------------
         break

      # -----------------------------------------------------------------------------
      if sAttrHistory or sAlert :
         sText = '{Locati} ({0}) - {model} ({serial})'.format( sDev, **sDisk )
         stat.Body += '| {0:84} |\n'.format( sText )

         for sID in [ str( x ) for x in sorted( [ int( x ) for x in iList ] ) ] :
            sPredict = sAttrHistory.get( sID, False )
            if sPredict :
               sText = '#ID {0:>3} : {1}'.format( sID, sPredict['Name'] )
               stat.Body += '| {0:84} |\n'.format( sText )

               for sDate in sorted( sPredict['List'] ) :
                  sTimeStamp = sDate + ' :'
                  iItem = sPredict['List'][ sDate ]
                  iItem[0] = '{0:>15}'.format( iItem[0] )
                  iItem[1] = '{0:<15}'.format( iItem[1] )

                  sText = '          {0:21} {1} to {2}'.format( sTimeStamp, iItem[0], iItem[1] )
                  stat.Body += '| {0:84} |\n'.format( sText )

            if not sAlert : continue

            iItem = sAlert.get( sID, False )
            if iItem :
               if not sPredict :
                  sText = '#ID {0:>3} : {1}'.format( sID, iItem[0] )
                  stat.Body += '| {0:84} |\n'.format( sText )

               sTimeStamp = 'Warning notice      :'
               iItem = sAlert[ sID ]

               iItem[1] = '{0:>15}'.format( iItem[1] )
               iItem[2] = '{0:<15}'.format( iItem[2] )

               sText = '          {0:21} {1} to {2}'.format( sTimeStamp, iItem[1], iItem[2] )
               stat.Body += '| {0:84} |\n'.format( sText )
         # --------------------------------------------------------------------------
         stat.Body += "|                                                                                      |\n"

# -----------------------------------------------------------------------------------
# --- Add SMART-Infos to Status-Report ...
# -----------------------------------------------------------------------------------
def getSmart():

   import Smart
   f       = Smart.Smart( )
   iSdl = f.scanDrives()[ 'iSdl' ]

   for sDev in iSdl :
      sDisk = f.getDiskInfo( sDev )

      if sDisk[ 'isSSD' ] : sDisk[ 'SSD' ] = 'SSD'
      else                : sDisk[ 'SSD' ] = 'HDD'

      stat.Body += '+=[ S.M.A.R.T. ]=----------------------------------------------------------------------+\n'
      stat.Body += '|                                                                                      |\n'

      # -----------------------------------------------------------------------------
      while switch( sDisk['type'] ) :
         # --------------------------------------------------------------------------
         if case( 'internal' ) :
            sText = '{loc} - {name} ({0}) Type : {SSD}'.format( sDev, **sDisk )
            break
         if case( 'ebox' ) :
            sName = ' '.join( sDisk[ 'name' ].split( ' ' )[0:2] )
            sText = 'EUnit {loc} - {0} ({1}) Type : {SSD}'.format( sName, sDev, **sDisk )
            break
         if case( 'usb' ) :
            sText = 'External - {name} ({0}) Type : {SSD}'.format( sDev, **sDisk )
            break
         if case( 'esata' ) :
            sText = 'External - {name} ({0}) Type : {SSD}'.format( sDev, **sDisk )
            break
         # --------------------------------------------------------------------------
         break

      stat.Body += '| {0:84} |\n'.format( sText )
      stat.Body += '|                                                                                      |\n'

      # --- Get some Device-Infos ...
      sSmart = sDisk[ 'sInfo' ]
      for sKey in sSmart[ 'iKey' ] :
         if sKey in eInfo : continue
         stat.Body += '|    {0:16} : {1:55}        |\n'.format( sKey, sSmart[ 'sValue' ][ sKey ] )

      sSmart = sDisk[ 'sConf' ]
      for sKey in sSmart[ 'iKey' ] :
         stat.Body += '|    {0:16} : {1:55}        |\n'.format( sKey, sSmart[ 'sValue' ][ sKey ] )

      # --- Get SMART-Values ...
      stat.Body += '|                                                                                      |\n'
      stat.Body += '|    {0:<4} - {1:25.25} - {2:>20.20}                           |\n'.format( ' #ID', 'Attribut', 'Rohwert' )

      sSmart = sDisk[ 'sAttr' ]
      for sKey in sSmart[ 'iKey' ] :
         iAttr = sSmart[ 'sValue' ][ sKey ]
         stat.Body += '|    {0:>4} - {1:25.25} : {2:>20.20}                           |\n'.format( sKey, iAttr[0], iAttr[4] )

      stat.Body += '|                                                                                      |\n'

# --------------------------------------- END ---------------------------------------
# -----------------------------------------------------------------------------------
#  sys.exit()
# -----------------------------------------------------------------------------------
# -----------------------------------------------------------------------------------
