# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------------#
# AvrLogger - Monitoring and visualization tool.                                     #
# Copyright (C) 2018 by BigRonin.                                                    #
# Member of the German Synology Community Forum "https://www.synology-forum.de"      #
# coding: utf-8                                                                      #
# -----------------------------------------------------------------------------------#
# This file is part of "AvrLogger.spk"                                               #
#                                                                                    #
# AvrLogger is free software: you can redistribute it and/or modify                  #
# it under the terms of the GNU General Public License as published by               #
# the Free Software Foundation, either version 3 of the License, or                  #
# (at your option) any later version.                                                #
# -----------------------------------------------------------------------------------#
import os
import sys
import time
import json

import asset
import basics   as io

import lib

avrInfo  = dict()

# -----------------------------------------------------------------------------------
sys = sys.modules[ __name__ ]
# -----------------------------------------------------------------------------------

# --------------------------------------------------------------------------
# --- Set the notification depending on the setting :
# --> True  = you will see this message
# --> False = Suppresses this message
# --------------------------------------------------------------------------
def SetAdminNotification() :

   if not os.path.exists( asset.sysAdminNotification ) : return

   rDict = json.loads( io.ioRead( asset.sysAdminNotification ) )

   iStatus = rDict[ 'DisableAdminNotification.js' ][ 'SYNO.SDS.App.DisableAdminNotification.Instance' ][ 'autoLaunch' ]

   if asset.AvrConf[ 'AdminNotification' ] == 'Yes' :
      AvrNotify = True
   else :
      AvrNotify = False

   if AvrNotify != iStatus :
      rDict[ 'DisableAdminNotification.js' ][ 'SYNO.SDS.App.DisableAdminNotification.Instance' ][ 'autoLaunch' ] = AvrNotify
      io.ioWrite( json.dumps( rDict ), asset.sysAdminNotification, opt=False )

# --------------------------------------------------------------------------
# ---  Add the expected number of values since the last update to avr.dat
# --------------------------------------------------------------------------
def AddOffline() :

   # --- No LogData = No update ...
   if not os.path.exists( asset.avrLogDataFile ) : return

   lib.TimerStart()
   # -----------------------------------------------------------------------

   # --- Prepare String ...
   pine = ' '.join( (['0.000'] * asset.iTabLast) )

   # --- Get Seconds since the last update ...
   count = int( ( time.time() - io.ioStat( asset.avrLogDataFile ) ) / asset.sys_Tick / asset.sys_Div )

   if count > asset.sys_MaxLines : count = asset.sys_MaxLines

   for i in range( 0, count ) :
      io.ioAdd( pine, asset.avrLogDataFile )

   # -----------------------------------------------------------------------
   lib.TimerStop()
   lib.Echo ( 1, asset.msg['msgAddOffline'] + ' ' + str(count) )

# --------------------------------------------------------------------------
# --- Clear database (AvrLogger.dat) With rotating backup ...
# --------------------------------------------------------------------------
def RotateClear( sFile ) :

   if not os.path.exists( sFile ) : return

   for i in reversed(range( 0, 9 )):
      if os.path.exists( sFile + '.' + str(i) ) :
         io.ioMove( sFile + '.' + str(i), sFile + '.' + str(i + 1) )

   if os.path.exists( sFile ) :
      io.ioMove( sFile, sFile + '.0' )

# --------------------------------------------------------------------------
# --- Cut a file from the end to numbered lines
# --------------------------------------------------------------------------
def TrimFile( iCnt, sFile) :

   if not os.path.exists( sFile ) : return

   sList = io.ioRead( sFile ).split('\n')
   iList = len(sList)

   if iList <= iCnt : return

   iCut = iList - iCnt
   io.ioWrite( '\n'.join(sList[iCut:]) + '\n', sFile )

# --------------------------------------------------------------------------
# --- Get some System-Infos: --> sys_SynoInfo[@]
# --------------------------------------------------------------------------
# --->  0 = Manufacturer
# --->  1 = Product Collection
# --->  2 = Processor Number
# --->  3 = # of Cores
# --->  4 = # of Threads
# --->  5 = Base Frequency GHz
# --->  6 = Synology Product
# --->  7 = Product Type ("RackStation" / "DiskStation")
# --->  8 = Platform
# --->  9 = Hostname
# ---> 10 = Uptime
# ---> 11 = System language
# ---> 12 = AvrLogger Uptime
# ---> 13 = Serial Number
# ---> 14 = System RAM in GB
# ---> 15 = Mailfrom
# --------------------------------------------------------------------------
def GetSystemInfo() :

   aTemp = [ x.strip() for x in io.ioRead( '/proc/syno_cpu_arch' ).split(',') ]
   if aTemp[3] == "unknown" : aTemp[3] = "0"

   sys.avrInfo['CPU_Manufacturer']  = aTemp[0]
   sys.avrInfo['CPU_Collection']    = aTemp[1]
   sys.avrInfo['CPU_Processor']     = aTemp[2]
   sys.avrInfo['CPU_Cores']         = aTemp[3]
   sys.avrInfo['CPU_Platform']      = io.ioRead( '/proc/syno_platform' )

   sTemp = io.ioRead( '/proc/cpuinfo' ).split('\n')
   sys.avrInfo['CPU_Threads']       = sum( 'processor' in x for x in sTemp )
   sys.avrInfo['CPU_Frequency']     = str( int (int( io.ioRead( '/sys/devices/system/cpu/cpu0/cpufreq/cpuinfo_max_freq' ) ) / 1000 ))

   if not bool( asset.SynoInfo ) : asset.SynoInfo = lib.FileConfig( asset.sysSynoConfig )
   sys.avrInfo['NAS_Product']       = asset.SynoInfo[ 'upnpmodelname' ]
   sys.avrInfo['NAS_Type']          = asset.SynoInfo[ 'product'       ]
   sys.avrInfo['NAS_Language']      = asset.SynoInfo[ 'language'      ]
   sys.avrInfo['Mailfrom']          = asset.SynoInfo[ 'mailfrom'      ]

   sys.avrInfo['Hostname']          = io.ioRead( '/etc/hostname' )

   atmp = int( io.ioRead( '/proc/uptime' ).split(' ')[0].split('.')[0] )
   sys.avrInfo['NAS_Uptime']        = "%d:%02d:%02d:%02d" % ( atmp/60/60/24, atmp/60/60%24, atmp/60%60, atmp%60 )

   atmp = int( time.time() ) - int(io.ioRead( asset.avrUpTimeFile ))
   sys.avrInfo['Avr_Uptime']        = "%d:%02d:%02d:%02d" % ( atmp/60/60/24, atmp/60/60%24, atmp/60%60, atmp%60 )

   sTemp = lib.FileConfig( asset.avrInfoFile ) 
   sys.avrInfo['Avr_Created']       = sTemp[ 'created' ]
   sys.avrInfo['Avr_Version']       = sTemp[ 'version' ]

   sys.avrInfo['NAS_Serial']        = io.ioRead( '/proc/sys/kernel/syno_serial' )

   if not 'sys_RamSizeGB' in dir() :
      import Memory
      Memory.init()

   sys.avrInfo['NAS_RAM_GB']        = str( asset.sys_RamSizeGB )

   if 'sys_Cpu' not in dir() :
      import Sensors
      Sensors.Hwmon()
   #  Sensors.CPU_Thermal()
   #  Sensors.Fan_Rpm    ()
   #  Sensors.i2c_Thermal()

   if asset.sys_Cpu : sys.avrInfo['NAS_Thermal']       = str( len( asset.sys_Cpu ) )
   else :             sys.avrInfo['NAS_Thermal']       = "0"

   if asset.sys_Fan : sys.avrInfo['NAS_Fan']           = str( len( asset.sys_Fan ) )
   else :             sys.avrInfo['NAS_Fan']           = "0"

   if asset.sys_i2c : sys.avrInfo['NAS_i2c']           = str( len( asset.sys_i2c ) )
   else :             sys.avrInfo['NAS_i2c']           = "0"

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
def Out_InitStat() :
   asset.OutCnt   = 0
   asset.OutTime  = 0
   asset.OutSgm   = 0
   asset.OutStart = 0
   asset.oid5     = None

def Out_TimeStat( opt ) :
   asset.OutCnt  += 1
   asset.OutLap   = (time.time() - asset.OutStart) * 1000
   asset.OutTime += asset.OutLap
   asset.OutSgm   = asset.OutTime / asset.OutCnt
   lib.Echo ( 2, f'Loop #{opt:2.0f} = {asset.OutSgm:4.1f} ({asset.OutLap:4.1f}) [{asset.oid5:4.1f}]' )

def Out_EchoStat( opt ) :
   asset.OutCnt  += 1
   asset.OutLap   = (time.time() - asset.OutStart) * 1000
   asset.OutTime += asset.OutLap
   asset.OutSgm   = asset.OutTime / asset.OutCnt
   lib.Echo ( 3, f'Loop #{opt:2.0f} = {asset.OutSgm:4.1f} ({asset.OutLap:4.1f}) [{asset.oid5:4.1f}]' )

# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
# sys.exit()
# --------------------------------------------------------------------------
# --------------------------------------------------------------------------
